<?php

/**
 * Rozšíření třídy Zend_Auth_Storage_Session pro možnost nastavení vypršení
 * platnosti úložiště přihlášeného uživatele
 *
 * @package     Custom
 * @author      Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright   Bc. Petr Jajtner, Webbia Standard © 2012
 */
class Custom_Auth_Storage_Session extends Zend_Auth_Storage_Session {

  const CUSTOM_EXPIRATION_TIME = 3600;

  /**
   * Nastaví možnosti relace a zinicializuje objekt session
   *
   * @param mixed $namespace
   * @param mixed $member
   * @return void
   */
  public function __construct($namespace = self::NAMESPACE_DEFAULT, $member = self::MEMBER_DEFAULT) {
    parent::__construct($namespace, $member);

    $expTime = self::getExpirationSeconds();
    $this->setExpirationSeconds($expTime);
  }

  /**
   * Nastaví vypršení platnosti úložiště přihlášeného uživatele po zadaném počtu
   * sekund
   *
   * @param integer $seconds  Vyprší v tolika sekundách
   * @throws Zend_Session_Exception
   * @return void
   */
  public function setExpirationSeconds($seconds) {
    $this->_session->setExpirationSeconds($seconds);
  }

  /**
   * Vrátí počet sekund do expirace přihlášení
   *
   * @return integer
   */
  public static function getExpirationSeconds() {
    $expTime = self::CUSTOM_EXPIRATION_TIME;
    if (Zend_Registry::isRegistered('session')) {
      $session = Zend_Registry::get('session');
      if (isset($session['auth']['expiration_time']) &&
          is_numeric($session['auth']['expiration_time'])) {
        $expTime = (int) $session['auth']['expiration_time'];
      }
    }
    return $expTime;
  }

}
