<?php

/**
 * Obecná třída pro nastavení stránek
 *
 * @package    custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
abstract class Custom_Controller_Action extends Zend_Controller_Action {

  const CUSTOM_ERROR_STACK_INDEX = 200;

  /**
   * Model pro breadcrumbs
   *
   * @var Custom_Breadcrumbs
   */
  protected $_breadcrumbs = null;

  /**
   * Model pro layout
   *
   * @var Zend_Layout
   */
  protected $_layout = null;

  /**
   * Počáteční link domů
   *
   * @var string
   */
  protected $_title = '';

  /**
   * Přihlášení
   *
   * @var Zend_Auth
   */
  protected $_auth = null;


  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    $this->_auth = Zend_Auth::getInstance();
    $this->_layout = Zend_Layout::getMvcInstance();

    $ctrl = $this->getFrontController();
    if (!$ctrl->getParam('noErrorHandler') &&
        !$ctrl->hasPlugin('Custom_Controller_Plugin_ErrorHandler')) {
      require_once 'Custom/Controller/Plugin/ErrorHandler.php';
      $plugin = new Custom_Controller_Plugin_ErrorHandler();
      $ctrl->registerPlugin($plugin, self::CUSTOM_ERROR_STACK_INDEX);
    }

    if ($this->_auth->hasIdentity() && Zend_Registry::isRegistered('session')) {
      $session = Zend_Registry::get('session');
      if (isset($session['auth']['expiration_time']) &&
          is_numeric($session['auth']['expiration_time'])) {
        $expirationTime = (int) $session['auth']['expiration_time'];
        $authSession = new Zend_Session_Namespace($this->_auth->getStorage()->getNamespace());
        $authSession->setExpirationSeconds($expirationTime);
        $this->view->expiration_time = $expirationTime;
      }
    }

    $this->_title = $this->view->translate('System_Common_Breadcrumb_Home');
    $this->_breadcrumbs = new Custom_Breadcrumbs();
  }

  /**
   * Logika po zpracování požadavku
   *
   * @return void
   */
  public function postDispatch() {
    $breadcrumbs = $this->_breadcrumbs->getBreadCrumbs();

    $layout = $this->_layout->getLayout();
    if (($layout != 'front') && is_object($home = $breadcrumbs->findOneBy('label', $this->_title))) {
      $home->setController($layout);
    }

    $this->view->breadcrumbs = $breadcrumbs;
  }

}
