<?php

/**
 * Třída pluginu pro skiny
 *
 * @package    Custom
 * @author     Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Petr Jajtner, Webbia Standard © 2015
 */
class Custom_Controller_Plugin_Theme extends Zend_Controller_Plugin_Abstract {

  /**
   * Objekt bootstrapu
   *
   * @var Zend_Controller_Front
   */
  protected $_bootstrap = null;

  /**
   * Objekt viewRendereru
   *
   * @var Zend_Controller_Action_Helper_ViewRenderer
   */
  protected $_renderer = null;

  /**
   * Objekt View
   *
   * @var Zend_View
   */
  protected $_view = null;

  /**
   * Cesta ke skriptům
   *
   * @var string
   */
  protected $_viewBasePathSpec = ':moduleDir/views';

  /**
   * Url použitého skinu
   *
   * @var string
   */
  protected $_currentUrl = '';

  /**
   * Rutiny před tím, než vstoupí Zend_Controller_Front do smyčky zpracování akcí.
   *
   * @param  Zend_Controller_Request_Abstract $request
   * @return void
   */
  public function dispatchLoopStartup(Zend_Controller_Request_Abstract $request) {
    $this->_bootstrap = Zend_Controller_Front::getInstance()->getParam('bootstrap');
    $this->_view = $this->_bootstrap->bootstrap('View')->getResource('View');
    $this->_renderer = Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
  }

  /**
   * Rutiny po zpracování akce Zend_Controller_Dispatcherem.
   *
   * @param  Zend_Controller_Request_Abstract $request
   * @return void
   */
  public function postDispatch(Zend_Controller_Request_Abstract $request) {
    $layout = $this->_bootstrap->bootstrap('Layout')->getResource('Layout');
    $spec = $this->_renderer->getViewScriptPathSpec();

    if (file_exists($path = $this->_renderer->setViewScriptPathSpec(THEME_PATH."/layouts/{$layout->getLayout()}.:suffix")->getViewScript())) {
      $this->_view->addScriptPath($dir = dirname($path));
      $layout->setLayoutPath($dir);
    }

    $this->_renderer->setViewScriptPathSpec($spec);
    $this->_renderer->setViewBasePathSpec($this->_viewBasePathSpec);
  }

}
