<?php

/**
 * Třída pro rozšíření abstraktní třídy tabulky
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class Custom_Db_Table_Abstract extends Zend_Db_Table_Abstract {

  /**
   * Paměťová cache
   *
   * @var Custom_Cache
   */
  protected $_memoryCache = null;

  /**
   * Inicializace paměťové cache
   *
   * @return void
   */
  protected function _setup() {
    parent::_setup();

    if (Zend_Registry::isRegistered('Custom_Cache')) {
      $this->_memoryCache = Zend_Registry::get('Custom_Cache');
    }
  }

  /**
   * Vypne nebo zapne paměťovou cache
   *
   * @param boolean $flag
   * @return Articles_Model_Nodes
   */
  public function setDisableCache($flag = true) {
    if (is_object($this->_memoryCache)) {
      $this->_memoryCache->setDisabled($flag);
    }
    return $this;
  }

  /**
   * Vrátí TRUE, pokud je cache aktivní, jinak FALSE
   *
   * @return boolean
   */
  public function isCacheDisabled() {
    if (is_object($this->_memoryCache)) {
      return $this->_memoryCache->isDisabled();
    }
    return false;
  }

  /**
   * Vrátí paměťovou cache
   *
   * @return Custom_Cache|null
   */
  public function getMemoryCache() {
    return $this->_memoryCache;
  }

}
