<?php

/**
 * Třída pro práci s rewrity
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class Custom_Db_Table_Rewrite extends Custom_Db_Table_Abstract {

  /**
   * Instance třídy modelu pro rewrity
   *
   * @var System_Model_Rewrites
   */
  protected $_modelRewrites = null;

  /**
   * Název třídy řádku
   *
   * @var string
   */
  protected $_rowClass = 'Custom_Db_Table_Row_Rewrite';

  /**
   * Konstruktor.
   *
   * @param  mixed $config Pole konfiguračních možností specifikovaných uživatelem nebo jen Db adaptér.
   * @return void
   */
  public function __construct($config = array()) {
    $this->_modelRewrites = new System_Model_Rewrites();

    parent::__construct($config);
  }

  /**
   * Vrátí model rewritů
   *
   * @return System_Model_Rewrites
   */
  public function getRewritesModel() {
    return $this->_modelRewrites;
  }

  /**
   * Nastaví referenci na tabulku rewritů
   *
   * @return void
   */
  protected function _addReference() {
    $this->_setupMetadata();

    $columns = $this->_modelRewrites->info(self::PRIMARY);
    $primaryKey = reset($columns);
    $refTableClass = get_class($this->_modelRewrites);
    $onDelete = self::CASCADE;

    $this->addReference($primaryKey, $columns, $refTableClass, $columns, $onDelete);
  }

  /**
   * Pridá sebe do seznamu v modelu rewritů
   *
   * @return void
   */
  protected function _addSelfAsDependent() {
    $rewriteDependent = $this->_modelRewrites->getDependentTables();
    $rewriteDependent += array(get_class($this));
    $this->_modelRewrites->setDependentTables($rewriteDependent);
  }

  /**
   * Vrátí objekt dle parametru $param, případně vrátí mutaci dle $language
   *
   * @param string $param
   * @param string $language
   * @return Custom_Db_Table_Row_Rewrite
   */
  public function get($param, $language = null) {
    $where = array();
    $primary = reset($this->info(self::PRIMARY));
    $rewriteKey = Custom_Db_Table_Row_Rewrite::REWRITE_PRIMARY_KEY;
    $hasLanguage = $language && in_array('language', $this->info(self::COLS));

    if (is_numeric($param)) {
      $condition = $primary.' = ?';
      $where[$condition] = $param;
      
      if ($hasLanguage) {
        $where['language = ?'] = $language;
      }
    } else {
      $condition = $rewriteKey.' = ?';
      $where[$condition] = $param;
    }

    $object = $this->fetchRow($where);
    if ($object && $hasLanguage && $object->language != $language) {
      $object = $this->fetchRow(array(
        $primary.' = ?' => $object->{$primary},
        'language = ?'  => $language
      ));
    }

    return $object;
  }

}
