<?php

/**
 * Třída pro práci se skupinou řádků s rozšířením o filtrování
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class Custom_Db_Table_Rowset_Filter extends Zend_Db_Table_Rowset_Abstract {

  /**
   * Vrátí asociativní pole z hodnot sloupců $keyColumn => $valueColumn
   *
   * @param string $keyColumn  Název sloupce, který bude použit jako klíč
   * @param string $valueColumn  Název sloupce, který bude použit jako hodnota
   * @return array
   */
  public function getOptions($keyColumn, $valueColumn) {
    $result = array();
    foreach ($this as $row) {
      $result[$row->{$keyColumn}] = $row->{$valueColumn};
    }
    return $result;
  }

  /**
   * Metoda pro fitrování řádků po výběru z databáze
   *
   * @param array $filter  asoc. pole ('column' =>, 'value' =>[, 'operator' =>])
   *        string $filter  identifikator =(>=|<=|!=) hodnota
   * @return Custom_Db_Table_Rowset_Filter
   */
  public function filter($filter = null) {
    if (null === $filter) {
      return $this;
    }

    $column = $operator = $value = '';

    if (is_string($filter)) {
      $matches = array();
      if (preg_match('#^(.*?\s*)(=|>=|<=|!=)(\s*.*?)$#i', $filter, $matches)) {
        $filter = array(
          'column'   => trim($matches[1]),
          'operator' => trim($matches[2]),
          'value'    => trim($matches[3])
        );
      } else {
        $filter = $matches;
      }
    }

    if (!isset($filter['column']) ||
        !isset($filter['value'])) {
      return $this;
    }

    if (!isset($filter['operator']) || ('=' == $filter['operator'])) {
      $filter['operator'] = '==';
    }

    extract($filter);

    $rows = $this->_data;
    foreach ($rows as $index => $row) {
      $fn = create_function('', "return {$this->_quote($row[$column])} {$operator} {$this->_quote($value)};");
      if (isset($row[$column]) && !$fn()) {
        unset($rows[$index]);
      }
      unset($fn);
    }

    $data = array(
      'table'    => $this->_table,
      'data'     => array_values($rows),
      'readOnly' => $this->_readOnly,
      'rowClass' => $this->_rowClass,
      'stored'   => true
    );
    $rowsetClass = $this->_table->getRowsetClass();

    return new $rowsetClass($data);
  }

  /**
   * Pokud hodnota není číslem, upraví výraz
   *
   * @param mixed $value
   * @return mixed
   */
  protected function _quote($value) {
    if (null === $value) {
      return 'null';
    }
    if (!is_numeric($value)) {
      return "'".addcslashes($value, "'")."'";
    }
    return $value;
  }

}
