<?php

/**
 * Třída různých nástrojů pro systém Webbia
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class Custom_Tools {

  const REWRITE_MAX_LENGTH = 250;
  const REGEXP_VERSION = '((\d+)(\.\d+)*)(-(alpha|beta|RC)(\d+)?)?';

  const REQUEST_FOLDERS  = 'folders';
  const REQUEST_FILE     = 'file';
  const REQUEST_QUERY    = 'query';
  const REQUEST_FRAGMENT = 'fragment';
  const REQUEST_IS_INDEX = 'is_index';
  const REQUEST_PAGE     = 'page';

  /**
   * Vrátí jedinečná jména všech kontrolerů a akcí
   *
   * @return array
   */
  public static function reservedNames() {
    $names = array('common', 'media', 'theme', 'uploads');

    $dirname = dirname(dirname(__DIR__)).'/system';
    $names[] = strtolower(basename($dirname));
    $dir = $dirname.'/controllers/*Controller.php';

    $files = glob($dir);
    foreach ($files as $filename) {
      $names[] = strtolower(str_replace('Controller.php', '', basename($filename)));
      $class = ucfirst(basename($dirname)).'_'.substr(basename($filename), 0, -4);

      require_once $filename;
      $methods = get_class_methods($class);
      foreach ($methods as $method) {
        if (false !== strpos($method, 'Action')) {
          $names[] = strtolower(str_replace('Action', '', $method));
        }
      }
    }

    return array_unique($names);
  }

  /**
   * Vrátí rewrite z řetězce
   *
   * @param string $string  Řetězec, ze kterého má být rewrite
   * @return string
   */
  public static function getRewrite($string) {
    $a = array('á','à','ä','ć','č','ç','ď','é','è','ë','ě','í','ï','ĺ','ľ','ń','ň','ó','ö','ŕ','ř','ś','š','ť','ú','ù','ů','ü','ý','ÿ','ź','ž','Á','À','Ä','Ć','Č','Ç','Ď','É','È','Ě','Ë','Ï','Í','Ĺ','Ľ','Ń','Ň','Ó','Ö','Ŕ','Ř','Ś','Š','Ť','Ú','Ù','Ů','Ü','Ý','Ÿ','Ź','Ž');
    $b = array('a','a','a','c','c','c','d','e','e','e','e','i','i','l','l','n','n','o','o','r','r','s','s','t','u','u','u','u','y','y','z','z','A','A','A','C','C','C','D','E','E','E','E','I','I','L','L','N','N','O','O','R','R','S','S','T','U','U','U','U','Y','Y','Z','Z');

    $string = strtolower(str_replace($a, $b, $string));
    $string = preg_replace('/[^a-z0-9]+/', '-', $string);
    $string = preg_replace('/-+$/m', '', $string);
    $string = preg_replace('/^-+/m', '', $string);

    if (strlen($string) > self::REWRITE_MAX_LENGTH) {
      $string = preg_replace('/\-+?([^\-]+)?$/', '', substr($string, 0, self::REWRITE_MAX_LENGTH + 3));
    }
    if (is_numeric($string)) {
      $string = '_'.$string;
    }

    return $string;
  }

  /**
   * Generátor hesla
   *
   * @param integer $length  Délka hesla
   * @return string
   */
  public static function generatePassword($length = 8) {
    $validChars = "0123456789_!@#$%&*()-+abcdfghjkmnpqrstvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
    $validLength = strlen($validChars);

    $password = '';
    $counter = 0;

    while ($counter < $length) {
      $actChar = substr($validChars, rand(0, $validLength - 1), 1);
      if (!strstr($password, $actChar)) {
        $password .= $actChar;
        $counter++;
      }
    }

    return $password;
  }

  /**
   * Zašle uživateli email s adresou pro změnu hesla
   *
   * @param System_Model_User $user  Uživatel
   */
  public static function notifyChangePassword($user) {
    $translator = Zend_Registry::get('Zend_Translate');

    $session = new Zend_Session_Namespace('System_UsersController');
    $params = array(
      'code'  => $session->code,
      'email' => $user->email
    );

    $name = $_SERVER['SERVER_NAME'];
    $view = Zend_Registry::get('Zend_View');
    $url = $view->serverUrl().$view->simpleUrl('activate', 'users', null, $params);

    $mail = new Zend_Mail('UTF-8');
    $mail->setFrom('no-reply@'.$name, $translator->translate('System_Tools_Site_CMS').' '.$name);
    $mail->addTo($user->email, $user->getFullName());
    $mail->setSubject($translator->translate('System_Tools_Password_Reset').' '.$name);
    $mail->setBodyHtml(sprintf($translator->translate('System_Tools_Change_Password'), $name, $url), 'UTF-8');
    $mail->send();
  }

  /**
   * Zašle uživateli email s aktivačním klíčem
   *
   * @param System_Model_User $user  Uživatel
   */
  public static function notifyActivationKey($user) {
    $translator = Zend_Registry::get('Zend_Translate');

    $params = array(
      'id'  => $user->id_user,
      'key' => $user->user_password
    );

    $name = $_SERVER['SERVER_NAME'];
    $view = Zend_Registry::get('Zend_View');
    $url = $view->serverUrl().$view->simpleUrl('activate', 'users', 'system', $params);

    $mail = new Zend_Mail('UTF-8');
    $mail->setFrom('no-reply@'.$name, $translator->translate('System_Tools_Site_CMS').' '.$name);
    $mail->addTo($user->email, $user->getFullName());
    $mail->setSubject($translator->translate('System_Tools_Activation_Key').' '.$name);
    $mail->setBodyHtml(sprintf($translator->translate('System_Tools_Activation_By_Key'), $name, $user->email, $url), 'UTF-8');
    $mail->send();
  }

  /**
   * Zašle uživateli email ohledně změny hesla na serveru
   *
   * @param System_Model_User $user  Uživatel
   * @param string $password         Nové heslo
   */
  public static function notifyNewPassword($user, $password) {
    $translator = Zend_Registry::get('Zend_Translate');
    $name = $_SERVER['SERVER_NAME'];

    $mail = new Zend_Mail('UTF-8');
    $mail->setFrom('no-reply@'.$name, $translator->translate('System_Tools_Site_CMS').' '.$name);
    $mail->addTo($user->email, $user->getFullName());
    $mail->setSubject($translator->translate('System_Tools_Server_Pass_Change').' '.$name);
    $mail->setBodyHtml(sprintf($translator->translate('System_Tools_New_Password'), $user->user_name, $password), 'UTF-8');
    $mail->send();
  }

  /**
   * Otestuje modifikaci souboru a odešle příslušné hlavičky
   *
   * @param string $hashID      Hash souboru
   * @param string $lastModify  Datetime poslední změny
   * @param integer $expire     Expirace souboru (nastaveno na 1 den)
   * @return void
   */
  public static function testModify($hashID, $lastModify = null, $expire = 86400) {
    $stampLastModify = (null === $lastModify) ? time() : strtotime($lastModify);

    $notModified = false;
    $hashIDMatch = false;

    $headers = apache_request_headers();
    if (isset($headers['If-Modified-Since']) &&
        (strtotime($headers['If-Modified-Since']) == $stampLastModify)) {
      $notModified = true;
    }

    if (isset($headers['If-None-Match']) &&
        (trim($headers['If-None-Match']) == $hashID)) {
      $hashIDMatch = true;
    }

    if ($notModified || $hashIDMatch) {
      header('HTTP/1.1 304 Not Modified');
      header('Connection: close');
      exit;
    }

    header('ETag: '.$hashID);
    header('Cache-Control: max-age='.$expire);
    header('Expires: '.gmdate('D, d M Y H:i:s', time() + $expire).' GMT');
    header('Last-Modified: '.gmdate('D, d M Y H:i:s', $stampLastModify).' GMT');
  }

  /**
   * Vrátí informaci o $request
   *
   * @param string $request
   * @param string $component
   * @return array
   */
  public static function parseRequest($request, $component = null) {
    $result = array(
      self::REQUEST_FOLDERS  => array(),
      self::REQUEST_FILE     => $request,
      self::REQUEST_QUERY    => array(),
      self::REQUEST_FRAGMENT => '',
      self::REQUEST_IS_INDEX => false,
      self::REQUEST_PAGE     => 1
    );

    $matches = null;
    if (preg_match('/^((.*?\/)*)(.*?)(\?(.*?))?(#(.*?))?$/ui', trim($request), $matches)) {
      $result[self::REQUEST_FOLDERS] = array_values(array_filter(explode('/', $matches[1])));
      if (is_numeric($matches[3]) && ($file = array_pop($result[self::REQUEST_FOLDERS]))) {
        $result[self::REQUEST_PAGE] = (int) $matches[3];
        $result[self::REQUEST_FILE] = $file;
      } else {
        $result[self::REQUEST_FILE] = $matches[3];
      }
      $result[self::REQUEST_IS_INDEX] = ('index' == $result[self::REQUEST_FILE]);

      if (isset($matches[5]) && !empty($matches[5])) {
        parse_str($matches[5], $result[self::REQUEST_QUERY]);
      }
      if (isset($matches[7]) && !empty($matches[7])) {
        $result[self::REQUEST_FRAGMENT] = $matches[7];
      }
    }

    return array_key_exists($component, $result) ? $result[$component] : $result;
  }

  /**
   * Vrátí locale v pořadí dle nastavení SESSION, prohlížeče nebo výchozího locale
   *
   * @param string $default  Výchozí locale
   * @return string
   */
  public static function getCurrentLocale($default = 'cs_CZ') {
    $session = new Zend_Session_Namespace('init-locale');
    if (is_string($strLocale = $session->locale)) {
      return $strLocale;
    }

    $locale = new Zend_Locale();
    $language = $locale->getLanguage();
    if (false === ($region = $locale->getRegion())) {
      $region = Zend_Locale_Data::getContent($locale, 'territorytolanguage', $language);
      if (!is_string($region)) {
        $region = '';
      }
    }

    $languages = System_Model_Settings::getValue('languages', array('cs' => 'čeština'));
    if (array_key_exists($language, $languages) && $region) {
      return "{$language}_{$region}";
    }

    return $default;
  }

  /**
   * Náhrada za readfile
   *
   * @param string $path
   * @return void
   */
  public static function output($path) {
    $output = fopen($path, 'rb');
    while (!feof($output)) {
      echo fread($output, 65536);
      ob_flush();
      flush();
    }
    fclose($output);
  }

  /**
   * Do URL přidá parametry $query
   *
   * @param string $url
   * @param array $query
   * @return string
   */
  public static function buildRequest($url, array $query = array()) {
    $part1 = $url;
    $params = array();
    if (false !== strpos($url, '?')) {
      list($part1, $part2) = explode('?', $url);
      parse_str($part2, $params);
    }
    foreach ($query as $key => $value) {
      $params[$key] = $value;
    }
    return $part1.'?'.http_build_query($params);
  }

}
