<?php

/**
 * Třída helperu pro renderování bloku
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class Custom_View_Helper_Container extends Zend_View_Helper_Abstract {

  /**
   * Výstupní layout
   *
   * @var Zend_Layout
   */
  public $layout;

  /**
   * Helper pro renderování výstupu akce
   *
   * @var Zend_View_Helper_Action
   */
  public $actionHelper;

  /**
   * Cesta pro dočasné přidání do scriptPath
   *
   * @var string|boolean
   */
  private $_addPath = false;

  /**
   * Konstruktor
   *
   * @return void
   */
  public function __construct() {
    require_once 'Zend/View/Helper/Action.php';

    $this->layout = Zend_Layout::getMvcInstance();
    $this->actionHelper = new Zend_View_Helper_Action();
  }

  /**
   * Vyrenderovaný výstup akce bloku
   *
   * @param string $action  Název akce kontroleru
   * @param string $controller  Název kontroleru
   * @param string $module  Název modulu
   * @param array $params  Asociativní pole parametrů akce
   * @return string  Vyrenderovaná akce
   */
  public function container($action, $controller, $module = null, array $params = array()) {
    $currentLayout = $this->layout->getLayout();

    $this->_addPath($module);
    $output = $this->actionHelper->action($action, $controller, $module, $params);
    $this->_removePath();

    $this->layout->setLayout($currentLayout);

    return $output;
  }

  /**
   * Dočasně přiřadí do scriptPath cestu ke skriptům
   *
   * @param string $module  Modul, jehož cesta ke skriptům bude dočasně přidána
   * @return void
   */
  private function _addPath($module = null) {
    $this->_addPath = false;
    if (null !== $module) {
      $this->_addPath = dirname(dirname(dirname(dirname(__DIR__))))."/{$module}/views/scripts/";
    }
    if ($this->_addPath) {
      $this->view->addScriptPath($this->_addPath);
    }
  }

  /**
   * Odebere cestu ze scriptPath
   *
   * @return void
   */
  private function _removePath() {
    if ($this->_addPath) {
      $paths = $this->view->getScriptPaths();
      if (false !== ($key = array_search($this->_addPath, $paths))) {
        unset($paths[$key]);
      }
      $this->view->setScriptPath($paths);
    }
  }

}
