<?php

/**
 * Třída view helperu pro rychlé uchovávání proměnných
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class Custom_View_Helper_Registry extends Zend_View_Helper_Abstract {

  /**
   * Pole prvků
   *
   * @var array
   */
  protected static $_registry = null;

  /**
   * Pokud neexistuje registr, vytvoří jej a vrátí instanci
   *
   * @return Custom_View_Helper_Registry
   */
  public function registry() {
    if (!is_array(self::$_registry)) {
      self::$_registry = array();
    }

    return $this;
  }

  /**
   * Přidá prvek do registru
   *
   * @param string $index  Název identifikátoru
   * @param mixed $value  Libovolná hodnota
   * @return void
   */
  public function __set($name, $value) {
    $this->set($name, $value);
  }

  /**
   * Vrátí prvek z registru
   *
   * @param string $index  Název identifikátoru
   * @return mixed|null
   */
  public function __get($name) {
    return $this->get($name);
  }

  /**
   * Přidá prvek do registru
   *
   * @param string $index  Název identifikátoru
   * @param mixed $value  Libovolná hodnota
   * @return void
   */
  public function set($index, $value) {
    if (array_key_exists($index, self::$_registry) && (null === $value)) {
      unset(self::$_registry[$index]);
    } else {
      self::$_registry[$index] = $value;
    }
  }

  /**
   * Vrátí prvek z registru
   *
   * @param string $index   Název identifikátoru
   * @return mixed|null
   */
  public function get($index) {
    if (array_key_exists($index, self::$_registry)) {
      return self::$_registry[$index];
    }
    return null;
  }

  /**
   * Vrátí celé pole prvků
   *
   * @return array
   */
  public function toArray() {
    return self::$_registry;
  }

  /**
   * Vyprázdní registr
   *
   * @return Custom_View_Helper_Registry
   */
  public function unsetAll() {
    self::$_registry = array();
    return $this;
  }

}
