<?php

/**
 * Třída view helperu pro zkrácení obsahu HTML na daný počet odstavců
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class Custom_View_Helper_Shorten extends Zend_View_Helper_Abstract {

  /**
   * Wrapper pro HTML
   *
   * @var string
   */
  private $_wrapper = "<!DOCTYPE html>
<html xmlns=\"http://www.w3.org/1999/xhtml\">
  <head>
    <meta charset=\"utf-8\" />
  </head>
  <body>
    %s
  </body>
</html>";

  /**
   * Zkrátí obsah HTML na daný počet odstavců
   *
   * @param string $content         Obsah HTML
   * @param mixed $count            Počet odstavců (0 = všechny, výchozí 1); TRUE vrátí počet
   * @param integer $offset         Počet „přeskočených” odstavců (výchozí NULL)
   * @param boolean $preserveAttrs  Zachovat atributy odstavce (výchozí FALSE)
   * @param boolean $addHellip      Doplnit tři tečky (výchozí FALSE)
   * @return string|integer         Zkrácený obsah HTML
   */
  public function shorten($content, $count = 1, $offset = null, $preserveAttrs = false, $addHellip = false) {
    $document = new DOMDocument('1.0', 'utf-8');
    $document->loadHTML($x = sprintf($this->_wrapper, $content));

    $paragraphs = $document->getElementsByTagName('p');
    if (true === $count) {
      return $paragraphs->length;
    }
    if (!$paragraphs->length) {
      return '';
    }

    $p = array();
    foreach ($paragraphs as $paragraph) {
      if (is_numeric($offset) && (--$offset > 0)) {
        continue;
      }
      if ($preserveAttrs) {
        $p[] = $paragraph->ownerDocument->saveXML($paragraph);
      } else {
        $p[] = '<p>'.$paragraph->nodeValue.'</p>';
      }
      if ($count == count($p)) {
        break;
      }
    }

    if ($addHellip && ($paragraphs->length > $count)) {
      $p[] = '<p>&hellip;</p>';
    }

    return implode('', $p);
  }

}
