<?php

/**
 * Třída helperu pro haldu
 *
 * @package    Custom
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class Custom_View_Helper_Stack extends Zend_View_Helper_Abstract {

  /**
   * Pole prvků
   *
   * @var array
   */
  protected static $_stack = null;

  /**
   * Pokud je požadováno, vytvoří novou haldu a vrátí instanci
   *
   * @param boolean $newStack  Vytvořit nový stack?
   * @return Custom_View_Helper_Stack
   */
  public function stack($newStack = false) {
    if ($newStack) {
      self::$_stack = array();
    }
    return $this;
  }

  /**
   * Přidá prvek do haldy
   *
   * @param mixed $item  Prvek
   * @return integer     Velikost pole
   */
  public function push($item) {
    if (is_array(self::$_stack)) {
      return array_push(self::$_stack, $item);
    }
    return 0;
  }

  /**
   * Vrátí poslední prvek z haldy a zkrátí jej
   *
   * @return mixed|null
   */
  public function pop() {
    if (is_array(self::$_stack)) {
      return array_pop(self::$_stack);
    }
    return null;
  }

  /**
   * Nastaví prvky pole do haldy
   *
   * @param array $stack
   * @return Custom_View_Helper_Stack
   */
  public function setStack(array $stack) {
    if (is_array(self::$_stack) && !empty($stack)) {
      array_map(array($this, 'push'), $stack);
    }
    return $this;
  }

  /**
   * Vrátí celout haldu
   *
   * @return array
   */
  public function getStack() {
    if (is_array(self::$_stack)) {
      return self::$_stack;
    }
    return array();
  }

}
