<?php

/**
 * Třída kontroleru pro zobrazení článku
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_ArticleController extends Custom_Controller_Front_Action {

  /**
   * Akce pro zobrazení článku
   */
  public function indexAction() {
    $id_article = $this->_getParam('id');
    $rewrite = $this->_getParam('rewrite');

    if (!is_numeric($id_article) && !is_string($rewrite)) {
      $invalid = $this->view->translate('System_Article_Invalid_Parameter');
      throw new Custom_Exception($invalid, 404);
    }

    $article = null;
    $modelArticles = new System_Model_Articles();
    if (is_string($rewrite)) {
      $article = $modelArticles->getArticleByRewrite($rewrite);
    }
    if (is_numeric($id_article)) {
      $article = $modelArticles->getArticleById($id_article);
    }
    if (!is_object($article)) {
      $notFound = $this->view->translate('System_Article_Not_Found');
      throw new Custom_Exception($notFound, 404);
    }
    if (!$article->isReadable()) {
      $notFound = $this->view->translate('System_Article_Not_Found');
      throw new Custom_Exception($notFound, 404);
    }
    if ($article->redirect) {
      $this->_redirect($article->redirect);
    }

    $this->view->title = $article->caption;
    $this->view->keywords = $article->getKeywords();
    $this->view->description = $article->getDescription();
    $this->view->article = $article;

    $this->_breadcrumbs->addPages($article->getBreadcrumbs());
  }

  /**
   * Akce pro výpis menu
   */
  public function menuAction() {
    $language = Zend_Registry::get('Zend_Locale')->getLanguage();

    $where = array('is_menuitem = ?' => (int) true, 'language = ?' => $language);
    if (!$this->_auth->hasIdentity()) {
      $where['is_visible = ?'] = (int) true;
    }
    $order = 'article_order ASC';

    $modelArticles = new System_Model_Articles();
    $this->view->articles = $modelArticles->fetchAll($where, $order);
    $this->view->kind = $this->getParam('kind', 'major');
  }

}
