<?php

/**
 * Třída kontroleru pro administraci článků
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_ArticlesController extends Custom_Controller_Admin_Action {

  const ARTICLES_MAX_COUNT = 50;
  const ARTICLES_PER_PAGE  = 25;

  /**
   * Model pro články
   *
   * @var System_Model_Articles
   */
  protected $_modelArticles = null;

  /**
   * Model pro session
   *
   * @var Zend_Session_Namespace
   */
  protected $_session = null;

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    parent::init();

    if (!$this->_auth->hasIdentity()) {
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin'));
    }

    $this->view->title = $this->view->translate(__CLASS__);
    $page = array(
      'label'      => $this->view->title,
      'title'      => $this->view->title,
      'controller' => $this->getRequest()->getControllerName(),
      'action'     => 'index'
    );
    $this->_breadcrumbs->addPage($page);
    $this->_modelArticles = new System_Model_Articles();
    $this->_session = new Zend_Session_Namespace(__CLASS__);
  }

  /**
   * Výchozí akce
   */
  public function indexAction() {
    if (!isset($this->_session->article_page) || !$this->_session->article_page) {
      $this->_session->article_page = 1;
    }

    $page = $this->_getParam('page', $this->_session->article_page);
    $count = $this->_modelArticles->getArticlesByFilter(null, null, null, true);
    $limit = self::ARTICLES_PER_PAGE;

    $this->view->paginator = Zend_Paginator::factory($count);
    $this->view->paginator->setItemCountPerPage($limit);
    $this->view->paginator->setCurrentPageNumber($page);

    $this->_session->article_page = $this->view->paginator->getCurrentPageNumber();
    $offset = $limit * ($this->view->paginator->getCurrentPageNumber() - 1);

    $this->view->articles = $this->_modelArticles->getArticlesByFilter(null, $limit, $offset);
    $this->view->messages = $this->getHelper('FlashMessenger')->getMessages();
  }

  /**
   * Akce pro úpravu článku
   */
  public function editAction()  {
    if (!is_numeric($id_article = $this->_getParam('id', $this->_getParam('id_article'))) &&
        (self::ARTICLES_MAX_COUNT <= $this->_modelArticles->getCount())) {
      $message = array('error' => $this->view->translate('System_Articles_Limit_Exceeded', self::ARTICLES_MAX_COUNT));
      $this->getHelper('FlashMessenger')->addMessage($message);
      $this->_redirect($this->getHelper('Url')->simple('index', 'articles'));
    }

    $this->view->form = new System_Form_Article();
    $this->view->form->setAction($this->getHelper('Url')->simple('edit', 'articles'));

    if ($this->getRequest()->isGet()) {
      if (is_string($language = $this->_getParam('language'))) {
        $this->view->form->getElement('language')->setValue($language);
      }
      $this->view->form->getElement('id_article')->setValue($id_article);
    }

    $article = null;
    if ($this->getRequest()->isPost() && $this->view->form->isValid($this->_getAllParams())) {
      $id_article = $this->view->form->getValue('id_article');
      $language = $this->view->form->getValue('language');
      if (!is_object($article = $this->_modelArticles->getArticleById($id_article, $language))) {
        $article = $this->_modelArticles->createRow();
      }
      if ($article->setFromArray($this->view->form->getValues())->save()) {
        if ($this->_getParam('save', false)) {
          $param = array('id' => $article->id_article);
          $this->_redirect($this->getHelper('Url')->simple('edit', 'articles', null, $param));
        }
        if ($this->_getParam('submit', false)) {
          $this->_redirect($this->getHelper('Url')->simple('index', 'articles'));
        }
      }
      $this->view->form->getElement('id_article')->setValue($article->id_article);
    }

    if (is_numeric($id_article = $this->_getParam('id')) &&
        is_object($article = $this->_modelArticles->getArticleById($id_article, $language))) {
      if (is_string($switch = $this->_getParam('switch')) && ('visibility' == $switch)) {
        $article->is_visible = ! (int) $article->is_visible;
        $article->save();
        $this->_redirect($this->getHelper('Url')->simple('index', 'articles'));
      }
      $this->view->form->populate($article->toArray());
    }

    if ($this->view->form->getValue('id_article')) {
      $this->view->title = $this->view->translate('System_Articles_Edit_Article');
    } else {
      $this->view->title = $this->view->translate('System_Articles_New_Article');
    }

    $request = $this->getRequest();
    $page = array(
      'label'      => $this->view->title,
      'title'      => $this->view->title,
      'module'     => $request->getModuleName(),
      'controller' => $request->getControllerName(),
      'action'     => $request->getActionName()
    );
    $this->_breadcrumbs->addPage($page);
  }

  /**
   * Akce pro změnu pořadí článků
   */
  public function moveAction() {
    if (is_array($id_articles = $this->_getParam('ids')) && !empty($id_articles)) {
      $this->_modelArticles->reorder($id_articles);
    }
    if (is_numeric($id_article = $this->_getParam('id')) &&
        in_array($direction = $this->_getParam('direction'), array('up', 'down'))) {
      $this->_modelArticles->move($id_article, $direction);
    }
    $this->_redirect($this->getHelper('Url')->simple('index', 'articles'));
  }


  /**
   * Akce pro smazání článku
   */
  public function deleteAction() {
    if (is_numeric($id_article = $this->_getParam('id')) &&
        is_object($article = $this->_modelArticles->getArticleById($id_article))) {
      $article->delete();
    }
    $this->_redirect($this->getHelper('Url')->simple('index', 'articles'));
  }

}
