<?php

/**
 * Třída kontroleru pro dialogy
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2014
 */
class System_DialogController extends Custom_Controller_Dialog_Action {

  /**
   * Typy dialogů
   *
   * @var array
   */
  protected static $_DIALOG_TYPES = array(
    System_Form_Objects::FORM_FILES,
    System_Form_Objects::FORM_IMAGES,
    System_Form_Objects::FORM_MEDIA
  );

  /**
   * Dialog pro dynamický obsah
   */
  public function dynamicAction() {
    $this->view->headScript()->appendFile('/common/js/tinymce/utils/form_utils.js');
    $this->view->headScript()->appendFile('/common/js/tinymce/utils/validate.js');
    $this->view->headScript()->appendFile('/common/js/tinymce/utils/editable_selects.js');
    $this->view->headScript()->appendFile('/common/js/tinymce/plugins/dynamic/js/dialog.js');

    $this->view->base = array('href' => $this->view->serverUrl(), 'target' => '_self');
    $this->view->title = '{#dynamic_dlg.title}';

    $this->view->galleries = new System_Form_Galleries();
    $this->view->browser = new System_Form_Browser();
    $this->view->contactForm = new System_Form_ContactForm();
  }

  /**
   * Akce pro zobrazení vlastností objektu
   */
  public function propertiesAction() {
    $this->view->base = array('href' => $this->view->serverUrl());

    $modelObjects = new System_Model_Objects();
    if (is_numeric($id_object = $this->_getParam('id'))) {
      if (!is_string($language = $this->_getParam('language')) || empty($language)) {
        $language = null;
      }
      if (is_object($this->view->object = $modelObjects->getObjectById($id_object, $language))) {
        return;
      }
    }
    $this->view->close = true;
  }

  /**
   * Dialog pro procházení souborů - TinyMCE
   */
  public function browseAction() {
    $this->view->base = array('href' => $this->view->serverUrl());
    $this->view->headScript()->appendFile('/common/js/jquery.dialog.js');

    $type = $this->_getParam('type', $default = reset(self::$_DIALOG_TYPES));
    if (!in_array($type, self::$_DIALOG_TYPES)) {
      $type = $default; // files
    }

    $this->view->title = $this->view->translate('System_Common_Open');
    $this->view->articles = ($type == System_Form_Objects::FORM_FILES) ? new System_Form_Articles() : null;
    $this->view->files = new System_Form_Objects(array('type' => $type));

    $this->_processObjects();
  }

  /**
   * Akce pro vrácení názvů objektů v aktuální složce
   */
  public function filesAction() {
    $modelObjects = new System_Model_Objects();
    $rows = $modelObjects->getObjectsByFilter($this->getAllParams());
    $this->getHelper('Json')->sendJson($rows->getCaptions());
  }

  /**
   * Zpracuje data pro souborový dialog
   *
   * @return void
   */
  protected function _processObjects() {
    $modelObjects = new System_Model_Objects();

    $params = $this->getAllParams();
    if (array_key_exists('url', $params) && is_object($this->view->files->getView()->selected = $modelObjects->getObjectByCode($params['url']))) {
      $params['id_parent'] = $this->view->files->getView()->selected->id_parent;
    }

    $this->view->files->setAction($this->getHelper('Url')->simple('browse'));
    $this->view->files->populate($params);

    if ($this->view->files->isValid($params) && is_array($filter = $this->view->files->getValues()) && !empty($filter)) {
      $this->view->files->getView()->current = $modelObjects->getObjectById($filter['id_parent']);
      $this->view->files->getView()->objects = $modelObjects->getObjectsByFilter($filter);
    }
  }

}
