<?php

/**
 * Třída kontroleru pro zobrazení objektu
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_ObjectController extends Custom_Controller_Front_Action {

  /**
   * Akce pro zobrazení objektu
   */
  public function indexAction() {
    $id_object = $this->_getParam('id');
    $rewrite = $this->_getParam('rewrite');

    if (!is_numeric($id_object) && !is_string($rewrite)) {
      $invalid = $this->view->translate('System_Object_Invalid_Parameter');
      throw new Custom_Exception($invalid, 404);
    }

    $object = null;
    $modelObjects = new System_Model_Objects();
    if (is_string($rewrite)) {
      $object = $modelObjects->getObjectByRewrite($rewrite);
    }
    if (is_numeric($id_object)) {
      $object = $modelObjects->getObjectById($id_object);
    }
    if (!is_object($object)) {
      $notFound = $this->view->translate('System_Object_Not_Found');
      throw new Custom_Exception($notFound, 404);
    }
    if (!$object->isReadable()) {
      $notFound = $this->view->translate('System_Object_Not_Found');
      throw new Custom_Exception($notFound, 404);
    }
    if ($object->is(System_Model_Object::SYSTEM_OBJECT_FOLDER)) {
      $this->_forward('index', 'gallery', 'system', array('rewrite' => $object->rewrite));
    }
    if (is_object($file = $object->getFile())) {
      if ($file->isImage() && is_string($size = $this->_getParam('size', false))) {
        $this->_forward('image', 'file', 'system', array('id' => $file->id_file, 'size' => $size));
      } else {
        $this->_forward('download', 'file', 'system', array('id' => $file->id_file));
      }
    }
    return;
  }

}
