<?php

/**
 * Třída kontroleru pro administraci složek a souborů
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_ObjectsController extends Custom_Controller_Admin_Action {

  const FOLDERS_MAX_COUNT = 50;

  /**
   * Model pro objekty
   *
   * @var System_Model_Objects
   */
  protected $_modelObjects = null;

  /**
   * Model pro session
   *
   * @var Zend_Session_Namespace
   */
  protected $_session = null;

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    parent::init();

    if (!$this->_auth->hasIdentity()) {
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin'));
    }

    $this->view->title = $this->view->translate(__CLASS__);
    $page = array(
      'label'      => $this->view->title,
      'title'      => $this->view->title,
      'controller' => $this->getRequest()->getControllerName(),
      'action'     => 'index'
    );
    $this->_breadcrumbs->addPage($page);
    $this->_modelObjects = new System_Model_Objects();
    $this->_session = new Zend_Session_Namespace(__CLASS__);
  }

  /**
   * Akce pro přehled objektů
   */
  public function indexAction() {
    $id_object = (int) $this->_getParam('id', 0);

    if (!is_array($this->_session->pages) || !isset($this->_session->pages[$id_object])) {
      $this->_session->pages[$id_object] = 1;
    }
    if (!isset($this->_session->style)) {
      $this->_session->style = reset(System_Model_Objects::$STYLES);
    }
    if (is_string($style = $this->_getParam('view')) && in_array($style, System_Model_Objects::$STYLES)) {
      $this->_session->style = $style;
    }

    $filter = array('id_parent' => (0 < $id_object) ? $id_object : null);

    $page = (int) $this->_getParam('page', $this->_session->pages[$id_object]);
    $count = $this->_modelObjects->getObjectsByFilter($filter, null, null, true);
    $limit = System_Model_Objects::$PER_PAGE[$this->_session->style];

    $this->view->paginator = Zend_Paginator::factory($count);
    $this->view->paginator->setItemCountPerPage($limit);
    $this->view->paginator->setCurrentPageNumber($page);

    $this->_session->pages[$id_object] = $this->view->paginator->getCurrentPageNumber();
    $offset = $limit * ($this->view->paginator->getCurrentPageNumber() - 1);

    $this->view->style = $this->_session->style;
    $this->view->objects = $this->_modelObjects->getObjectsByFilter($filter, $limit, $offset);
    $this->view->folders = $this->_modelObjects->getFolders();
    $this->view->messages = $this->getHelper('FlashMessenger')->getMessages();

    $this->view->object = is_numeric($filter['id_parent']) ? $this->_modelObjects->getObjectById($filter['id_parent']) : null;
    if (is_object($this->view->object)) {
      $this->_breadcrumbs->addPages($this->view->object->getBreadcrumbs());
    }
  }

  /**
   * Akce pro úpravu objektu
   */
  public function editAction() {
    $language = $this->_getParam('language');

    $existing = null;
    $type = $this->_getParam('object_type', $this->_getParam('type'));
    $types = array_keys(System_Model_Object::$OBJECT_TYPES);

    $this->view->object = $id_parent = null;
    if (0 < ($id_object = (int) $this->_getParam('id'))) {
      $this->view->object = $this->_modelObjects->getObjectById($id_object, $language);
      if ((null === $this->view->object) && is_object($object = $this->_modelObjects->getMutationById($id_object))) {
        $type = $object->object_type;
        $id_parent = $object->id_parent;
        $existing = $object->getCode();
        $object = null;
      }
    }

    if (is_object($this->view->object)) {
      $type = $this->view->object->object_type;
    }

    if ((null === $type) || (is_string($type) && !in_array($type, $types))) {
      $message = $this->view->translate('System_Objects_Invalid_Type');
      $this->getHelper('FlashMessenger')->addMessage(array('error' => $message));
      $this->_redirect($this->getHelper('Url')->simple('index', 'objects'));
    }

    if (System_Model_Object::SYSTEM_OBJECT_FOLDER == $type) {
      $count = $this->_modelObjects->getFolders(null, null, true);
      if (!$id_object && (self::FOLDERS_MAX_COUNT <= $count)) {
        $message = $this->view->translate('System_Objects_Limit_N_Exhausted', self::FOLDERS_MAX_COUNT);
        $this->getHelper('FlashMessenger')->addMessage(array('error' => $message));
        $this->_redirect($this->getHelper('Url')->simple('index', 'objects'));
      }
      if (!$id_object && ($id_parent = (int) $this->_getParam('parent'))) {
        $message = $this->view->translate('System_Objects_Nested_Folders_Not_Allowed');
        $this->getHelper('FlashMessenger')->addMessage(array('error' => $message));
        $this->_redirect($this->getHelper('Url')->simple('index', 'objects', null, array('id' => $id_parent)));
      }
    }

    $this->view->form = $form = System_Form_Object::factory($type);
    $this->view->form->setAction($this->getHelper('Url')->simple('edit', 'objects'));

    if ($this->_request->isPost() && $this->view->form->process($this->_getAllParams(), $this->view->object)) {
      if ($this->_getParam('save', false)) {
        $param = array('id' => $this->view->object->id_object, 'language' => $this->_getParam('mutation'));
        $this->_redirect($this->getHelper('Url')->simple('edit', 'objects', null, $param));
      }
      if ($this->_getParam('submit', false)) {
        $param = is_numeric($this->view->object->id_parent) ? array('id' => $this->view->object->id_parent) : array();
        $this->_redirect($this->getHelper('Url')->simple('index', 'objects', null, $param));
      }
    }

    if ($this->_request->isGet()) {
      $data = array();
      $isObject = is_object($this->view->object);
      if ($isObject && ('visibility' == $this->_getParam('switch'))) {
        $this->_changeState($this->view->object);
      }
      if ($isObject) {
        $data = $this->view->object->toArray();
      }
      if (is_numeric($id_parent = $this->_getParam('parent', $id_parent))) {
        $data['id_parent'] = $id_parent;
      }
      if (is_numeric($id_object = $this->_getParam('id')) &&
          is_string($language = $this->_getParam('language'))) {
        $data['id_object'] = $id_object;
        $data['language'] = $language;
      }
      if (is_string($existing)) {
        $data['existing'] = $existing;
      }
      if (!empty($data)) {
        $this->view->form->populate($data);
      }
    }

    if ($this->view->form->getValue('id_object')) {
      $this->view->title = $this->view->translate($form::EDIT_OBJECT_TEXT);
    } else {
      $this->view->title = $this->view->translate($form::NEW_OBJECT_TEXT);
    }

    $page = array(
      'label'      => $this->view->title,
      'title'      => $this->view->title,
      'controller' => $this->getRequest()->getControllerName(),
      'action'     => $this->getRequest()->getActionName()
    );
    $this->_breadcrumbs->addPage($page);
  }

  /**
   * Akce pro přesun objektu v seznamu
   */
  public function moveAction() {
    $param = null;
    if (is_array($id_objects = $this->_getParam('ids')) && !empty($id_objects)) {
      $this->_modelObjects->reorder($id_objects);
      if (is_object($object = $this->_modelObjects->getObjectById(reset($id_objects))) &&
          is_object($parent = $object->getParent())) {
        $param = array('id' => $parent->id_object);
      }
    }
    if (is_numeric($id_object = $this->_getParam('id')) &&
        in_array($direction = $this->_getParam('direction'), array('up', 'down'))) {
      $this->_modelObjects->move($id_object, $direction);
      if (is_object($object = $this->_modelObjects->getObjectById($id_object)) &&
          is_object($parent = $object->getParent())) {
        $param = array('id' => $parent->id_object);
      }
    }
    $this->_redirect($this->getHelper('Url')->simple('index', 'objects', null, $param));
  }

  /**
   * Akce pro smazání objektu
   */
  public function deleteAction() {
    $param = array();
    if (is_numeric($id_object = $this->_getParam('id')) &&
        is_string($language = $this->_getParam('language')) &&
        is_object($object = $this->_modelObjects->getObjectById($id_object, $language))) {
      if (is_object($parent = $object->getParent())) {
        $param = array('id' => $parent->id_object);
      }
      $object->delete();
    }
    $this->_redirect($this->getHelper('Url')->simple('index', 'objects', null, $param));
  }

  /**
   * Nahraje soubory na server
   */
  public function uploadAction() {
    set_time_limit(0);
    clearstatcache();

    $fileName = urldecode($this->getRequest()->getServer('HTTP_X_FILE_NAME', uniqid()));
    $fileSize = (int) $this->getRequest()->getServer('HTTP_X_FILE_SIZE');
    $language = Zend_Registry::get('Zend_Locale')->getLanguage();

    $uploadDir = System_Model_File::getUploadDir();
    $filePath = $uploadDir.'/'.$fileName;

    $input = fopen('php://input', 'r');
    $tmpFile = fopen($filePath, 'w');
    while ($data = fread($input, 1024)) {
      fwrite($tmpFile, $data);
    }
    fclose($tmpFile);
    fclose($input);

    chmod($filePath, 0666);

    if ($fileSize != filesize($filePath)) {
      unlink($filePath);

      $this->getResponse()->setRawHeader('HTTP/1.1 409 Conflict');
      $this->getHelper('Json')->sendJson(array(
        'data'   => 'Velikost dat se neshoduje',
        'status' => 'Chyba'
      ));
    }

    $file = System_Model_File::saveFile($filePath);
    $fileInfo = array(
      'id_parent'    => $this->_getParam('parent'),
      'id_file'      => $file->id_file,
      'language'     => $this->_getParam('language', $language),
      'caption'      => $file->getNameOnly(),
      'object_type'  => System_Model_Object::SYSTEM_OBJECT_FILE,
      'is_visible'   => (int) true
    );

    $object = $this->_modelObjects->createRow();
    $object->setFromArray($fileInfo)->save();

    $this->getHelper('Json')->sendJson(array(
      'data'   => 'Úspěšně nahráno',
      'status' => 'OK'
    ));
  }

  /**
   * Akce pro výběr souboru v dialogovém okně
   */
  public function browseAction() {
    $object = null;
    $params = $this->_getAllParams();

    if (!isset($params['folder']) || !is_numeric($params['folder'])) {
      $params['folder'] = null;
    }
    if (!isset($params['type']) || !is_string($params['type'])) {
      $params['type'] = 'file';
    }
    if (isset($params['url']) && !empty($params['url'])) {
      $url = reset(explode('#', trim($params['url'], '/')));
      $parts = explode('?', $url);
      $rewrites = explode('/', reset($parts));
      parse_str(end($parts), $query);

      if (in_array('original', $query)) {
        $params['is_original'] = (int) true;
      }
      if (in_array('thumb', $query)) {
        $params['is_thumb'] = (int) true;
      }

      if (is_numeric($rewrite = end($rewrites))) {
        $object = $this->_modelObjects->getObjectById($rewrite);
      } elseif (is_string($rewrite)) {
        $object = $this->_modelObjects->getObjectByRewrite($rewrite);
      }

      if (is_object($object) && !is_numeric($rewrite)) {
        if ($object->is(System_Model_Object::SYSTEM_OBJECT_FOLDER)) {
          $params['folder'] = $object->id_object;
        } else {
          $params['folder'] = is_object($parent = $object->getParent()) ? $parent->id_object : null;
        }
      }
    }

    $options = System_Model_FileType::getOptions($params['type']);
    if (!isset($params['filetype']) || !is_string($params['filetype'])) {
      $keys = array_keys($options);
      $params['filetype'] = reset($keys);
    }

    $formBrowser = new System_Form_Browse();
    $formBrowser->setAction($this->getHelper('Url')->simple('browse', 'objects'));
    $formBrowser->getElement('filetype')->setMultiOptions($options);
    if ('image' != $params['type']) {
      $formBrowser->removeElement('is_original');
      $formBrowser->removeElement('is_thumb');
    }
    $formBrowser->populate($params);

    if ($formBrowser->isValid($params)) {
      $filter = $formBrowser->getValues();
      $this->view->registry()->selected = $object;
      $this->view->registry()->current = $this->_modelObjects->getObjectById($filter['folder']);
      $this->view->registry()->objects = $this->_modelObjects->getObjectsByFilter($filter);
    }
    $this->view->form = $formBrowser;

    $this->_layout->disableLayout();
  }

  /**
   * Akce pro změnu stavu objektů
   */
  public function visibilityAction() {
    if (is_array($id_nodes = $this->_getParam('id')) && !empty($id_nodes) &&
        in_array($state = $this->_getParam('state'), array(0, 1))) {
      $this->_changeStates($id_nodes, $state);
    }
    $this->_redirect($_SERVER['HTTP_REFERER']);
  }

  /**
   * Změní stav objektu
   *
   * @param System_Model_Object $object    Měněný objekt
   * @param boolean|integer     $state     Hodnota stavu (výchozí NULL)
   * @param boolean             $redirect  Přesměrovat po změně stavu
   * @param integer             $id_parent Export hodnoty ID rodiče
   * @return boolean                       Výsledek operace
   */
  private function _changeState(System_Model_Object $object, $state = null, $redirect = true, &$id_parent = null) {
    $param = is_numeric($object->id_parent) ? array('id' => $object->id_parent) : null;

    $object->is_visible = is_numeric($state) ? (int) $state : !((int) $object->is_visible);
    $saved = $object->save();

    if ($saved && $redirect) {
      $this->_redirect($this->getHelper('Url')->simple('index', 'objects', null, $param));
    }

    return $saved;
  }

  /**
   * Změní stavy objektů
   *
   * @param array $id_nodes  Identifikátory měněných objektů
   * @param integer $state   Hodnota stavu (výchozí NULL)
   * @return void
   */
  private function _changeStates(array $id_nodes, $state = null) {
    $saved = true;

    $objects = $this->_modelObjects->fetchAll(array('id_node IN (?)' => $id_nodes));
    foreach ($objects as $object) {
      $saved = $saved && $this->_changeState($object, $state, false, $id_parent);
    }

    if ($saved) {
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'files', array('node' => $id_parent)));
    }
  }

}
