<?php

/**
 * Třída kontroleru pro registraci uživatele
 *
 * @category   diploma-thesis
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2016
 */
class System_RegisterController extends Custom_Controller_Front_Action {

  /**
   * Model pro uživatele
   *
   * @var System_Model_Users
   */
  protected $_modelUsers = null;

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    $this->_modelUsers = new System_Model_Users();
  }

  /**
   * Výchozí akce registrace
   */
  public function indexAction() {
    $this->view->form = new System_Form_Register();
    $this->view->form->setAction($this->getHelper('Url')->simple('index'));

    if ($this->_request->isPost() && $this->view->form->isValid($this->_getAllParams())) {
      $values = $this->view->form->getValues();
      $values['user_password'] = Custom_Tools::generatePassword();

      $modelRoles = new System_Model_Roles();
      $values['id_role'] = $modelRoles->getRoleByName('registered')->id_role;

      $user = $this->_modelUsers->createRow();
      if ($user->setFromArray($values)->save()) {
        Custom_Tools::notifyActivationKey($user);

        $message = $this->view->translate('System_Successfully_Registered');
        $this->getHelper('FlashMessenger')->addMessage(array('info' => $message));

        $this->_redirect($this->getHelper('Url')->simple('done'));
      }
    }
  }

  /**
   * Akce pro zobrazení informace o registraci
   */
  public function doneAction() {
    $messages = $this->getHelper('FlashMessenger')->getMessages();
    if (!is_array($messages) || empty($messages)) {
      $this->_redirect($this->getHelper('Url')->simple('index'));
    }
    $this->view->messages = $messages;
  }

}
