<?php

/**
 * Třída kontroleru pro uživatele
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class System_UsersController extends Custom_Controller_Admin_Action {

  const USERS_PER_PAGE = 20;

  /**
   * Model pro uživatele
   *
   * @var System_Model_Users
   */
  protected $_modelUsers = null;

  /**
   * Model pro session
   *
   * @var Zend_Session_Namespace
   */
  protected $_session = null;

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    parent::init();

    $this->view->title = $this->view->translate(__CLASS__);
    $page = array(
      'label'      => $this->view->title,
      'title'      => $this->view->title,
      'module'     => $this->getRequest()->getModuleName(),
      'controller' => $this->getRequest()->getControllerName(),
      'action'     => 'index'
    );
    $this->_breadcrumbs->addPage($page);
    $this->_modelUsers = new System_Model_Users();
    $this->_session = new Zend_Session_Namespace(__CLASS__);
  }

  /**
   * Výchozí akce (přehled)
   */
  public function indexAction() {
    if (!$this->_auth->hasIdentity()) {
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin'));
    }

    if (null === $this->_session->page) {
      $this->_session->page = 1;
    }

    $page = $this->_getParam('page', $this->_session->page);
    $count = (int) $this->_modelUsers->getAllUsers(null, null, true);
    $limit = self::USERS_PER_PAGE;

    $this->view->paginator = Zend_Paginator::factory($count);
    $this->view->paginator->setItemCountPerPage($limit);
    $this->view->paginator->setCurrentPageNumber($page);

    $this->_session->page = $this->view->paginator->getCurrentPageNumber();
    $offset = $limit * ($this->view->paginator->getCurrentPageNumber() - 1);
    $this->view->users = $this->_modelUsers->getAllUsers($limit, $offset);

    $this->view->messages = $this->getHelper('FlashMessenger')->getMessages();
  }

  /**
   * Akce pro úpravu uživatele
   *
   * @throws Custom_Acl_Exception
   */
  public function editAction() {
    if (!$this->_auth->hasIdentity()) {
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin'));
    }

    if (is_numeric($active = $this->_getParam('active', false))) {
      $ids = $this->_getParam('id', array());
      if (!empty($ids)) {
        $this->_modelUsers->setUsersState((array) $ids, $active);
      }
      $this->_redirect($this->getHelper('Url')->simple('index'));
    }

    $this->view->form = new System_Form_User();
    $this->view->form->setAction($this->getHelper('Url')->simple('edit'));

    if ($this->_request->isPost() && $this->view->form->isValid($this->_getAllParams())) {
      $id_user = $this->view->form->getValue('id_user');
      if (!is_object($user = $this->_modelUsers->getUserById((int) $id_user))) {
        $user = $this->_modelUsers->createRow();
      }
      if ($user->setFromArray($this->view->form->getValues())->save()) {
        $this->_redirect($this->getHelper('Url')->simple('index'));
      }
    }

    if (is_numeric($id_user = $this->_getParam('id')) &&
        is_object($user = $this->_modelUsers->getUserById((int) $id_user))) {
      $this->view->form->populate($user->toArray());
    }

    if ($this->view->form->getValue('id_user')) {
      $this->view->title = $this->view->translate('System_Users_Account_Editing');
    } else {
      $this->view->title = $this->view->translate('System_Users_New_Account');
    }

    $page = array(
      'label'      => $this->view->title,
      'title'      => $this->view->title,
      'module'     => $this->getRequest()->getModuleName(),
      'controller' => $this->getRequest()->getControllerName(),
      'action'     => $this->getRequest()->getActionName()
    );
    $this->_breadcrumbs->addPage($page);
  }

  /**
   * „Smazání” uživatele
   *
   * @throws Custom_Acl_Exception
   */
  public function deleteAction() {
    if (!$this->_auth->hasIdentity()) {
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin'));
    }

    if (is_numeric($id_user = $this->_getParam('id')) &&
        is_object($user = $this->_modelUsers->getUserById((int) $id_user))) {
      $user->delete();
    }

    if ($id_user == $this->_auth->getIdentity()->id_user) {
      System_Model_Log::out();
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
    }

    $this->_redirect($this->getHelper('Url')->simple('index', 'users'));
  }

  /**
   * Změna hesla
   */
  public function passwordAction() {
    $this->view->form = new System_Form_Password();
    $this->view->form->setAction($this->getHelper('Url')->simple('password'));

    if ($this->_request->isPost() && $this->view->form->isValid($this->_getAllParams())) {
      $user_name = $this->_auth->getIdentity()->user_name;
      $user_pass = $this->view->form->getValue('user_password');

      if (is_object($user = $this->_modelUsers->userExists($user_name, $user_pass))) {
        $new_pass = $this->view->form->getValue('newpass1');
        $user->user_password = System_Model_User::EncryptPassword($new_pass);
        if ($user->save()) {
          $this->_auth->getStorage()->write($user->hidePassword());

          $message = array('info' => $this->view->translate('System_Users_Password_Been_Changed'));
          $this->getHelper('FlashMessenger')->addMessage($message);
          $this->_redirect($this->getHelper('Url')->simple('index', 'users'));
        }
      }
    }

    $this->view->title = $this->view->translate('System_Users_Password_Change');

    $page = array(
      'label'      => $this->view->title,
      'title'      => $this->view->title,
      'module'     => $this->getRequest()->getModuleName(),
      'controller' => $this->getRequest()->getControllerName(),
      'action'     => $this->getRequest()->getActionName()
    );
    $this->_breadcrumbs->addPage($page);
  }

  /**
   * Ověření uživatele
   */
  public function authenticateAction() {
    $module = $this->_getParam('mod', 'system');
    $controller = $this->_getParam('con', 'none');
    $action = $this->_getParam('act', 'none');
    $id = $this->_getParam('id', 0);

    $this->view->form = new System_Form_Authenticate();
    $this->view->form->setAction($this->getHelper('Url')->simple('authenticate'));
    if ($this->_request->isPost() && $this->view->form->isValid($this->getAllParams())) {
      $this->_redirect($this->getHelper('Url')->simple($action, $controller, $module, array('id' => $id)));
    }

    $this->view->form->populate($this->getAllParams());
    $this->view->title = $this->view->translate('System_Users_User_Authentication');
  }

  /**
   * Aktivace uživatele
   */
  public function activateAction() {
    if (is_numeric($id_user = $this->_getParam('id')) &&
        is_object($user = $this->_modelUsers->getUserById($id_user))) {
      if (is_string($hash = $this->_getParam('key')) &&
          ($hash === $user->user_password)) {
        $user->setActive();
        $this->_forward('change');
      } else {
        $message = array('error' => $this->view->translate('System_Users_Incorrect_Key'));
        $this->getHelper('FlashMessenger')->addMessage($message);
        $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
      }
    }
    if (is_string($email = $this->_getParam('email')) &&
        is_object($user = $this->_modelUsers->getUserByEmail($email))) {
      if (is_string($code = $this->_getParam('code')) &&
          ($code === $this->_session->code)) {
        $user->setActive();
        $this->_request->setParam('id', $user->id_user);
        $this->_forward('change');
      } else {
        $message = array('error' => $this->view->translate('System_Users_Incorrect_Code'));
        $this->getHelper('FlashMessenger')->addMessage($message);
        $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
      }
    }
    if (!is_object($user)) {
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
    }
  }

  /**
   * Akce pro změnu hesla uživateli systémem
   */
  public function changeAction() {
    $type = 'error';

    if (!is_numeric($id_user = $this->_getParam('id'))) {
      $message = $this->view->translate('System_Users_Invalid_ID');
      $this->getHelper('FlashMessenger')->addMessage(array($type => $message));
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
    }

    if (!is_object($user = $this->_modelUsers->getUserById($id_user))) {
      $message = $this->view->translate('System_Users_User_Not_Found');
      $this->getHelper('FlashMessenger')->addMessage(array($type => $message));
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
    }

    $new_password = Custom_Tools::generatePassword();
    $user->user_password = System_Model_User::EncryptPassword($new_password);

    if (!$user->save()) {
      $message = $this->view->translate('System_Users_Change_Failed');
      $this->getHelper('FlashMessenger')->addMessage(array($type => $message));
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
    }

    try {
      Custom_Tools::notifyNewPassword($user, $new_password);
    } catch (Exception $e) {
      $this->getHelper('FlashMessenger')->addMessage(array($type => $e->getMessage()));
      $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
    }

    $type = 'info';
    $message = $this->view->translate('System_Users_Change_Succeed');
    $this->getHelper('FlashMessenger')->addMessage(array($type => $message));

    $this->_redirect($this->getHelper('Url')->simple('index', $this->_auth->hasIdentity() ? 'users' : 'admin'));
  }

  /**
   * Obnovení přístupu pro uživatele se zapomenutým heslem
   */
  public function renewAction() {
    $this->view->form = new System_Form_Renew();
    $this->view->form->setAction($this->getHelper('Url')->simple('renew'));

    if ($this->_request->isPost() && $this->view->form->isValid($this->getAllParams())) {
      if (is_object($user = $this->_modelUsers->getUserByEmail($this->view->form->getValue('email')))) {
        $this->_session->code = Custom_Tools::generatePassword(6);

        try {
          Custom_Tools::notifyChangePassword($user);
        } catch (Exception $e) {
          $this->getHelper('FlashMessenger')->addMessage(array('error' => $e->getMessage()));
          $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
        }

        $message = array('info' => $this->view->translate('System_Users_Next_Steps_Sent'));
        $this->getHelper('FlashMessenger')->addMessage($message);
        $this->_redirect($this->getHelper('Url')->simple('index', 'admin', 'system'));
      } else {
        $message = array('invalidEmail' => 'System_Users_Email_Not_Found');
        $this->view->form->getElement('email')->setErrors($message);
      }
    }
  }

}
