<?php

/**
 * Třída abstraktní třídy formuláře pro objekty správce souborů
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
abstract class System_Form_Abstract extends Custom_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array();

  /**
   * Model pro objekty
   *
   * @var System_Model_Objects
   */
  protected $_modelObjects = null;

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $this->setMethod('post');

    $languageOptions = System_Model_Texts::getLanguages();
    $languageKeys = array_keys($languageOptions);

    // - data -
    $language = $this->createElement('select', 'language');
    $language->setDisableTranslator(true);
    $language->setRequired(true);
    $language->setMultiOptions($languageOptions);
    $language->setValidators(array(
      'NotEmpty',
      array('InArray', true, array('haystack' => $languageKeys))
    ));
    $language->setValue(Zend_Registry::get('Zend_Locale')->getLanguage());
    $this->addElement($language);

    $caption = $this->createElement('text', 'caption');
    $caption->setFilters(array('StripTags', 'StringTrim'));
    $caption->setValidators(array(
      'NotEmpty',
      array('StringLength', true, array(3, 256))
    ));
    $this->addElement($caption);

    $description = $this->createElement('textarea', 'description');
    $description->setRequired(false);
    $description->setAttribs(array(
      'cols'  => 80,
      'rows'  => 25
    ));
    $description->setFilters(array('StringTrim', 'Null'));
    $this->addElement($description);

    $is_visible = $this->createElement('checkbox', 'is_visible');
    $is_visible->setFilters(array('Int'));
    $is_visible->setValue((int) true);
    $this->addElement($is_visible);

    // - vlastnosti -
    $rewrite = $this->createElement('text', 'rewrite');
    $rewrite->setRequired(false);
    $this->addElement($rewrite);

    // - skryté -
    $id_object = $this->createElement('hidden', 'id_object');
    $id_object->setDecorators(array('ViewHelper'));
    $this->addElement($id_object);

    $object_type = $this->createElement('hidden', 'object_type');
    $object_type->setDecorators(array('ViewHelper'));
    $this->addElement($object_type);

    // - tlačítka -
    $submit = $this->createElement('submit', 'submit');
    $submit->setLabel('System_Common_Save');
    $submit->setDecorators(array('ViewHelper'));
    $this->addElement($submit);

    $save = $this->createElement('submit', 'save');
    $save->setLabel('System_Common_Save_Continue');
    $save->setDecorators(array('ViewHelper'));
    $this->addElement($save);

    $cancel = $this->createElement('button', 'cancel');
    $cancel->setLabel('System_Common_Cancel');
    $cancel->setDecorators(array('ViewHelper'));
    $this->addElement($cancel);

    $elements = array('id_object', 'object_type', 'submit', 'save', 'cancel');
    $this->addDisplayGroup($elements, 'submits');
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Abstract
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    $group = $this->getDisplayGroup('submits');
    $group->removeDecorator('Fieldset');
    $group->removeDecorator('HtmlTag');
    $group->getDecorator('DtDdWrapper')->setOption('dtLabel', '');

    return $this->setDecorators($this->_formDecorator);
  }

  /**
   * Metoda pro zpracování dat formuláře.
   *
   * Metoda ověří data na platnost a provede určitou logiku nad daty
   *
   * @param array $values  Data formuláře
   * @param System_Model_Object $object  Vytvořený objekt
   * @return boolean  Data ověřena a objekt uložen
   */
  abstract public function process(array $values, System_Model_Object &$object = null);

}
