<?php

/**
 * Třída formuláře pro úpravu článku
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Form_Article extends Custom_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/article.phtml')),
    'Form'
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $this->setMethod('post');
    $this->setAttrib('id', 'article-form');

    $languageOptions = System_Model_Texts::getLanguages();
    $languageKeys = array_keys($languageOptions);

    // - data -
    $language = $this->createElement('select', 'language');
    $language->setDisableTranslator(true);
    $language->setLabel($this->_view->translate('System_Form_Article_Language'));
    $language->setRequired(true);
    $language->setMultiOptions($languageOptions);
    $language->setValidators(array(
      'NotEmpty',
      array('InArray', true, array('haystack' => $languageKeys))
    ));
    $language->setValue(Zend_Registry::get('Zend_Locale')->getLanguage());
    $this->addElement($language);

    $caption = $this->createElement('text', 'caption');
    $caption->setLabel('System_Form_Article_Caption');
    $caption->setRequired(true);
    $caption->setFilters(array('StripTags', 'StringTrim'));
    $caption->setValidators(array(
      'NotEmpty',
      array('StringLength', true, array(3, 256))
    ));
    $this->addElement($caption);

    $pereximg = $this->createElement('text', 'pereximg');
    $pereximg->setLabel('System_Form_Article_Perex_Image');
    $pereximg->setRequired(false);
    $pereximg->setFilters(array('StripTags', 'StringTrim', 'Null'));
    $pereximg->setDecorators(array('Label', 'ViewHelper', 'Errors'));
    $this->addElement($pereximg);

    $perexbtn = $this->createElement('button', 'perexbtn');
    $perexbtn->setDisableTranslator(true);
    $perexbtn->setLabel('...');
    $perexbtn->setDecorators(array('ViewHelper'));
    $this->addElement($perexbtn);

    $content = $this->createElement('textarea', 'content');
    $content->setLabel('System_Form_Article_Content');
    $content->setRequired(false);
    $content->setAttribs(array(
      'cols'  => 80,
      'rows'  => 25
    ));
    $content->setFilters(array('StringTrim', 'Null'));
    $this->addElement($content);

    foreach (System_Model_Article::$ATTR_OPTIONS as $attrKey => $attrTitle) {
      $title = $this->_view->translate($attrTitle).':';
      $$attrKey = $this->createElement('checkbox', $attrKey);
      $$attrKey->setDisableTranslator(true);
      $$attrKey->setLabel($title);
      $$attrKey->setFilters(array('Int'));
      $$attrKey->setValue((int) true);
      $this->addElement($$attrKey);
    }

    $id_article = $this->createElement('hidden', 'id_article');
    $id_article->setDecorators(array('ViewHelper'));
    $this->addElement($id_article);

    $submit = $this->createElement('submit', 'submit');
    $submit->setLabel('System_Common_Save');
    $submit->setDecorators(array('ViewHelper'));
    $this->addElement($submit);

    $save = $this->createElement('submit', 'save');
    $save->setLabel('System_Common_Save_Continue');
    $save->setDecorators(array('ViewHelper'));
    $this->addElement($save);

    $cancel = $this->createElement('button', 'cancel');
    $cancel->setLabel('System_Common_Cancel');
    $cancel->setDecorators(array('ViewHelper'));
    $this->addElement($cancel);

    $this->addDisplayGroup(array('id_article', 'submit', 'save', 'cancel'), 'submits');

    foreach (System_Model_Article::$PART_OPTIONS as $partKey => $partTitle) {
      $$partKey = $this->createElement('checkbox', $partKey);
      $$partKey->setLabel($partTitle);
      $$partKey->setFilters(array('Int'));
      $$partKey->setValue((int) System_Model_Article::$PART_DEFAULTS[$partKey]);
      $this->addElement($$partKey);
    }

    $keywords = $this->createElement('text', 'keywords');
    $keywords->setLabel('System_Form_Article_Keywords');
    $keywords->setRequired(false);
    $keywords->setFilters(array('StripTags', 'StringTrim', 'Null'));
    $this->addElement($keywords);

    $redirect = $this->createElement('text', 'redirect');
    $redirect->setLabel('System_Form_Article_Redirect');
    $redirect->setRequired(false);
    $redirect->setFilters(array('StripTags', 'StringTrim', 'Null'));
    $this->addElement($redirect);

    $rewrite = $this->createElement('text', 'rewrite');
    $rewrite->setLabel('System_Form_Article_Rewrite');
    $rewrite->setRequired(false);
    $this->addElement($rewrite);
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Article
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    $submits = $this->getDisplayGroup('submits');
    $submits->removeDecorator('Fieldset');
    $submits->removeDecorator('HtmlTag');
    $submits->getDecorator('DtDdWrapper')->setOption('dtLabel', '');

    return $this->setDecorators($this->_formDecorator);
  }

}
