<?php

/**
 * Třída formuláře pro získání kódů URL článků
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Form_Articles extends Zend_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/articles.phtml')),
    'Form'
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $modelArticles = new System_Model_Articles();

    $filter = array('is_visible' => true);
    $options = $this->_getOptions($modelArticles->getArticlesByFilter($filter));
    $keys = array_keys($options);

    $list = $this->createElement('radio', 'list');
    $list->setDisableTranslator(true);
    $list->setLabel($this->getView()->translate('System_Form_Articles_Urls'));
    $list->setMultiOptions($options);
    $list->setValue(reset($keys));
    $this->addElement($list);

    $select = $this->createElement('button', 'select');
    $select->setLabel('System_Common_Open');
    $this->addElement($select);

    $revoke = $this->createElement('button', 'revoke');
    $revoke->setLabel('System_Common_Cancel');
    $this->addElement($revoke);
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Articles
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    return $this->setDecorators($this->_formDecorator);
  }

  /**
   * Projde články a připraví páry kódů URL => caption
   *
   * @param Zend_Db_Table_Rowset_Abstract $rowset  Seznam článků
   * @return array  Asociativní pole kódů URL => caption
   */
  private function _getOptions(Zend_Db_Table_Rowset_Abstract $rowset) {
    $result = array();

    foreach ($rowset as $row) {
      if (is_object($row)) {
        $result[$row->getCode()] = $row->caption;
      }
    }

    return $result;
  }

}
