<?php

/**
 * Třída formuláře TinyMCE pluginu pro procházení galeriemi
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2016
 */
class System_Form_Browser extends Custom_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/browser.phtml')),
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $languageOptions = array('' => $this->_view->translate('System_Form_Browser_Default'))
                     + System_Model_Texts::getLanguages();

    $selectOptions = array(
      'all'      => $this->_view->translate('System_Form_Browser_Gallery_All'),
      'selected' => $this->_view->translate('System_Form_Browser_Gallery_Selected'),
    );

    $modelObjects = new System_Model_Objects();
    $galleryOptions = $modelObjects->getObjectsByFilter(array(
      'is_visible' => true,
      'is_gallery' => true
    ))->getCaptions();

    $sizeOptions = array(
      'mini'    => $this->_view->translate('System_Form_Browser_Size_Small'),
      'thumb'   => $this->_view->translate('System_Form_Browser_Size_Medium'),
      'display' => $this->_view->translate('System_Form_Browser_Size_Large')
    );

    // - data -
    $language = $this->createElement('select', 'language');
    $language->setLabel($this->_view->translate('System_Form_Gallery_Language'));
    $language->setDisableTranslator(true);
    $language->setMultiOptions($languageOptions);
    $language->setValue(Zend_Registry::get('Zend_Locale')->getLanguage());
    $this->addElement($language);

    $gallery = $this->createElement('radio', 'gallery');
    $gallery->setLabel($this->_view->translate('System_Form_Browser_Gallery'));
    $gallery->setDisableTranslator(true);
    $gallery->setSeparator(' ');
    $gallery->setMultiOptions($selectOptions);
    $gallery->setValue('all');
    $this->addElement($gallery);

    $selection = $this->createElement('multiCheckbox', 'selection');
    $selection->setDisableTranslator(true);
    $selection->setMultiOptions($galleryOptions);
    $this->addElement($selection);

    $size = $this->createElement('select', 'size');
    $size->setLabel($this->_view->translate('System_Form_Browser_Size'));
    $size->setDisableTranslator(true);
    $size->setMultiOptions($sizeOptions);
    $size->setValue('thumb');
    $this->addElement($size);

    $count = $this->createElement('text', 'count');
    $count->setLabel('System_Form_Browser_Limit');
    $count->setDescription('System_Form_Browser_All');
    $count->setValue(4);
    $this->addElement($count);

    $captions = $this->createElement('checkbox', 'captions');
    $captions->setLabel('System_Form_Browser_Captions');
    $captions->setAttrib('disableHidden', true);
    $captions->setValue(true);
    $this->addElement($captions);

    $links = $this->createElement('checkbox', 'links');
    $links->setLabel('System_Form_Browser_Links');
    $links->setAttrib('disableHidden', true);
    $this->addElement($links);

    $index = $this->createElement('checkbox', 'index');
    $index->setLabel('System_Form_Browser_Index');
    $index->setAttrib('disableHidden', true);
    $this->addElement($index);
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Browser
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    return $this->setDecorators($this->_formDecorator);
  }

}
