<?php

/**
 * Třída formuláře pro kontaktování
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia standard © 2013
 */
class System_Form_ContactUs extends Custom_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/contact-us.phtml')),
    'Form'
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $this->setMethod('post');
    $this->setAttrib('id', 'contactus-form');

    // -- data --
    $name = $this->createElement('text', 'name');
    $name->setLabel('System_Form_ContactUs_Name');
    $name->setRequired(true);
    $name->setFilters(array('StripTags', 'StringTrim'));
    $name->setValidators(array(
      'NotEmpty',
      array('StringLength', true, array(3, 128))
    ));
    $name->setDecorators(array('Label', 'ViewHelper', 'Errors'));
    $this->addElement($name);

    $email = $this->createElement('text', 'email');
    $email->setLabel('System_Form_ContactUs_Email');
    $email->setRequired(true);
    $email->setFilters(array('StripTags', 'StringTrim'));
    $email->setValidators(array(
      'NotEmpty',
      array('StringLength', true, array(4, 255)),
      'EmailAddress'
    ));
    $email->setDecorators(array('Label', 'ViewHelper', 'Errors'));
    $this->addElement($email);

    $phone = $this->createElement('text', 'phone');
    $phone->setLabel('System_Form_ContactUs_Phone');
    $phone->setRequired(false);
    $phone->setFilters(array('StripTags', 'StringTrim'));
    $phone->setValidators(array(
      array('StringLength', true, array(0, 16))
    ));
    $phone->setDecorators(array('Label', 'ViewHelper', 'Errors'));
    $this->addElement($phone);

    $url = $this->createElement('text', 'url');
    $url->setLabel('System_Form_ContactUs_NoSpam');
    $url->setRequired(true);
    $url->setFilters(array('StripTags', 'StringTrim', 'StringToLower'));
    $url->setValidators(array(
      array('Identical', true, array('token' => 'nospam'))
    ));
    $url->setDecorators(array('Label', 'ViewHelper', 'Errors'));
    $this->addElement($url);

    $message = $this->createElement('textarea', 'message');
    $message->setAttribs(array('cols' => 40, 'rows' => 5));
    $message->setLabel('System_Form_ContactUs_Message');
    $message->setRequired(true);
    $message->setFilters(array('StripTags', 'StringTrim'));
    $message->setValidators(array('NotEmpty'));
    $message->setDecorators(array('Label', 'ViewHelper', 'Errors'));
    $this->addElement($message);

    // -- skryté --
    $postid = $this->createElement('hidden', 'postid');
    $postid->setDecorators(array('ViewHelper'));
    $postid->setValue('contactus');
    $this->addElement($postid);

    // -- tlačítka --
    $submit = $this->createElement('submit', 'submit');
    $submit->setLabel('System_Common_Send');
    $submit->setDecorators(array('ViewHelper'));
    $this->addElement($submit);
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Contactus
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    return $this->setDecorators($this->_formDecorator);
  }

}
