<?php

/**
 * Třída formuláře pro úpravu složky
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Form_Folder extends System_Form_Abstract {

  const NEW_OBJECT_TEXT  = 'System_Objects_Folder_New';
  const EDIT_OBJECT_TEXT = 'System_Objects_Folder_Edit';

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/folder.phtml')),
    'Form'
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    parent::init();

    $this->setAttrib('id', 'folder-form');

    // - data -
    $this->getElement('language')->setLabel($this->_view->translate('System_Form_Folder_Language'));
    if (is_object($caption = $this->getElement('caption'))) {
      $caption->setLabel('System_Form_Folder_Caption');
      $caption->setRequired(true);
    }
    if (is_object($is_visible = $this->getElement(System_Model_Object::ATTR_IS_VISIBLE))) {
      $is_visible->setLabel('System_Form_Folder_Is_Published');
    }

    $is_gallery = $this->createElement('checkbox', System_Model_Object::ATTR_IS_GALLERY);
    $is_gallery->setLabel('System_Form_Folder_Is_Gallery');
    $is_gallery->setFilters(array('Int'));
    $is_gallery->setValue((int) false);
    $this->addElement($is_gallery);

    $this->getElement('description')->setLabel('System_Form_Folder_Description');
    $this->getElement('rewrite')->setLabel('System_Form_Folder_Rewrite');
    $this->getElement('object_type')->setValue(System_Model_Object::SYSTEM_OBJECT_FOLDER);
  }

  /**
   * Metoda pro zpracování dat formuláře.
   *
   * Metoda ověří data na platnost a provede určitou logiku nad daty
   *
   * @param array $values  Data formuláře
   * @param System_Model_Object $object  Vytvořený objekt
   * @return boolean  Data ověřena a objekt uložen
   */
  public function process(array $values, System_Model_Object &$object = null) {
    if (!$this->isValid($values)) {
      return false;
    }

    $data = $this->getValues();
    $modelObjects = new System_Model_Objects();
    if (!is_object($object = $modelObjects->getObjectById($data['id_object'], $data['language']))) {
      $object = $modelObjects->createRow();
    }

    $object->object_type = System_Model_Object::SYSTEM_OBJECT_FOLDER;
    if ($object->setFromArray($data)->save()) {
      return true;
    }

    return false;
  }

  /**
   * Nastaví výchozí hodnoty elementům
   *
   * Nastaví hodnoty všech elementů uvedených v poli $defaults.
   *
   * @param  array $defaults
   * @return System_Form_Folder
   */
  public function setDefaults(array $defaults) {
    parent::setDefaults($defaults);

    if (0 < ($id_object = (int) $this->getElement('id_object')->getValue())) {
      $modelObjects = new System_Model_Objects();
      if (is_object($object = $modelObjects->getObjectById($id_object))) {
        $this->getDisplayGroup('submits')->setAttrib('size', $object->getSize());
      }
    }

    return $this;
  }

}
