<?php

/**
 * Třída formuláře TinyMCE pluginu pro získání galerií
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class System_Form_Galleries extends Custom_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/galleries.phtml')),
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $languageOptions = array('' => $this->_view->translate('System_Form_Gallery_Default'))
                     + System_Model_Texts::getLanguages();

    $modelObjects = new System_Model_Objects();
    $galleryOptions = $modelObjects->getObjectsByFilter(array(
      'is_visible' => true,
      'is_gallery' => true
    ))->getCaptions();
    $galleryKeys = array_keys($galleryOptions);

    // - data -
    $language = $this->createElement('select', 'language');
    $language->setLabel($this->_view->translate('System_Form_Gallery_Language'));
    $language->setDisableTranslator(true);
    $language->setMultiOptions($languageOptions);
    $language->setValue(Zend_Registry::get('Zend_Locale')->getLanguage());
    $this->addElement($language);

    $gallery = $this->createElement('radio', 'gallery');
    $gallery->setLabel($this->_view->translate('System_Form_Gallery_Select'));
    $gallery->setDisableTranslator(true);
    $gallery->setMultiOptions($galleryOptions);
    $gallery->setValue(reset($galleryKeys));
    $this->addElement($gallery);

    $limit = $this->createElement('text', 'limit');
    $limit->setLabel('System_Form_Gallery_Limit');
    $limit->setDescription('System_Form_Gallery_All');
    $limit->setValue(0);
    $this->addElement($limit);

    $pager = $this->createElement('checkbox', 'pager');
    $pager->setLabel('System_Form_Gallery_Pager');
    $pager->setAttrib('disableHidden', true);
    $this->addElement($pager);

    $link = $this->createElement('checkbox', 'link');
    $link->setLabel('System_Form_Gallery_Link');
    $link->setAttrib('disableHidden', true);
    $this->addElement($link);

    $caption = $this->createElement('checkbox', 'caption');
    $caption->setLabel('System_Form_Gallery_Caption');
    $caption->setAttrib('disableHidden', true);
    $this->addElement($caption);
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Galleries
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    return $this->setDecorators($this->_formDecorator);
  }

}
