<?php

/**
 * Třída formuláře pro přihlášení do administrace
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class System_Form_Log extends Custom_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/log.phtml')),
    'Form'
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $view = $this->getView();

    $this->setMethod('post');
    $this->setAttrib('id', 'log-form');

    // - data -
    $username = $this->createElement('text', 'user');
    $username->setLabel('System_Form_Common_UserName');
    $username->setRequired(true);
    $username->addFilter('StripTags');
    $this->addElement($username);

    $password = $this->createElement('password', 'pass');
    $password->setLabel('System_Form_Common_Password');
    $password->setRequired(true);
    $this->addElement($password);

    $permanent = $this->createElement('checkbox', 'permanent');
    $permanent->setLabel('System_Form_Common_Permanent');
    $permanent->setDecorators(array(
      'ViewHelper',
      array('Label', array('placement' => 'APPEND'))
    ));
    $permanent->setFilters(array('Int'));
    $this->addElement($permanent);

    // - tlačítka -
    $submit = $this->createElement('submit', 'submit');
    $submit->setLabel('System_Form_Common_Login');
    $submit->setDecorators(array('ViewHelper'));
    $this->addElement($submit);
    
    // - zapomenuté údaje -
    $link = $view->simpleUrl('renew', 'users', 'system');
    $caption = $view->translate('System_Admin_Forgotten_Credentials');
    $forgottenValue = "<a href=\"{$link}\">{$caption}</a>";    
    
    $forgotten = $this->createElement('note', 'forgotten');
    $forgotten->setValue($forgottenValue);
    $forgotten->setDecorators(array('ViewHelper', 'DtDdWrapper'));
    $this->addElement($forgotten);
    
    // - skryté -
    $type = $this->createElement('hidden', 'type');
    $type->setValue('database');
    $type->setDecorators(array('ViewHelper'));
    $this->addElement($type);

    $this->addDisplayGroup(array('submit', 'type'), 'submits');
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Log
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    $group = $this->getDisplayGroup('submits');
    $group->removeDecorator('Fieldset');
    $group->removeDecorator('HtmlTag');
    $group->getDecorator('DtDdWrapper')->setOption('dtLabel', '');

    return $this->setDecorators($this->_formDecorator);
  }

}
