<?php

/**
 * Třída formuláře pro získání kódů URL souborů
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Form_Objects extends Zend_Form {

  const FORM_FILES  = 'file';
  const FORM_IMAGES = 'image';
  const FORM_MEDIA  = 'media';

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/objects.phtml')),
    'Form'
  );

  /**
   * Model pro objekty
   *
   * @var System_Model_Objects
   */
  protected $_modelObjects = null;

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $unset = array('type');

    $this->getView();
    $this->setAttrib('id', 'objects-form');

    $formType = $this->getAttrib('type');

    $extensionOptions = System_Model_FileType::getOptions($formType);
    $extensionKeys = array_keys($extensionOptions);

    $this->_modelObjects = new System_Model_Objects();
    $folderOptions = array('0' => $this->_view->translate('System_Objects_Root_Folder'))
                   + $this->_padOptions($this->_modelObjects->getFolders()->getCaptions());
    $folderKeys = array_keys($folderOptions);

    // - data -
    $id_parent = $this->createElement('select', 'id_parent');
    $id_parent->setLabel($this->getView()->translate('System_Form_Objects_Folder'));
    $id_parent->setFilters(array('Int', 'Null'));
    $id_parent->setDisableTranslator(true);
    $id_parent->setMultiOptions($folderOptions);
    $id_parent->setValidators(array(
      array('InArray', true, array('haystack' => $folderKeys))
    ));
    $id_parent->setFilters(array('Int', 'Null'));
    $this->addElement($id_parent);

    if (self::FORM_IMAGES == $formType) {
      $is_original = $this->createElement('checkbox', 'is_original');
      $is_original->setLabel('System_Form_Objects_Original');
      $is_original->setFilters(array('Int'));
      $this->addElement($is_original);

      $is_thumb = $this->createElement('checkbox', 'is_thumb');
      $is_thumb->setLabel('System_Form_Objects_Thumbnail');
      $is_thumb->setFilters(array('Int'));
      $this->addElement($is_thumb);
    }

    $fileName = $this->createElement('text', 'file_name');
    $fileName->setLabel('System_Form_Objects_Filename');
    $fileName->setFilters(array('StripTags', 'StringTrim'));
    $fileName->setFilters(array('StripTags', 'StringTrim'));
    $fileName->setValidators(array(
      array('StringLength', true, array(0, 256))
    ));
    $this->addElement($fileName);

    $fileType = $this->createElement('select', 'file_type');
    $fileType->setLabel($this->getView()->translate('System_Form_Objects_Filetype'));
    $fileType->setDisableTranslator(true);
    $fileType->setMultiOptions($extensionOptions);
    $fileType->setFilters(array('Null'));
    $fileType->setValidators(array(
      array('InArray', true, array('haystack' => $extensionKeys))
    ));
    $this->addElement($fileType);

    // - skryté -
    $type = $this->createElement('hidden', 'type');
    $type->setDecorators(array('ViewHelper'));
    $this->addElement($type);

    $url = $this->createElement('hidden', 'url');
    $url->setDecorators(array('ViewHelper'));
    $this->addElement($url);

    // - tlačítka -
    $open = $this->createElement('button', 'open');
    $open->setLabel('System_Common_Open');
    $open->setDecorators(array('ViewHelper'));
    $this->addElement($open);

    $close = $this->createElement('button', 'close');
    $close->setLabel('System_Common_Cancel');
    $close->setDecorators(array('ViewHelper'));
    $this->addElement($close);

    array_map(array($this, 'removeAttrib'), $unset);
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Objects
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    return $this->setDecorators($this->_formDecorator);
  }

  /**
   * Projde články a připraví páry kódů URL => caption
   *
   * @param Zend_Db_Table_Rowset_Abstract $rowset  Seznam objektů
   * @return array  Asociativní pole kódů URL => caption
   */
  private function _getOptions(Zend_Db_Table_Rowset_Abstract $rowset) {
    $result = array();

    foreach ($rowset as $row) {
      if (is_object($row)) {
        $result[$row->getCode()] = $row->caption;
      }
    }

    return $result;
  }

  /**
   * Doplní mezery (&nbsp;) před položky
   *
   * @param array $options
   * @return array
   */
  private function _padOptions(array $options = array()) {
    $last = end($options);
    array_walk($options, function (&$option) use ($last) {
      $sign = ($option == $last) ? '└' : '├';
      $option = " {$sign} {$option}";
    });
    return $options;
  }

}
