<?php

/**
 * Třída formuláře pro přihlášení do administrace pomocí openID
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class System_Form_OpenID extends Custom_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/open-id.phtml')),
    'Form'
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $view = $this->getView();
    
    $this->setMethod('post');
    $this->setAttrib('id', 'openid-form');

    // - data -
    $identifier = $this->createElement('text', 'openid_identifier');
    $identifier->setDisableTranslator(true);
    $identifier->setLabel('OpenID:');
    $identifier->setFilters(array('StripTags'));
    $this->addElement($identifier);

    $permanent = $this->createElement('checkbox', 'permanent');
    $permanent->setLabel('System_Form_Common_Permanent');
    $permanent->setDecorators(array(
      'ViewHelper',
      array('Label', array('placement' => 'APPEND'))
    ));
    $permanent->setFilters(array('Int'));
    $this->addElement($permanent);

    // - tlačítka -
    $login = $this->createElement('submit', 'submit');
    $login->setLabel('System_Form_Common_Login');
    $login->setDecorators(array('ViewHelper'));
    $this->addElement($login);
        
    // - zapomenuté údaje -
    $link = $view->simpleUrl('renew', 'users', 'system');
    $caption = $view->translate('System_Admin_Forgotten_Credentials');
    $forgottenValue = "<a href=\"{$link}\">{$caption}</a>";    
    
    $forgotten = $this->createElement('note', 'forgotten');
    $forgotten->setValue($forgottenValue);
    $forgotten->setDecorators(array('ViewHelper', 'DtDdWrapper'));
    $this->addElement($forgotten);

    // - skryté -
    $mode = $this->createElement('hidden', 'openid_mode');
    $mode->setDecorators(array('ViewHelper'));
    $this->addElement($mode);

    $type = $this->createElement('hidden', 'type');
    $type->setValue('openid');
    $type->setDecorators(array('ViewHelper'));
    $this->addElement($type);

    $this->addDisplayGroup(array('submit', 'openid_mode', 'type'), 'submits');
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_OpenID
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    $group = $this->getDisplayGroup('submits');
    $group->removeDecorator('Fieldset');
    $group->removeDecorator('HtmlTag');
    $group->getDecorator('DtDdWrapper')->setOption('dtLabel', '');

    return $this->setDecorators($this->_formDecorator);
  }

}
