<?php

/**
 * Třída formuláře pro přihlášení do administrace pomocí jednorázového hesla
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard, ČZU © 2017
 */
class System_Form_Otp extends Custom_Form {

  /**
   * Nastavení dekorátoru formuláře
   *
   * @var array
   */
  protected $_formDecorator = array(
    array('ViewScript', array('viewScript' => 'forms/otp.phtml')),
    'Form'
  );

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $view = $this->getView();

    $this->setMethod('post');
    $this->setAttrib('id', 'otp-form');

    // - data -
    $otp = $this->createElement('password', 'otp');
    $otp->setLabel('System_Form_Common_OTP');
    $otp->setAttrib('autocomplete', 'off');
    $otp->setRequired(true);
    $otp->setFilters(array('StripTags'));
    $this->addElement($otp);

    // - tlačítka -
    $login = $this->createElement('submit', 'submit');
    $login->setLabel('System_Form_Common_Login');
    $login->setDecorators(array('ViewHelper'));
    $this->addElement($login);

    // - skryté -
    $type = $this->createElement('hidden', 'type');
    $type->setValue('otp');
    $type->setDecorators(array('ViewHelper'));
    $this->addElement($type);

    $this->addDisplayGroup(array('submit', 'type'), 'submits');
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Otp
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    $group = $this->getDisplayGroup('submits');
    $group->removeDecorator('Fieldset');
    $group->removeDecorator('HtmlTag');
    $group->getDecorator('DtDdWrapper')->setOption('dtLabel', '');

    return $this->setDecorators($this->_formDecorator);
  }

}
