<?php

/**
 * Třída formuláře pro změnu hesla
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class System_Form_Password extends Custom_Form {

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $this->setMethod('post');
    $this->setAttrib('id', 'password-form');

    // - data -
    $user_password = $this->createElement('password', 'user_password');
    $user_password->setLabel('System_Form_Password_Old');
    $user_password->setRequired(true);
    $user_password->setFilters(array('StripTags'));
    $user_password->setValidators(array('ValidUser'));
    $this->addElement($user_password);

    $newpass1 = $this->createElement('password', 'newpass1');
    $newpass1->setLabel('System_Form_Password_New');
    $newpass1->setRequired(true);
    $newpass1->setFilters(array('StripTags'));
    $newpass1->setValidators(array(
      array('StringLength', true, array('min' => 6))
    ));
    $this->addElement($newpass1);

    $newpass2 = $this->createElement('password', 'newpass2');
    $newpass2->setLabel('System_Form_Password_Confirm');
    $newpass2->setRequired(true);
    $newpass2->setFilters(array('StripTags'));
    $newpass2->setValidators(array(
      array('StringLength', true, array('min' => 6)),
      array('Identical', true, array('token' => 'newpass1'))
    ));
    $this->addElement($newpass2);

    // - tlačítka -
    $submit = $this->createElement('submit', 'submit');
    $submit->setLabel('System_Common_Submit');
    $submit->setDecorators(array('ViewHelper'));
    $this->addElement($submit);

    $cancel = $this->createElement('button', 'cancel');
    $cancel->setLabel('System_Common_Cancel');
    $cancel->setDecorators(array('ViewHelper'));
    $this->addElement($cancel);

    $this->addDisplayGroup(array('submit', 'cancel'), 'submits');
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Password
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    $group = $this->getDisplayGroup('submits');
    $group->removeDecorator('Fieldset');
    $group->removeDecorator('HtmlTag');
    $group->getDecorator('DtDdWrapper')->setOption('dtLabel', '');

    return $this;
  }

}
