<?php

/**
 * Třída formuláře pro registraci uživatele
 *
 * @category   diploma-thesis
 * @package    system
 * @author     Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Petr Jajtner, Webbia Standard © 2016
 */
class System_Form_Register extends Zend_Form {

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $this->setMethod('post');
    $this->setAttrib('id', 'register-form');

    // - data -
    $firstname = $this->createElement('text', 'first_name');
    $firstname->setLabel('System_Form_Register_FirstName');
    $firstname->setRequired(true);
    $firstname->setFilters(array('StripTags', 'StringTrim'));
    $firstname->setValidators(array(
      'Alpha',
      array('StringLength', true, array(3, 128))
    ));
    $this->addElement($firstname);

    $nickname = $this->createElement('text', 'nick_name');
    $nickname->setLabel('System_Form_Register_NickName');
    $nickname->setRequired(false);
    $nickname->setFilters(array('StripTags', 'StringTrim', 'Null'));
    $nickname->setValidators(array(
      'Alnum',
      array('StringLength', true, array(0, 64))
    ));
    $this->addElement($nickname);

    $lastname = $this->createElement('text', 'last_name');
    $lastname->setLabel('System_Form_Register_LastName');
    $lastname->setRequired(true);
    $lastname->setFilters(array('StripTags', 'StringTrim'));
    $lastname->setValidators(array(
      'Alpha',
      array('StringLength', true, array(3, 128))
    ));
    $this->addElement($lastname);

    $email = $this->createElement('text', 'email');
    $email->setLabel('System_Form_Register_Email');
    $email->setRequired(true);
    $email->setFilters(array('StripTags', 'StringTrim'));
    $email->setValidators(array(
      'EmailAddress',
      array('StringLength', true, array(4, 255)),
      array('UniqueEmail')
    ));
    $this->addElement($email);

    $username = $this->createElement('text', 'user_name');
    $username->setLabel('System_Form_Register_LoginName');
    $username->setRequired(true);
    $nickname->setFilters(array('StripTags', 'StringTrim'));
    $lastname->setValidators(array(
      'Alnum',
      'UniqueName',
      array('StringLength', true, array(3, 64))
    ));
    $this->addElement($username);

    // -- tlačítka --
    $submit = $this->createElement('submit', 'submit');
    $submit->setLabel('System_Form_Register_Register');
    $submit->setDecorators(array('ViewHelper'));
    $this->addElement($submit);

    $this->addDisplayGroup(array('submit'), 'submits');
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_Register
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    $submits = $this->getDisplayGroup('submits');
    $submits->removeDecorator('Fieldset');
    $submits->removeDecorator('HtmlTag');
    $submits->getDecorator('DtDdWrapper')->setOption('dtLabel', '');

    return $this;
  }

}
