<?php

/**
 * Třída formuláře pro úpravu uživatele
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2011
 */
class System_Form_User extends Custom_Form {

  /**
   * Inicializace formuláře
   *
   * @return void
   */
  public function init() {
    $this->setMethod('post');
    $this->setAttrib('id', 'user-form');

    // - data -
    $firstname = $this->createElement('text', 'first_name');
    $firstname->setLabel('System_Form_User_FirstName');
    $firstname->setRequired(true);
    $firstname->setFilters(array('StripTags', 'StringTrim'));
    $firstname->setValidators(array(
      'Alpha',
      array('StringLength', true, array(3, 128))
    ));
    $this->addElement($firstname);

    $lastname = $this->createElement('text', 'last_name');
    $lastname->setLabel('System_Form_User_LastName');
    $lastname->setRequired(true);
    $lastname->setFilters(array('StripTags', 'StringTrim'));
    $lastname->setValidators(array(
      'Alpha',
      array('StringLength', true, array(3, 128))
    ));
    $this->addElement($lastname);

    $nickname = $this->createElement('text', 'nick_name');
    $nickname->setLabel('System_Form_User_NickName');
    $nickname->setRequired(false);
    $nickname->setFilters(array('StripTags', 'StringTrim', 'Null'));
    $lastname->setValidators(array(
      'Alnum',
      array('StringLength', true, array(0, 64))
    ));
    $this->addElement($nickname);

    $email = $this->createElement('text', 'email');
    $email->setLabel('System_Form_User_Email');
    $email->setRequired(true);
    $email->setFilters(array('StripTags', 'StringTrim'));
    $email->setValidators(
      array(
        'EmailAddress',
        array('StringLength', true, array(4, 255)),
        array('UniqueEmail', false, array(new System_Model_Users()))
      )
    );
    $this->addElement($email);

    $username = $this->createElement('text', 'user_name');
    $username->setLabel('System_Form_User_UserName');
    $username->setRequired(true);
    $username->setFilters(array('StripTags', 'StringTrim'));
    $username->setValidators(array(
      'Alnum',
      array('StringLength', true, array(3, 64))
    ));
    $this->addElement($username);

    $is_active = $this->createElement('checkbox', 'is_active');
    $is_active->setLabel('System_Form_User_IsActive');
    $is_active->setFilters(array('Int'));
    $this->addElement($is_active);

    $otpOptions = array(
      ''     => 'System_Form_User_2F_None',
      'hotp' => 'System_Form_User_2F_HOTP',
      'totp' => 'System_Form_User_2F_TOTP'
    );

    $otpType = $this->createElement('radio', 'otp_type');
    $otpType->setLabel('System_Form_User_2ndFactor');
    $otpType->setMultiOptions($otpOptions);
    $otpType->setSeparator('  ');
    $otpType->setFilters(array('StripTags', 'StringTrim'));
    $otpType->setValidators(array(
      array('InArray', true, array('haystack' => array_keys($otpOptions)))
    ));
    $otpType->setValue(key($otpOptions));
    $this->addElement($otpType);

    // - tlačítka -
    $submit = $this->createElement('submit', 'submit');
    $submit->setLabel('System_Common_Save');
    $submit->setDecorators(array('ViewHelper'));
    $this->addElement($submit);

    $cancel = $this->createElement('button', 'cancel');
    $cancel->setLabel('System_Common_Cancel');
    $cancel->setDecorators(array('ViewHelper'));
    $this->addElement($cancel);

    // - skryté -
    $id_user = $this->createElement('hidden', 'id_user');
    $id_user->setDecorators(array('ViewHelper'));
    $this->addElement($id_user);

    $this->addDisplayGroup(array('id_user', 'submit', 'cancel'), 'submits');
  }

  /**
   * Načte výchozí dekorátory a nastaví nové vlastnosti
   *
   * @return System_Form_User
   */
  public function loadDefaultDecorators() {
    parent::loadDefaultDecorators();

    $submits = $this->getDisplayGroup('submits');
    $submits->removeDecorator('Fieldset');
    $submits->removeDecorator('HtmlTag');
    $submits->getDecorator('DtDdWrapper')->setOption('dtLabel', '');

    return $this;
  }

  /**
   * Nastaví výchozí hodnoty elementů
   *
   * Nastaví hodnoty pro všechny elementy uvedené v poli $defaults.
   *
   * @param  array $defaults
   * @return System_Form_User
   */
  public function setDefaults(array $defaults) {
    if (array_key_exists('otp_type', $defaults) && !$defaults['otp_type']) {
      $defaults['otp_type'] = (string) $defaults['otp_type'];
    }
    return parent::setDefaults($defaults);
  }

  /**
   * Načte všechny hodnoty elementů formuláře
   *
   * @param  boolean $suppressArrayNotation
   * @return array
   */
  public function getValues($suppressArrayNotation = false) {
    $values = parent::getValues($suppressArrayNotation);
    if (array_key_exists('otp_type', $values) && !$values['otp_type']) {
      $values['otp_type'] = null;
    }
    return $values;
  }

}
