<?php

/**
 * Třída databázového modelu pro blokování přihlašování
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, ČZU © 2016
 */
class System_Model_Blocked extends Zend_Db_Table_Abstract {

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_blocked';

  /**
   * Sloupec primárního klíče
   *
   * @var string
   */
  protected $_primary = 'ip_address';

  /**
   * Vrátí TRUE, pokud je IP ještě blokována; jinak FALSE
   *
   * @param string $ip
   * @return boolean
   */
  public static function isBlocked($ip) {
    $self = new self();
    $self->delete("date_expire < NOW()");

    $select = $self->select(self::SELECT_WITH_FROM_PART);
    $select->reset(self::COLUMNS)->columns('COUNT(*)');
    $select->where('ip_address = ?', $ip);

    return $self->getAdapter()->fetchOne((string) $select);
  }

  /**
   * Uloží IP $ip pro blokování na $secords sekund
   *
   * @param string $ip
   * @param integer $seconds
   * @return integer | boolean
   */
  public static function blockIP($ip, $seconds) {
    if (!self::isBlocked($ip)) {
      $self = new self();
      return $self->insert(array(
        'ip_address'  => $ip,
        'date_expire' => date('Y-m-d H:i:s', time() + $seconds)
      ));
    }
    return false;
  }

}
