<?php

/**
 * Třída modelu pro odesílání emailové notifikace
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class System_Model_Email {

  /**
   * Odešle emailovou zprávu příjemcům
   *
   * @param array $emails  Emaily příjemců
   * @param array $values  Hodnoty z formuláře
   * @return void
   */
  public static function notify(array $emails, array $values) {
    if (empty($emails) || empty($values)) {
      return;
    }

    ksort($emails);

    $view = Zend_Registry::get('Zend_View');
    $view->data = (object) $values;

    $subject = 'Zpráva ze stránek '.$_SERVER['SERVER_NAME'];
    $message = $view->render('email/notify.phtml');

    foreach ($emails as $email) {
      self::_send($values['email'], $values['name'], $email, $subject, $message);
    }
  }

  /**
   * Obecné odesílání emailu uživateli
   *
   * @param string $from  Email tazatele
   * @param string $name  Jméno tazatele
   * @param string $to  Email příjemce
   * @param string $subject  Předmět emailu
   * @param string $message  Tělo emailu
   * @return void
   */
  protected static function _send($from, $name, $to, $subject, $message) {
    try {
      $mail = new Zend_Mail('UTF-8');
      $mail->setFrom($from, $name);
      $mail->addTo($to);
      $mail->setSubject($subject);
      $mail->setBodyHtml($message);
      $mail->setBodyText(strip_tags($message));
      $mail->send();
    } catch (Exception $e) {
      // nereagovat na chybu
    }
  }

}
