<?php

/**
 * Třída pro práci s tabulkou webbia_files
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Files extends Zend_Db_Table_Abstract {

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_files';

  /**
   * Sloupec primárního klíče
   *
   * @var string
   */
  protected $_primary = 'id_file';

  /**
   * Třída řádku
   *
   * @var string
   */
  protected $_rowClass = 'System_Model_File';

  /**
   * Třída skupiny řádků
   *
   * @var string
   */
  protected $_rowsetClass = 'System_Model_FileSet';

  /**
   * Pole názvů tříd závislých tabulek
   *
   * @var array
   */
  protected $_dependentTables = array(
    'System_Model_Objects'
  );

  /**
   * Vrátí informace o souboru dle $id_file
   *
   * @param integer $id_file
   * @return System_Model_File
   */
  public function getFileById($id_file) {
    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $key = md5(__METHOD__.$id_file);

    if (!is_object($cache) || (($result = $cache->load($key)) === false)) {
      $result = $this->find($id_file)->current();
      if (is_object($cache)) {
        $cache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí informace o souboru dle $key
   *
   * @param string $key
   * @return System_Model_File
   */
  public function getFileByKey($key) {
    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $hash = md5(__METHOD__.$key);

    if (!is_object($cache) || (($result = $cache->load($hash)) === false)) {
      $result = $this->fetchRow(array('file_key = ?' => $key));
      if (is_object($cache)) {
        $cache->save($result, $hash);
      }
    }

    return $result;
  }

  /**
   * Vrátí seznam souborů dle modulu a typu
   *
   * @param string|array $type     Typ nebo typy souborů
   * @return Zend_Db_Table_Rowset
   */
  public function getFilesByType($type = '') {
    $where = array();
    if (is_string($type) && !empty($type)) {
      $where['file_type = ?'] = $type;
    }
    if (is_array($type) && !empty($type)) {
      $where['file_type IN (?)'] = $type;
    }
    return $this->fetchAll($where);
  }

  /**
   * Vrátí soubory dle pole $id_files
   *
   * @param array $id_files  Pole ID souborů
   * @return System_Model_FileSet
   */
  public function getFilesByIDs(array $id_files) {
    return $this->fetchAll(array('id_file IN (?)' => $id_files));
  }

  /**
   * Vrátí informace o souboru dle $identifier - id_file nebo key
   *
   * @param integer|string $identifier
   * @return System_Model_File
   */
  public static function getFile($identifier) {
    $self = new self();
    return is_numeric($identifier)
           ? $self->getFileById($identifier)
           : $self->getFileByKey($identifier);
  }

  /**
   * Vrátí soubory dle pole $id_files
   *
   * @param array $id_files  Pole ID souborů
   * @return System_Model_FileSet
   */
  public static function getFiles(array $id_files) {
    $self = new self();
    return $self->getFilesByIDs($id_files);
  }

  /**
   * Vrátí prázdnou množinu souborů
   *
   * @return System_Model_FileSet
   */
  public static function emptySet() {
    $self = new self();
    return $self->fetchAll('1 = 0');
  }

}
