<?php

/**
 * Třída pro státní svátky a dnů pracovního volna (včetně nedělí)
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Holidays {

  /**
   * Výčet pevně stanovených svátků v ČR
   *
   * @var array
   */
  protected static $_holidays = array(
    '01-01' => 'Nový rok, den obnovy českého státu (1993)',
    '05-01' => 'Svátek práce',
    '05-08' => 'Den osvobození',
    '07-05' => 'Den slovanských věrozvěstů - Cyrila a Metoděje (863)',
    '07-06' => 'Den upálení mistra Jana Husa (1415)',
    '09-28' => 'Den české státnosti',
    '10-28' => 'Den vzniku československého státu (1918)',
    '11-17' => 'Den boje za svobodu a demokracii',
    '12-24' => 'Štědrý den',
    '12-25' => '1. svátek vánoční',
    '12-26' => '2. svátek vánoční',
    'Velikonoční pondělí'
  );

  /**
   * Vrátí TRUE, pokud je v $date státní svátek. Pokud není parametr uveden,
   * bude použit aktuální den.
   *
   * @param string $date  Testované datum v ISO 8601, výchozí NULL (aktuální den)
   * @param string $name  Výstup názvu státního svátku nebo neděle
   * @return boolean      TRUE, pokud je státní svátek nebo neděle; jinak FALSE
   */
  public static function isHoliday($date = null, &$name = null) {
    $dateObj = new Zend_Date($date, Zend_Date::ISO_8601);
    $key = $dateObj->get('MM-dd');

    if (array_key_exists($key, self::$_holidays)) {
      $name = self::$_holidays[$key];
      return true;
    }

    if (0 == $dateObj->get('e')) {
      $name = 'Neděle';
      return true;
    }

    $easter = date('m-d', easter_date($dateObj->get('yyyy')) + 86400);
    if ($key == $easter) {
      $name = end(self::$_holidays);
      return true;
    }

    return false;
  }

  /**
   * Vrátí počet státních svátků v rozsahu $from - $to. Pokud je uveden parametr
   * $holidays, vloží do něj asociativní pole datum -> název svátku
   *
   * @param string $from    Datum počátku rozsahu v ISO 8601
   * @param string $to      Datum konce rozsahu v ISO 8601
   * @param array $holidays Asociativní pole státních svátků jako datum -> název
   * @return integer        Počet svátků v daném rozsahu
   */
  public static function numberOfHolidays($from, $to, array &$holidays = null) {
    $fromObj = new Zend_Date($from, Zend_Date::ISO_8601);
    $toObj = new Zend_Date($to, Zend_Date::ISO_8601);

    if ($fromObj->getTimestamp() > $toObj->getTimestamp()) {
      return -1;
    }

    $count = 0;
    $holidays = array();
    while ($fromObj->getTimestamp() < $toObj->getTimestamp()) {
      if (self::isHoliday($date = $fromObj->get('yyyy-MM-dd'), $name)) {
        $holidays[$date] = $name;
        ++$count;
      }
      $fromObj->add('+1', Zend_Date::DAY);
    }

    return $count;
  }

}
