<?php

if (!function_exists('imagecreatefrombmp')) {

  /**
   * Funkce pro vytvoření obrázku z bitmapy
   *
   * @param string $filename  Soubor s bitmapou
   * @return boolean|resource
   */
  function imagecreatefrombmp($filename) {
    if (!$f1 = fopen($filename, 'rb')) {
      return false;
    }

    $file = unpack("vfile_type/Vfile_size/Vreserved/Vbitmap_offset", fread($f1, 14));
    if ($file['file_type'] != 19778) {
      return false;
    }

    $bmp = unpack('Vheader_size/Vwidth/Vheight/vplanes/vbits_per_pixel'.
                  '/Vcompression/Vsize_bitmap/Vhoriz_resolution'.
                  '/Vvert_resolution/Vcolors_used/Vcolors_important', fread($f1, 40));

    $bmp['colors'] = pow(2, $bmp['bits_per_pixel']);
    if ($bmp['size_bitmap'] == 0) {
      $bmp['size_bitmap'] = $file['file_size'] - $file['bitmap_offset'];
    }
    $bmp['bytes_per_pixel'] = $bmp['bits_per_pixel'] / 8;
    $bmp['bytes_per_pixel2'] = ceil($bmp['bytes_per_pixel']);
    $bmp['decal'] = ($bmp['width']*$bmp['bytes_per_pixel'] / 4);
    $bmp['decal'] -= floor($bmp['width']*$bmp['bytes_per_pixel'] / 4);
    $bmp['decal'] = 4 - (4 * $bmp['decal']);
    if ($bmp['decal'] == 4) {
      $bmp['decal'] = 0;
    }

    $palette = array();
    if ($bmp['colors'] < 16777216) {
      $palette = unpack('V'.$bmp['colors'], fread($f1, $bmp['colors'] * 4));
    }

    $img = fread($f1, $bmp['size_bitmap']);
    $vide = chr(0);

    $res = imagecreatetruecolor($bmp['width'], $bmp['height']);
    $p = 0;
    $y = $bmp['height'] - 1;
    while ($y >= 0) {
      $x = 0;
      while ($x < $bmp['width']) {
        if ($bmp['bits_per_pixel'] == 24) {
          $color = unpack("V", substr($img, $p, 3).$vide);
        } elseif ($bmp['bits_per_pixel'] == 16) {
          $color = unpack("n", substr($img, $p, 2));
          $color[1] = $palette[$color[1] + 1];
        } elseif ($bmp['bits_per_pixel'] == 8) {
          $color = unpack("n", $vide.substr($img, $p, 1));
          $color[1] = $palette[$color[1] + 1];
        } elseif ($bmp['bits_per_pixel'] == 4) {
          $color = unpack("n", $vide.substr($img, floor($p), 1));
          if (($p * 2) % 2 == 0) {
            $color[1] = ($color[1] >> 4);
          } else {
            $color[1] = ($color[1] & 0x0F);
          }
          $color[1] = $palette[$color[1] + 1];
        } elseif ($bmp['bits_per_pixel'] == 1) {
          $color = unpack("n", $vide.substr($img, floor($p), 1));
          if (($p * 8) % 8 == 0) {
            $color[1] = $color[1] >> 7;
          } elseif (($p * 8) % 8 == 1) {
            $color[1] = ($color[1] & 0x40) >> 6;
          } elseif (($p * 8) % 8 == 2) {
            $color[1] = ($color[1] & 0x20) >> 5;
          } elseif (($p * 8) % 8 == 3) {
            $color[1] = ($color[1] & 0x10) >> 4;
          } elseif (($p * 8) % 8 == 4) {
            $color[1] = ($color[1] & 0x8) >> 3;
          } elseif (($p * 8) % 8 == 5) {
            $color[1] = ($color[1] & 0x4) >> 2;
          } elseif (($p * 8) % 8 == 6) {
            $color[1] = ($color[1] & 0x2) >> 1;
          } elseif (($p * 8) % 8 == 7) {
            $color[1] = ($color[1] & 0x1);
          }
          $color[1] = $palette[$color[1] + 1];
        } else {
          return false;
        }
        imagesetpixel($res, $x, $y, $color[1]);
        $x++;
        $p += $bmp['bytes_per_pixel'];
      }
      $y--;
      $p += $bmp['decal'];
    }

    fclose($f1);

    return $res;
  }
}

if (!function_exists('imagebmp')) {

  /**
   * Funkce pro výstup obrázku s bitmapou
   *
   * @param resource $im
   * @param string $fn
   * @return boolean
   */
  function imagebmp($im, $fn = null) {
    if (!$im) {
      return false;
    }

    if ($fn === null) {
      $fn = 'php://output';
    }

    $f = fopen($fn, 'w');
    if (!$f) {
      return false;
    }

    $biWidth = imagesx($im);
    $biHeight = imagesy($im);
    $biBPLine = $biWidth * 3;
    $biStride = ($biBPLine + 3) & ~3;
    $biSizeImage = $biStride * $biHeight;
    $bfOffBits = 54;
    $bfSize = $bfOffBits + $biSizeImage;

    fwrite($f, 'BM', 2);
    fwrite($f, pack('VvvV', $bfSize, 0, 0, $bfOffBits));
    fwrite($f, pack('VVVvvVVVVVV', 40, $biWidth, $biHeight, 1, 24, 0, $biSizeImage, 0, 0, 0, 0));

    $numpad = $biStride - $biBPLine;
    for ($y = $biHeight - 1; $y >= 0; --$y) {
      for ($x = 0; $x < $biWidth; ++$x) {
        $col = imagecolorat($im, $x, $y);
        fwrite($f, pack('V', $col), 3);
      }
      for ($i = 0; $i < $numpad; ++$i) {
        fwrite($f, pack('C', 0));
      }
    }

    fclose($f);
    return true;
  }

}

/**
 * Třída pro práci s obrázky
 *
 * @package  system
 * @author   David Grudl, Bc. Petr Jajtner
 *
 * @method System_Model_Image rotate(float $angle, $backgroundColor)
 */
class System_Model_Image {

  const FIT     = 0;  // Vyplnit
  const SHRINK  = 1;  // Zmenšit
  const STRETCH = 2;  // Natáhnout
  const FILL    = 4;  // Vyplnit
  const EXACT   = 8;  // Přesně

  const EMPTY_GIF = "GIF89a\x01\x00\x01\x00\x80\x00\x00\x00\x00\x00\x00\x00\x00!\xf9\x04\x01\x00\x00\x00\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x02D\x01\x00;";

  /**
   * @var resource
   */
  private $_image;

  /**
   * Vrátí barvu RGBA.
   *
   * @param integer $red  Červená (0 - 255)
   * @param integer $green Zelená (0 - 255)
   * @param integer $blue  Modrá (0 - 255)
   * @param integer $transparency  Půhlednost (0 - 127)
   * @return array
   */
  public static function rgba($red, $green, $blue, $transparency = 0) {
    return array(
      'red'   => max(0, min((int) $red, 255)),
      'green' => max(0, min((int) $green, 255)),
      'blue'  => max(0, min((int) $blue, 255)),
      'alpha' => max(0, min((int) $transparency, 127)),
    );
  }

  /**
   * Otevře obrázek ze souboru
   *
   * @param string $file
   * @param mixed $format  Zjištěný formát obrázku (pouze na výstup)
   * @return System_Model_Image
   */
  public static function fromFile($file, &$format = null) {
    if (!extension_loaded('gd')) {
      require_once 'Custom/Exception.php';
      throw new Custom_Exception('PHP rozšíření „GD” není k dispozici.');
    }

    /**
     * @ - Soubory menší než 12 bajtů způsobují chybu při čtení
     */
    $info = @getimagesize($file);
    switch ($format = $info[2]) {
      case IMAGETYPE_JPEG:
        return new self(imagecreatefromjpeg($file));
      case IMAGETYPE_PNG:
        return new self(imagecreatefrompng($file));
      case IMAGETYPE_GIF:
        return new self(imagecreatefromgif($file));
      case IMAGETYPE_BMP:
        return new self(imagecreatefrombmp($file));
      default:
        require_once 'Custom/Exception.php';
        throw new Custom_Exception("Neznámý obrazový typ nebo soubor „{$file}” nebyl nalezen.");
    }
  }

  /**
   * Vrátí MIME typ obsahu souboru.
   *
   * @param string $data
   * @return string
   */
  public static function mimeFromString($data) {
    if (extension_loaded('fileinfo') &&
        preg_match('#^(\S+/[^\s;]+)#', finfo_buffer(finfo_open(FILEINFO_MIME)), $m)) {
      return $m[1];
    } elseif (strncmp($data, "\xff\xd8", 2) === 0) {
      return 'image/jpeg';
    } elseif (strncmp($data, "\x89PNG", 4) === 0) {
      return 'image/png';
    } elseif (strncmp($data, "GIF", 3) === 0) {
      return 'image/gif';
    } elseif (strncmp($data, "BM", 2) === 0) {
      return 'image/bmp';
    } else {
      return 'application/octet-stream';
    }
  }

  /**
   * Vrátí IMAGETYPE dle $mime
   *
   * @param string $mime
   * @return integer|null
   */
  public static function imageType($mime) {
    $types = array(
      'image/bmp'  => IMAGETYPE_BMP,
      'image/jpeg' => IMAGETYPE_JPEG,
      'image/gif'  => IMAGETYPE_GIF,
      'image/png'  => IMAGETYPE_PNG
    );
    return isset($types[$mime]) ? $types[$mime] : null;
  }

  /**
   * Vrátí formát v řetězci obrazového streamu.
   *
   * @param string $s
   * @return mixed  Zjištěný typ obrázku
   */
  public static function getFormatFromString($s) {
    $mime = self::mimeFromString($s);
    return self::imageType($mime);
  }

  /**
   * Vytvoří nový obrázek z řetězce obrazového streamu.
   *
   * @param string $s
   * @param mixed $format  Zjištěný formát obrázku (pouze na výstup)
   * @return System_Model_Image
   */
  public static function fromString($s, &$format = null) {
    if (!extension_loaded('gd')) {
      require_once 'Custom/Exception.php';
      throw new Custom_Exception('PHP rozšíření „GD” není k dispozici.');
    }
    $format = self::getFormatFromString($s);
    return new self(imagecreatefromstring($s));
  }

  /**
   * Vytvoří prázdný obrázek.
   *
   * @param integer $width
   * @param integer $height
   * @param array $color
   * @return System_Model_Image
   */
  public static function fromBlank($width, $height, $color = null) {
    if (!extension_loaded('gd')) {
      require_once 'Custom/Exception.php';
      throw new Custom_Exception('PHP rozšíření „GD” není k dispozici.');
    }

    $width = (int) $width;
    $height = (int) $height;

    if ($width < 1 || $height < 1) {
      require_once 'Custom/Exception.php';
      throw new Custom_Exception('Šířka a výška obrázku musí být větší než nula.');
    }

    $image = imagecreatetruecolor($width, $height);
    if (is_array($color)) {
      $color += array('alpha' => 0);
      $color = imagecolorallocatealpha(
                $image,
                $color['red'],
                $color['green'],
                $color['blue'],
                $color['alpha']
              );
      imagealphablending($image, false);
      imagefilledrectangle($image, 0, 0, $width - 1, $height - 1, $color);
      imagealphablending($image, true);
    }

    return new self($image);
  }

  /**
   * Konstruktor
   *
   * Zapoudření GD.
   *
   * @param resource $image
   * @return System_Model_Image
   */
  public function __construct($image) {
    $this->setImageResource($image);
    imagesavealpha($image, true);
  }

  /**
   * Vrátí šířku obrázku.
   *
   * @return integer
   */
  public function getWidth() {
    return imagesx($this->_image);
  }

  /**
   * Vrátí výšku obrázku.
   *
   * @return integer
   */
  public function getHeight() {
    return imagesy($this->_image);
  }

  /**
   * Nastaví resource obrázku.
   *
   * @param resource $image
   * @return System_Model_Image  Poskytuje plynulé rozhraní
   */
  protected function setImageResource($image) {
    if (!is_resource($image) || get_resource_type($image) !== 'gd') {
      require_once 'Custom/Exception.php';
      throw new Custom_Exception('Obrázek není platný.');
    }
    $this->_image = $image;
    return $this;
  }

  /**
   * Vrátí GD resource obrázku.
   *
   * @return resource
   */
  public function getImageResource() {
    return $this->_image;
  }

  /**
   * Změní velikost obrázku.
   *
   * @param mixed $width  Šířka v pixelech nebo procentech
   * @param mixed $height  Výška v pixelech nebo procentech
   * @param integer $flags  Příznaky (FIT, SHRINK, STRETCH, FILL, EXACT)
   * @return System_Model_Image  Poskytuje plynulé rozhraní
   */
  public function resize($width, $height, $flags = self::FIT) {
    if ($flags & self::EXACT) {
      return $this->resize($width, $height, self::FILL)
                  ->crop('50%', '50%', $width, $height);
    }

    list($new_width, $new_height) = self::calculateSize(
                                      $this->getWidth(),
                                      $this->getHeight(),
                                      $width, $height, $flags
                                    );

    if ($new_width !== $this->getWidth() ||
        $new_height !== $this->getHeight()) {
      $newImage = self::fromBlank($new_width, $new_height, self::rgba(0, 0, 0, 127))->getImageResource();
      imagecopyresampled(
        $newImage, $this->getImageResource(),
        0, 0, 0, 0,
        $new_width, $new_height, $this->getWidth(), $this->getHeight()
      );
      $this->_image = $newImage;
    }

    if ($width < 0 || $height < 0) { // flip is processed in two steps for better quality
      $new_image = static::fromBlank($new_width, $new_height, self::rgba(0, 0, 0, 127))->getImageResource();
      imagecopyresampled(
        $new_image, $this->getImageResource(),
        0, 0,
        $width < 0 ? $new_width - 1 : 0,
        $height < 0 ? $new_height - 1 : 0,
        $new_width, $new_height,
        $width < 0 ? -$new_width : $new_width,
        $height < 0 ? -$new_height : $new_height
      );
      $this->_image = $new_image;
    }

    return $this;
  }

  /**
   * Spočítá nové rozměry obrazu.
   *
   * @param mixed $srcWidth  Šířka zdroje
   * @param mixed $srcHeight  Výška zdroje
   * @param mixed $newWidth  Šířka v pixelech nebo procentech
   * @param mixed $newHeight  Výška v pixelech nebo procentech
   * @param integer $flags  Příznaky (FIT, SHRINK, STRETCH, FILL, EXACT)
   * @return array
   */
  public static function calculateSize($srcWidth, $srcHeight, $newWidth, $newHeight, $flags = self::FIT) {
    if (substr($newWidth, -1) === '%') {
      $newWidth = round($srcWidth / 100 * abs($newWidth));
      $percents = true;
    } else {
      $newWidth = (int) abs($newWidth);
    }

    if (substr($newHeight, -1) === '%') {
      $newHeight = round($srcHeight / 100 * abs($newHeight));
      $flags |= empty($percents) ? 0 : self::STRETCH;
    } else {
      $newHeight = (int) abs($newHeight);
    }

    if ($flags & self::STRETCH) {
      if (empty($newWidth) || empty($newHeight)) {
        require_once 'Custom/Exception.php';
        throw new Custom_Exception('For stretching must be both width and height specified.');
      }

      if ($flags & self::SHRINK) {
        $newWidth = round($srcWidth * min(1, $newWidth / $srcWidth));
        $newHeight = round($srcHeight * min(1, $newHeight / $srcHeight));
      }
    } else {
      if (empty($newWidth) && empty($newHeight)) {
        throw new InvalidArgumentException('At least width or height must be specified.');
      }

      $scale = array();
      if ($newWidth > 0) {
        $scale[] = $newWidth / $srcWidth;
      }

      if ($newHeight > 0) {
        $scale[] = $newHeight / $srcHeight;
      }

      if ($flags & self::FILL) {
        $scale = array(max($scale));
      }

      if ($flags & self::SHRINK) {
        $scale[] = 1;
      }

      $minScale = min($scale);
      $newWidth = round($srcWidth * $minScale);
      $newHeight = round($srcHeight * $minScale);
    }

    return array(max((int) $newWidth, 1), max((int) $newHeight, 1));
  }

  /**
   * Ořízne obrázek.
   *
   * @param mixed $left  Posun v ose X v pixelech nebo procentech
   * @param mixed $top  Posun v ose Y v pixelech nebo procentech
   * @param mixed $width  Šířka v pixelech nebo procentech
   * @param mixed $height  Výška v pixelech nebo procentech
   * @return System_Model_Image  Poskytuje plynulé rozhraní
   */
  public function crop($left, $top, $width, $height) {
    list($left, $top, $width, $height) = static::calculateCutout($this->getWidth(), $this->getHeight(), $left, $top, $width, $height);
    $new_image = self::fromBlank($width, $height, self::rgba(0, 0, 0, 127))->getImageResource();
    imagecopy($new_image, $this->getImageResource(), 0, 0, $left, $top, $width, $height);
    $this->_image = $new_image;
    return $this;
  }

  /**
   * Spočítá rozměry výřezu v obraze.
   *
   * @param mixed $srcWidth  Šířka zdroje
   * @param mixed $srcHeight  Výška zdroje
   * @param mixed $left  Posun v ose X v pixelech nebo procentech
   * @param mixed $top  Posun v ose Y v pixelech nebo procentech
   * @param mixed $newWidth  Šířka v pixelech nebo procentech
   * @param mixed $newHeight  Výška v pixelech nebo procentech
   * @return array
   */
  public static function calculateCutout($srcWidth, $srcHeight, $left, $top, $newWidth, $newHeight) {
    if (substr($newWidth, -1) === '%') {
      $newWidth = round($srcWidth / 100 * $newWidth);
    }
    if (substr($newHeight, -1) === '%') {
      $newHeight = round($srcHeight / 100 * $newHeight);
    }
    if (substr($left, -1) === '%') {
      $left = round(($srcWidth - $newWidth) / 100 * $left);
    }
    if (substr($top, -1) === '%') {
      $top = round(($srcHeight - $newHeight) / 100 * $top);
    }
    if ($left < 0) {
      $newWidth += $left;
      $left = 0;
    }
    if ($top < 0) {
      $newHeight += $top;
      $top = 0;
    }

    $newWidth = min((int) $newWidth, $srcWidth - $left);
    $newHeight = min((int) $newHeight, $srcHeight - $top);

    return array($left, $top, $newWidth, $newHeight);
  }

  /**
   * Zostří obraz.
   *
   * Na základě magických čísel Davida Grudla
   *
   * @return System_Model_Image  Poskytuje plynulé rozhraní
   */
  public function sharpen() {
    imageconvolution(
      $this->getImageResource(),
      array(
        array(-1, -1, -1),
        array(-1, 24, -1),
        array(-1, -1, -1),
      ), 16, 0
    );

    return $this;
  }

  /**
   * Vloží $image do aktuálního obrázku.
   *
   * @param System_Model_Image $image
   * @param mixed $left  Souřadnice X v pixelech nebo procentech
   * @param mixed $top  Souřadnice Y v pixelech nebo procentech
   * @param integer $opacity  Neprůhlednost (0 - 100)
   * @return System_Model_Image  Poskytuje plynulé rozhraní
   */
  public function place(System_Model_Image $image, $left = 0, $top = 0, $opacity = 100) {
    $opacity = max(0, min(100, (int) $opacity));

    if (substr($left, -1) === '%') {
      $left = round(($this->getWidth() - $image->getWidth()) / 100 * $left);
    }

    if (substr($top, -1) === '%') {
      $top = round(($this->getHeight() - $image->getHeight()) / 100 * $top);
    }

    if ($opacity === 100) {
      imagecopy(
        $this->getImageResource(), $image->getImageResource(),
        $left, $top, 0, 0, $image->getWidth(), $image->getHeight()
      );
    } elseif ($opacity <> 0) {
      imagecopymerge(
        $this->getImageResource(), $image->getImageResource(),
        $left, $top, 0, 0, $image->getWidth(), $image->getHeight(),
        $opacity
      );
    }

    return $this;
  }

  /**
   * Uloží obrázek do souboru.
   *
   * @param string $file  Soubor
   * @param integer $quality  Kvalita (0 - 100: pro JPEG a PNG)
   * @param integer $type  Typ obrázku (nepovinné)
   * @return boolean  TRUE při úspěchu, FALSE při selhání.
   * @throws Custom_Exception
   */
  public function save($file = null, $quality = null, $type = null) {
    if ($type === null) {
      switch (strtolower(pathinfo($file, PATHINFO_EXTENSION))) {
        case 'jpg':
        case 'jpe':
        case 'jpeg':
          $type = IMAGETYPE_JPEG;
          break;
        case 'png':
          $type = IMAGETYPE_PNG;
          break;
        case 'gif':
          $type = IMAGETYPE_GIF;
          break;
        case 'bmp':
          $type = IMAGETYPE_BMP;
          break;
      }
    }

    switch ($type) {
      case IMAGETYPE_JPEG:
        $quality = $quality === null ? 90 : max(0, min(100, (int) $quality));
        return imagejpeg($this->getImageResource(), $file, $quality);
      case IMAGETYPE_PNG:
        $quality = $quality === null ? 9 : max(0, min(9, (int) $quality));
        return imagepng($this->getImageResource(), $file, $quality);
      case IMAGETYPE_GIF:
        if (null === $file) {
          return imagegif($this->getImageResource());
        } else {
          return imagegif($this->getImageResource(), $file); // PHP bug #44591
        }
      case IMAGETYPE_BMP:
        return imagebmp($this->getImageResource(), $file);
      default:
        require_once 'Custom/Exception.php';
        throw new Custom_Exception('Nepodporovaný typ obrázku.');
    }
  }

  /**
   * Vrátí obrázek jako řetězec.
   *
   * @param integer $type  Typ obrázku
   * @param integer $quality  Kvalita (0 - 100: pro JPEG a PNG)
   * @return string
   */
  public function toString($type = IMAGETYPE_JPEG, $quality = null) {
    ob_start();
     $this->save(null, $quality, $type);
    return ob_get_clean();
  }

  /**
   * Vrátí obrázek jako řetězec.
   *
   * @return string
   */
  public function __toString() {
    try {
      return $this->toString();
    } catch (Exception $e) {
      throw $e;
    }
  }

  /**
   * Odešle obrázek do prohlížeče.
   *
   * @param integer $type  IMAGETYPE obrázku
   * @param integer $quality  Kvalita (0 - 100: pro JPEG a PNG)
   * @return void
   * @throws Custom_Exception
   */
  public function send($type = IMAGETYPE_JPEG, $quality = null) {
    $types = array(IMAGETYPE_JPEG, IMAGETYPE_PNG, IMAGETYPE_GIF, IMAGETYPE_BMP);
    if (!in_array($type, $types)) {
      require_once 'Custom/Exception.php';
      throw new Custom_Exception('Nepodporovaný typ obrázku.');
    }

    $output = $this->toString($type, $quality);
    Custom_Tools::testModify(md5($output));

    header('Content-Type: '.image_type_to_mime_type($type));
    echo $output;
  }

  /**
   * Provolá nedefinovanou metodu
   *
   * @param string $name
   * @param array $args
   * @return mixed
   */
  public function __call($name, $args) {
    $function = 'image' . $name;
    if (!function_exists($function)) {
      throw new Custom_Exception("Funkce „{$function}“ neexistuje.");
    }

    foreach ($args as $key => $value) {
      if ($value instanceof self) {
        $args[$key] = $value->getImageResource();
      }
    }
    array_unshift($args, $this->_image);

    $res = call_user_func_array($function, $args);
    return is_resource($res) && get_resource_type($res) === 'gd'
           ? $this->setImageResource($res)
           : $res;
  }

}
