<?php

/**
 * Třída pro práci s tabulkou webbia_logins
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2013
 */
class System_Model_Logins extends Zend_Db_Table_Abstract {

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_logins';

  /**
   * Sloupec primárního klíče
   *
   * @var string
   */
  protected $_primary = 'id_login';

  /**
   * Asociativní pole pravidel referenční integrity
   *
   * @var array
   */
  protected $_referenceMap = array(
    'User' => array(
      'columns'       => array('id_user'),
      'refTableClass' => 'System_Model_Users',
      'refColumns'    => array('id_user')
    )
  );

  /**
   * Vrátí uživatele dle $id_user
   *
   * @param integer $id_user
   * @return System_Model_User
   */
  protected function _getUserById($id_user) {
    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $key = md5('System_Model_Users::getUserById'.$id_user);

    if (!is_object($cache) || (($result = $cache->load($key)) === false)) {
      $modelUsers = new System_Model_Users();
      $result = $modelUsers->find($id_user)->current();
      if (is_object($cache)) {
        $cache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Trvale zaloguje uživatele
   *
   * @param integer $id_user
   * @param string $duration
   * @return boolean
   */
  public static function login($id_user, $duration = '+1 year') {
    $data = array(
      'token'      => md5(uniqid(mt_rand(), true)),
      'id_user'    => $id_user,
      'date_login' => date('Y-m-d H:i:s')
    );

    $self = new self();
    if ($self->createRow($data)->save()) {
      array_pop($data);
      setcookie('rememberme', implode(':', $data), strtotime($duration), '/');
      return true;
    }

    return false;
  }

  /**
   * Přihlásí uživatele pomocí trvalého přihlášení
   *
   * @return boolean
   */
  public static function autolog() {
    $auth = Zend_Auth::getInstance();
    if ($auth->hasIdentity()) {
      return true;
    }

    $request = new Zend_Controller_Request_Http();
    $rememberMe = $request->getCookie('rememberme');
    if (null === $rememberMe) {
      return false;
    }

    $where = array();
    list($where['token = ?'], $where['id_user = ?']) = explode(':', $rememberMe);

    $self = new self();
    if (is_object($login = $self->fetchRow($where))) {
      $auth->clearIdentity();
      $auth->getStorage()->write($self->_getUserById($login->id_user));
    }

    return $auth->hasIdentity();
  }

  /**
   * Odhlásí uživatele a zruší trvalé přihlášení
   *
   * @return boolean
   */
  public static function logout() {
    $request = new Zend_Controller_Request_Http();
    $rememberMe = $request->getCookie('rememberme');
    if (null === $rememberMe) {
      return true;
    }

    $where = array();
    list($where['token = ?'], $where['id_user = ?']) = explode(':', $rememberMe);

    $self = new self();
    if (is_object($login = $self->fetchRow($where))) {
      $login->delete();
      setcookie('rememberme', '', 1);
      return true;
    }

    return false;
  }

  /**
   * Vrátí TRUE, pokud je zvoleno trvalé přihlášení; jinak vrátí FALSE
   *
   * @return boolean
   */
  public static function isRemembered() {
    $request = new Zend_Controller_Request_Http();
    $rememberMe = $request->getCookie('rememberme');
    if (null === $rememberMe) {
      return false;
    }

    $where = array();
    list($where['token = ?'], $where['id_user = ?']) = explode(':', $rememberMe);

    $self = new self();
    return is_object($self->fetchRow($where));
  }

}
