<?php

/**
 * Třída statických funkcí pro běh OTP
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, ČZU © 2017
 *
 * Copyright © 2017 Bc. Petr Jajtner, ČZU Praha
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
class System_Model_OTP {

  /**
   * Výchozí algoritmus OTP
   *
   * @var string
   */
  public static $ALGORITHM = 'sha1';

  /**
   * Výchozí počet číslic
   *
   * @var integer
   */
  public static $DIGITS = 6;

  /**
   * Výchozí interval změny TOTP
   *
   * @var integer
   */
  public static $PERIOD = 30;

  /**
   * Výčet znaků Base32 dle RFC3548
   *
   * @var string
   */
  protected static $_chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ234567';

  /**
   * Výplň dle RFC3548
   *
   * @var string
   */
  protected static $_pad = '=';

  /**
   * „Sůl“ pro kryptování
   * POZOR! NUTNO ZMĚNIT DLE VLASTNÍCH PREFERENCÍ
   *
   * @var string
   */
  protected static $_salt = '!0-123-456-789#';

  /**
   * Metoda šifrování pro ukládání tajemství do databáze
   * Možno změnit dle vlastních preferencí
   *
   * @var string
   */
  protected static $_cipher = 'AES-256-CTR';

  /**
   * Zakóduje data pomocí Base32
   *
   * @param mixed $data
   * @return string
   */
  public static function Base32Encode($data) {
    if (!$data) {
      return '';
    }

    $bytes = array_map('ord', str_split($data));
    $result = '';

    $padding = count($bytes) % 5;
    $length = count($bytes) - $padding;

    for ($i = 0; $i < $length; $i += 5) {
      $b1 = $bytes[$i] & 0xff;
      $b2 = $bytes[$i + 1] & 0xff;
      $b3 = $bytes[$i + 2] & 0xff;
      $b4 = $bytes[$i + 3] & 0xff;
      $b5 = $bytes[$i + 4] & 0xff;

      $result .= self::$_chars[($b1 >> 3) & 0x1f];
      $result .= self::$_chars[(($b1 << 2) | ($b2 >> 6)) & 0x1f];
      $result .= self::$_chars[($b2 >> 1) & 0x1f];
      $result .= self::$_chars[(($b2 << 4) | ($b3 >> 4)) & 0x1f];
      $result .= self::$_chars[(($b3 << 1) | ($b4 >> 7)) & 0x1f];
      $result .= self::$_chars[($b4 >> 2) & 0x1f];
      $result .= self::$_chars[(($b4 << 3) | ($b5 >> 5)) & 0x1f];
      $result .= self::$_chars[$b5 & 0x1f];
    }
    if (1 == $padding) {
      $b1 = $bytes[$length] & 0xff;
      $result .= self::$_chars[($b1 >> 3) & 0x1f];
      $result .= self::$_chars[($b1 << 2) & 0x1f];
      $result .= self::$_pad.self::$_pad.self::$_pad.self::$_pad.self::$_pad.self::$_pad;
    } elseif (2 == $padding) {
      $b1 = $bytes[$length] & 0xff;
      $b2 = $bytes[$length + 1] & 0xff;
      $result .= self::$_chars[($b1 >> 3) & 0x1f];
      $result .= self::$_chars[(($b1 << 2) | ($b2 >> 6)) & 0x1f];
      $result .= self::$_chars[($b2 >> 1) & 0x1f];
      $result .= self::$_chars[($b2 << 4) & 0x1f];
      $result .= self::$_pad.self::$_pad.self::$_pad.self::$_pad;
    } elseif (3 == $padding) {
      $b1 = $bytes[$length] & 0xff;
      $b2 = $bytes[$length + 1] & 0xff;
      $b3 = $bytes[$length + 2] & 0xff;
      $result .= self::$_chars[($b1 >> 3) & 0x1f];
      $result .= self::$_chars[(($b1 << 2) | ($b2 >> 6)) & 0x1f];
      $result .= self::$_chars[($b2 >> 1) & 0x1f];
      $result .= self::$_chars[(($b2 << 4) | ($b3 >> 4)) & 0x1f];
      $result .= self::$_chars[($b3 << 1) & 0x1f];
      $result .= self::$_pad.self::$_pad.self::$_pad;
    } elseif (4 == $padding) {
      $b1 = $bytes[$length] & 0xff;
      $b2 = $bytes[$length + 1] & 0xff;
      $b3 = $bytes[$length + 2] & 0xff;
      $b4 = $bytes[$length + 3] & 0xff;
      $result .= self::$_chars[($b1 >> 3) & 0x1f];
      $result .= self::$_chars[(($b1 << 2) | ($b2 >> 6)) & 0x1f];
      $result .= self::$_chars[($b2 >> 1) & 0x1f];
      $result .= self::$_chars[(($b2 << 4) | ($b3 >> 4)) & 0x1f];
      $result .= self::$_chars[(($b3 << 1) | ($b4 >> 7)) & 0x1f];
      $result .= self::$_chars[($b4 >> 2) & 0x1f];
      $result .= self::$_chars[($b4 << 3) & 0x1f];
      $result .= self::$_pad;
    }

    return $result;
  }

  /**
   * Dekóduje data pomocí Base32
   *
   * @param string $data
   * @return mixed
   * @throws Exception
   */
  public static function Base32Decode($data) {
    if (!$data) {
      return '';
    }

    $ignore = array("\n", "\r", "\t", " ", "-");
    $length = strlen($data = strtoupper($data));
    $current = $padding = $i = 0;

    $result = array();
    while ($length-- > 0) {
      $char = $data[$current];
      if (self::$_pad == $char) {
        ++$padding;
        continue;
      }
      if (in_array($char, $ignore)) {
        continue;
      }

      $index = strpos(self::$_chars, $char);
      if (false === $index) {
        return null;
      }

      switch ($current % 8) {
        case 0:
          $result[$i] = $index << 3;
          break;
        case 1:
          $result[$i++] |= $index >> 2;
          $result[$i] = ($index & 0x03) << 6;
          break;
        case 2:
          $result[$i] |= $index << 1;
          break;
        case 3:
          $result[$i++] |= $index >> 4;
          $result[$i] = ($index & 0x0f) << 4;
          break;
        case 4:
          $result[$i++] |= $index >> 1;
          $result[$i] = ($index & 0x01) << 7;
          break;
        case 5:
          $result[$i] |= $index << 2;
          break;
        case 6:
          $result[$i++] |= $index >> 3;
          $result[$i] = ($index & 0x07) << 5;
          break;
        case 7:
          $result[$i++] |= $index;
          break;
      }

      ++$current;
    }

    return implode('', array_map('chr', array_slice($result, 0, $i)));
  }

  /**
   * Nepříliš bezpečné zakódování
   *
   * @param string $string
   * @param string $key
   * @return string
   */
  public static function UnsafeEncrypt($string, $key) {
    $password = hash('sha256', self::$_salt.$key);
    $iv = openssl_random_pseudo_bytes(16);
    $options = OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING;
    $result = openssl_encrypt($string, self::$_cipher, $password, $options, $iv);
    return (string) base64_encode($iv.$result);
  }

  /**
   * Nepříliš bezpečné dekódování
   *
   * @param string $string
   * @param string $key
   * @return string
   */
  public static function UnsafeDecrypt($string, $key) {
    $password = hash('sha256', self::$_salt.$key);
    $encrypted = base64_decode($string);
    $options = OPENSSL_RAW_DATA | OPENSSL_ZERO_PADDING;
    $iv = substr($encrypted, 0, 16);
    return (string) openssl_decrypt(substr($encrypted, 16), self::$_cipher, $password, $options, $iv);
  }

  /**
   * Vygeneruje heslo
   *
   * @param integer $length
   * @param boolean $punctuation
   * @return string
   */
  public static function GeneratePassword($length = 10, $punctuation = true) {
    $validChars = '23456789abcdfghjkmnpqrstvwxyzABCDEFGHJKLMNPQRSTUVWXYZ';
    if ($punctuation) {
      $validChars .= '+-*.:;%^~=|!_';
    }

    $counter = 0;
    $max = strlen($validChars) - 1;

    $result = '';
    while ($counter++ < $length) {
      $result .= $validChars[rand(0, $max)];
    }

    return $result;
  }

  /**
   * Vygeneruje hmac-based OTP
   *
   * @param string $secret
   * @param integer $counter
   * @return string
   * @throws Exception
   */
  public static function GenerateHOTP($secret, $counter) {
    if (!in_array($algo = strtolower(self::$ALGORITHM), array('sha1', 'sha256', 'sha512', 'md5'))) {
      throw new UnexpectedValueException("Algoritmus „{$algo}“ není povolen.");
    }
    if (!is_numeric($digits = self::$DIGITS) || !in_array($digits, array(6, 8))) {
      throw new UnexpectedValueException("Délka OTP musí být 6 nebo 8 znaků.");
    }

    $binaryCounter = pack('J*', $counter);
    $hash = hash_hmac($algo, $binaryCounter, $secret, true);

    $offset = ord($hash[19]) & 0xf;
    $result = (
      ((ord($hash[$offset + 0]) & 0x7f) << 24 ) |
      ((ord($hash[$offset + 1]) & 0xff) << 16 ) |
      ((ord($hash[$offset + 2]) & 0xff) <<  8 ) |
       (ord($hash[$offset + 3]) & 0xff)
    ) % pow(10, $digits);

    return str_pad($result, $digits, '0', STR_PAD_LEFT);
  }

  /**
   * Vygeneruje time-based OTP
   *
   * @param string $secret
   * @return integer
   */
  public static function GenerateTOTP($secret) {
    if (!is_numeric($period = self::$PERIOD) || 0 >= $period) {
      throw new UnexpectedValueException("Perioda „{$period}“ nesmí být menší nebo rovna 0.");
    }
    return self::GenerateHOTP($secret, floor(time() / $period));
  }

  /**
   * Ověří HOTP, případně zkoriguje
   *
   * @param string $password
   * @param string $secret
   * @param integer $counter
   * @param integer $cntwin
   * @return boolean
   * @throws Exception
   */
  public static function VerifyHOTP($password, $secret, &$counter, $cntwin = 3) {
    if (!is_numeric($counter) || 0 > $counter) {
      throw new InvalidArgumentException('Neplatný čítač HOTP.');
    }
    if (!is_numeric($cntwin) || 0 > $cntwin) {
      throw new InvalidArgumentException('Neplatné okno pokusů čítače.');
    }
    if (self::SafeCompare($password, self::GenerateHOTP($secret, $counter))) {
      return true;
    }

    $attempts = $counter;
    while (++$attempts < $counter + $cntwin) {
      if (self::SafeCompare($password, self::GenerateHOTP($secret, $attempts))) {
        $counter = $attempts;
        return true;
      }
    }
    return false;
  }

  /**
   * Ověří TOTP
   *
   * @param string $password
   * @param string $secret
   * @param integer $timewin
   * @return boolean
   * @throws Exception
   */
  public static function VerifyTOTP($password, $secret, $timewin = 1) {
    if (!is_numeric($timewin) || 0 > $timewin) {
      throw new InvalidArgumentException('Neplatné okno času.');
    }
    if (self::SafeCompare($password, self::GenerateTOTP($secret))) {
      return true;
    }
    if (0 == $timewin) {
      return false; // není kam posouvat časové okno
    }

    $counter = floor(time() / self::$PERIOD);

    $attempts = $counter - $timewin;
    while (++$attempts < $counter + $timewin) {
      if ($attempts == $counter) {
        continue;
      }
      if (self::SafeCompare($password, self::GenerateHOTP($secret, $attempts))) {
        return true;
      }
    }
    return false;
  }

  /**
   * Bezpečně porovná dva vstupy
   *
   * @param mixed $a
   * @param mixed $b
   * @return boolean
   */
  public static function SafeCompare($a, $b) {
    $sha1A = sha1($a);
    $sha1B = sha1($b);

    $result = 0;
    for ($i = 0; $i < 40; $i++) {
      $result |= ord($sha1A[$i]) ^ ord($sha1B[$i]);
    }

    return 0 == $result;
  }

  /**
   * Sestaví Key URI dle https://github.com/google/google-authenticator/wiki/Key-Uri-Format
   *
   * @param string $type
   * @param string $label
   * @param string $secret
   * @param string $email
   * @param integer $counter
   * @return string
   * @throws Exception
   */
  public static function BuildKeyURI($type, $label, $secret, $email = null, $counter = 0) {
    if (!in_array($type = strtolower($type), array('hotp', 'totp'))) {
      throw new InvalidArgumentException('Typ musí být buď HOTP nebo TOTP.');
    }
    if (1 > strlen($label = trim($label))) {
      throw new InvalidArgumentException('Popisek musí být delší než jeden znak.');
    }
    if (1 > strlen($secret = trim($secret))) {
      throw new InvalidArgumentException('Není uvedeno tajemství.');
    }
    if ($type == 'hotp' && (!is_numeric($counter) || 0 > $counter)) {
      throw new InvalidArgumentException('Neplatný čítač HOTP.');
    }

    $result = "otpauth://{$type}/".rawurlencode(str_replace(':', '', $label))
            . ($email ? ':'.$email : '')
            . '?secret='.rawurlencode($secret)
            . '&issuer='. rawurlencode($label)
            . '&algorithm='.rawurlencode(self::$ALGORITHM);

    if ($type == 'hotp') {
      $result .= '&counter='.($counter + 1);
    }
    if ($type == 'totp') {
      $result .= '&period='.self::$PERIOD;
    }

    return $result;
  }

  /**
   * Sestaví QR kód dle Key URI
   *
   * @param string $type
   * @param string $label
   * @param string $secret
   * @param string $email
   * @param integer $counter
   * @return string
   * @throws Exception
   */
  public static function QRCodeInBase64($type, $label, $secret, $email = null, $counter = 0) {
    $otpauth = self::BuildKeyURI($type, $label, $secret, $counter);
    $url = 'https://chart.googleapis.com/chart?chs=135x135&cht=qr&chld=M|0&chl='
         . urlencode($otpauth);
    $result = base64_encode(file_get_contents($url));
    return "data:image/png;base64,{$result}";
  }

}
