<?php

/**
 * Třída pro práci se skupinou řádků tabulky webbia_objects
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_ObjectSet extends Zend_Db_Table_Rowset_Abstract {

  /**
   * Vrátí pole popisků objektů
   *
   * @return array  Pole popisků
   */
  public function getCaptions() {
    $result = array();
    foreach ($this as $row) {
      $altName = isset($row->alt_name) ? $row->alt_name : '';
      $result[$row->id_object] = $row->caption ? $row->caption : $altName;
    }
    return $result;
  }

  /**
   * Vrátí soubory k výsledkům
   *
   * @return System_Model_FileSet
   */
  public function getFileSet() {
    $IDs = array();
    foreach ($this as $row) {
      if (is_numeric($row->id_file)) {
        $IDs[] = $row->id_file;
      }
    }
    if (count($IDs)) {
      return System_Model_Files::getFiles($IDs);
    }
    return System_Model_Files::emptySet();
  }

  /**
   * Otestuje, zdali je iterator na začátku
   *
   * @return boolean
   */
  public function isFirst() {
    return (0 == $this->_pointer);
  }

  /**
   * Otestuje, zdali je iterator na konci
   *
   * @return boolean
   */
  public function isLast() {
    return ($this->_pointer == $this->_count - 1);
  }

  /**
   * Nastaví nebo vrátí aktuální pozici v rowsetu
   *
   * @param integer $set
   * @return integer
   */
  public function position($set = null) {
    if (is_numeric($set) && $this->_valid($set)) {
      $this->_pointer = $set;
      return;
    }
    return ($this->_count) ? 1 + $this->_pointer : 0;
  }

  /**
   * Vrátí předchozí prvek
   *
   * @return System_Model_Object
   */
  public function getPrevious() {
    $index = $this->_pointer - 1;
    if (!$this->_valid($index)) {
      return null;
    }
    return $this->_loadAndReturnRow($index);
  }

  /**
   * Vrátí následující prvek
   *
   * @return System_Model_Object
   */
  public function getNext() {
    $index = $this->_pointer + 1;
    if (!$this->_valid($index)) {
      return null;
    }
    return $this->_loadAndReturnRow($index);
  }

  /**
   * Přesunu ukazatel na předchozí prvek.
   *
   * @return void
   */
  public function previous() {
    --$this->_pointer;
  }

  /**
   * Najde v sadě objekt dle $by
   *
   * @param array $by
   * @return System_Model_Object
   */
  public function find($by) {
    foreach ($this as $row) {
      $match = true;
      foreach ($by as $key => $value) {
        $match = $match && ($row->{$key} == $value);
      }
      if ($match) {
        return $row;
      }
    }
    return null;
  }

  /**
   * Otestuje, zdali je $index v mezích
   *
   * @return boolean
   */
  protected function _valid($index) {
    return $index >= 0 && $index < $this->_count;
  }

}
