<?php

/**
 * Třída pro práci s tabulkou webbia_objects
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Objects extends Custom_Db_Table_Rewrite {

  const STYLE_TABLE  = 'table';
  const STYLE_THUMBS = 'thumbs';

  /**
   * Styly zobrazení
   *
   * @var array
   */
  public static $STYLES = array(
    self::STYLE_TABLE,
    self::STYLE_THUMBS
  );

  /**
   * Styly zobrazení
   *
   * @var array
   */
  public static $PER_PAGE = array(
    self::STYLE_TABLE  => 50,
    self::STYLE_THUMBS => 12
  );

  /**
   * Velikost ikon
   *
   * @var array
   */
  public static $ICON_SIZES = array(
    self::STYLE_TABLE  =>  16,
    self::STYLE_THUMBS => 128
  );

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_objects';

  /**
   * Sloupce primárního klíče
   *
   * @var array
   */
  protected $_primary = array('id_object', 'language');

  /**
   * Regionální nastavení
   *
   * @var Zend_Locale
   */
  protected $_locale = null;

  /**
   * Třída řádku
   *
   * @var string
   */
  protected $_rowClass = 'System_Model_Object';

  /**
   * Třída skupiny řádků
   *
   * @var string
   */
  protected $_rowsetClass = 'System_Model_ObjectSet';

  /**
   * Asociativní pole pravidel referenční integrity
   *
   * @var array
   */
  protected $_referenceMap = array(
    'Object' => array(
      'columns'       => array('id_parent', 'language'),
      'refTableClass' => __CLASS__,
      'refColumns'    => array('id_object', 'language')
    ),
    'Rewrite' => array(
      'columns'       => array('rewrite', 'language'),
      'refTableClass' => __CLASS__,
      'refColumns'    => array('rewrite', 'language')
    ),
    'File' => array(
      'columns'       => array('id_file'),
      'refTableClass' => 'System_Model_Files',
      'refColumns'    => array('id_file')
    )
  );

  /**
   * Pole názvů tříd závislých tabulek
   *
   * @var array
   */
  protected $_dependentTables = array(
    __CLASS__
  );

  /**
   * Inicializace objektu
   *
   * @return void
   */
  public function init() {
    parent::init();
    $this->_locale = Zend_Registry::get('Zend_Locale');
  }

  /**
   * Vrátí objekt dle $id_object a $language
   *
   * @param integer $id_object  ID objektu
   * @param string $language  Jazyk objektu
   * @return System_Model_Object
   */
  public function getObjectById($id_object, $language = null) {
    $_lang = (null === $language) ? $this->_locale->getLanguage() : $language;
    $key = md5(__METHOD__.$id_object.$language);
    if (!is_object($this->_memoryCache) || (($result = $this->_memoryCache->load($key)) === false)) {
      $result = $id_object ? $this->find($id_object, $_lang)->current() : null;
      if ($id_object && null === $result && null === $language) {
        $result = $this->fetchRow(array('id_object = ?' => $id_object));
      }
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí mutaci dle $id_object
   *
   * @param integer $id_object  ID objektu
   * @return System_Model_Object
   */
  public function getMutationById($id_object) {
    $key = md5(__METHOD__.$id_object);
    if (!is_object($this->_memoryCache) || (($result = $this->_memoryCache->load($key)) === false)) {
      $language = $this->getAdapter()->quote($this->_locale->getLanguage());
      $where = array('id_object = ?' => $id_object);
      $order = new Zend_Db_Expr("(language = {$language}) DESC");

      $result = $this->fetchRow($where, $order);
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí objekt dle $rewrite
   *
   * @param string $rewrite
   * @return System_Model_Object
   */
  public function getObjectByRewrite($rewrite) {
    $key = md5(__METHOD__.$rewrite);
    if (!is_object($this->_memoryCache) || (($result = $this->_memoryCache->load($key)) === false)) {
      $language = $this->getAdapter()->quote($this->_locale->getLanguage());
      $order = new Zend_Db_Expr("(language = {$language}) DESC");
      $result = $this->fetchRow(array('rewrite LIKE ?' => $rewrite), $order);
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí objekt dle kódu
   *
   * @param string $code
   * @return System_Model_Object
   */
  public function getObjectByCode($code) {
    $matches = array();
    $code = urldecode($code);
    return preg_match('~^\/?\[\[object id=(\d+)~ui', $code, $matches)
           ? $this->getObjectById($matches[1])
           : null;
  }

  /**
   * Vrátí první objekt
   *
   * @param string $language
   * @return System_Model_Object
   */
  public function getFirstObject($language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }

    $key = md5(__METHOD__.$language);
    if (!is_object($this->_memoryCache) || (($result = $this->_memoryCache->load($key)) === false)) {
      $result = $this->fetchRow(array('language = ?' => $language), 'object_order ASC');
      if (is_object($this->_memoryCache)) {
        $this->_memoryCache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí objekty dle pole $id_objects
   *
   * @param array $id_objects  Pole ID objektů
   * @return System_Model_ObjectSet
   */
  public function getObjects(array $id_objects, $language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }
    $where = array(
      'id_object IN (?)' => $id_objects,
      'language = ?'     => $language
    );
    return $this->fetchAll($where, 'object_order ASC');
  }

  /**
   * Vrátí složky správce souborů
   *
   * @param boolean $is_visible   Vracet viditelné (true), neviditelné (false)
   * @param string $language  Jazyk složek
   * @param boolean $count  Vrátit počet složek
   * @return System_Model_ObjectSet
   */
  public function getFolders($is_visible = null, $language = null, $count = false) {
    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $key = md5(__METHOD__.serialize((int) $is_visible . $language . (int) $count));

    if (!is_object($cache) || (($result = $cache->load($key)) === false)) {
      $filter = array('id_parent' => null, 'object_type' => System_Model_Object::SYSTEM_OBJECT_FOLDER);
      if (is_bool($is_visible)) {
        $filter['is_visible'] = $is_visible;
      }
      if (is_string($language)) {
        $filter['language'] = $language;
      }
      $result = $this->getObjectsByFilter($filter, null, null, $count);
      if (is_object($cache)) {
        $cache->save($result, $key);
      }
    }

    return $result;
  }

  /**
   * Vrátí objekty pro galerii
   *
   * @param integer $id_folder  ID složky galerie
   * @param boolean $is_visible  Pouze viditelné (výchozí TRUE)
   * @param string $language  Jazyk objektů
   * @param integer $limit  Omezení počtu souborů
   * @param integer $offset  Posunutí začátku souborů
   * @param boolean $count  Vrátit pouze počet (výchozí FALSE)
   * @return System_Model_ObjectSet
   */
  public function getMedia($id_folder, $is_visible = true, $language = null, $limit = null, $offset = null, $count = false) {
    $cache = Zend_Registry::isRegistered('Zend_Cache') ? Zend_Registry::get('Zend_Cache') : null;
    $key = md5(__METHOD__.serialize(func_get_args()));

    if (!is_object($cache) || (($result = $cache->load($key)) === false)) {
      $filter = array(
        'id_parent'   => $id_folder,
        'object_type' => System_Model_Object::SYSTEM_OBJECT_FILE,
        'file_type'   => array_merge(System_Model_FileType::$images, System_Model_FileType::$audio, System_Model_FileType::$video)
      );
      if (is_bool($is_visible)) {
        $filter['is_visible'] = $is_visible;
      }
      if (is_string($language)) {
        $filter['language'] = $language;
      }
      $result = $this->getObjectsByFilter($filter, $limit, $offset, $count);
      if (is_object($cache)) {
        $cache->save($result, $key, array('MediaCache'));
      }
    }

    return $result;
  }

  /**
   * Vrátí seznam objektů dle filtru
   *
   * @param array $filter  Kritéria pro omezení
   * @param boolean $count  Vrátit počet záznamů
   * @return System_Model_ObjectSet|integer
   */
  public function getObjectsByFilter(array $filter = null, $limit = null, $offset = null, $count = false) {
    $select = $this->_select($count);
    $select->limit($limit, $offset);

    if (isset($filter['id_object']) && is_numeric($filter['id_object'])) {
      $select->where('m.id_object = ?', $filter['id_object']);
    }
    if (isset($filter['id_object']) && is_array($filter['id_object'])) {
      $select->where('m.id_object IN (?)', $filter['id_object']);
    }
    if (array_key_exists('id_parent', $filter)) {
      $isset = is_numeric($filter['id_parent']) && !empty($filter['id_parent']);
      $select->where($isset ? 'm.id_parent = ?' : 'm.id_parent IS NULL', $filter['id_parent']);
    }
    if (isset($filter['object_type']) && is_string($filter['object_type'])) {
      $select->where('m.object_type = ?', $filter['object_type']);
    }
    if (isset($filter['file_name']) && is_string($filter['file_name'])) {
      $name = $this->getAdapter()->quote($filter['file_name'].'%');
      $select->where("((o.caption IS NULL AND b.alt_name LIKE {$name}) OR o.caption LIKE {$name})");
    }
    if (isset($filter['file_type']) && is_string($filter['file_type'])) {
      $folder = $this->getAdapter()->quote('folder');
      $select->where("(b.alt_type = {$folder} OR b.alt_type = ?)", $filter['file_type']);
    }
    if (isset($filter['file_type']) && is_array($filter['file_type'])) {
      $select->where("b.alt_type IN (?)", $filter['file_type']);
    }
    if (isset($filter['language']) && is_string($filter['language'])) {
      $select->where('o.language = ?', $filter['language']);
    }
    if (isset($filter['rewrite']) && is_string($filter['rewrite'])) {
      $select->where('o.rewrite LIKE ?', $filter['rewrite']);
    }
    if (isset($filter['is_visible']) && is_bool($filter['is_visible'])) {
      $select->where('o.is_visible = ?', (int) $filter['is_visible']);
    }
    if (isset($filter['is_gallery']) && is_bool($filter['is_gallery'])) {
      $select->where('o.is_gallery = ?', (int) $filter['is_gallery']);
    }

    return $count ? (int) $this->getAdapter()->fetchOne($select) : $this->fetchAll($select);
  }

  /**
   * Vrátí počet záznamů v tabulce seskupené dle id_object
   *
   * @return integer
   */
  public function getCount() {
    $sql = "SELECT COUNT(*) FROM {$this->info('name')} GROUP BY id_object";
    return (int) $this->getAdapter()->fetchOne($sql);
  }

  /**
   * Nastaví pořadí objektů
   *
   * @param array $newOrder
   * @return void
   */
  public function reorder(array $newOrder) {
    if (is_array($newOrder) && !empty($newOrder)) {
      $tableName = $this->info(self::NAME);
      foreach ($newOrder as $order => $id) {
        $priority = $order + 1; // Indexováno od nuly
        $sql = "UPDATE {$tableName} SET object_order = "
             . "{$priority} WHERE id_object = {$id}";
        $this->getAdapter()->query($sql);
      }
    }

    $this->cleanMediaCache();
  }

  /**
   * Změní pořadí objektů
   *
   * @param integer $id_object
   * @param string $direction
   * @return void
   */
  public function move($id_object, $direction = 'up') {
    $tableName = $this->info(self::NAME);
    $operator = ('up' == $direction) ? '<' : '>';
    $dir = ('up' == $direction) ? 'DESC' : 'ASC';

    $fromObject = $this->getObjectById($id_object);

    $where = array("object_order {$operator} ?" => $fromObject->object_order);
    $order = "object_order {$dir}";

    $toObject = $this->fetchRow($where, $order);

    $sql1 = "UPDATE {$tableName} "
          . "SET object_order = {$fromObject->object_order} "
          . "WHERE id_object = {$toObject->id_object}";

    $sql2 = "UPDATE {$tableName} "
          . "SET object_order = {$toObject->object_order} "
          . "WHERE id_object = {$fromObject->id_object}";

    $this->getAdapter()->query($sql1);
    $this->getAdapter()->query($sql2);

    $this->cleanMediaCache();
  }

  /**
   * Vymaže MediaCache
   *
   * @return void
   */
  public function cleanMediaCache() {
    $cache = Zend_Registry::isRegistered('Zend_Cache') ? Zend_Registry::get('Zend_Cache') : null;
    if (is_object($cache)) {
      $cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG, array('MediaCache'));
    }
  }

  /**
   * Vrátí objekt na základě identifikátoru
   *
   * @param mixed $identifier
   * @param string $language
   * @return System_Model_Object
   */
  public static function getObject($identifier, $language = null) {
    $self = new self();
    return is_numeric($identifier)
           ? $self->getObjectById($identifier, $language)
           : $self->getObjectByRewrite($identifier);
  }

  /**
   * Vrátí subselect mutací článků
   *
   * @return array
   */
  private function _mutations() {
    $mutations = $this->select(self::SELECT_WITH_FROM_PART);
    $mutations->reset(self::COLUMNS)->columns(array(
      'id_object',
      'id_parent',
      'object_type',
      'object_order',
      'GROUP_CONCAT(language) AS mutations'
    ));
    $mutations->group('id_object');
    return array('m' => new Zend_Db_Expr('('.$mutations.')'));
  }

  /**
   * Vrátí subselect alternativních názvů objektů
   *
   * @param string $language
   * @return array
   */
  private function _alternatives($language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }
    $quoted = $this->getAdapter()->quote($language);

    $modelFiles = new System_Model_Files();

    $tableFiles = $modelFiles->info(self::NAME);
    $tableObjects = $this->info(self::NAME);

    $alternative = $this->select(self::SELECT_WITHOUT_FROM_PART);
    $alternative->setIntegrityCheck(false);
    $alternative->from(array('o' => $tableObjects), array(
     'id_object', 'caption AS alt_name'
    ));
    $alternative->joinLeft(array('f' => $tableFiles), 'o.id_file = f.id_file', array(
      'id_file AS alt_file', "IF(file_type IS NULL, 'folder', file_type) AS alt_type", 'file_key'
    ));
    $alternative->group('id_object');
    $alternative->order(new Zend_Db_Expr("(language = {$quoted}) DESC"));

    return array('b' => new Zend_Db_Expr('('.$alternative.')'));
  }

  /**
   * Vrátí subselect objektů odpovídající jazyku $language
   *
   * @param string $language
   * @return array
   */
  private function _objects($language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }

    $objects = $this->select(self::SELECT_WITH_FROM_PART);
    $objects->where('language = ?', $language);

    return array('o' => new Zend_Db_Expr('('.$objects.')'));
  }

  /**
   * Vrátí subselect velikostí objektů
   *
   * @param string $language
   * @return array
   */
  private function _sizes($language = null) {
    if (null === $language) {
      $language = $this->_locale->getLanguage();
    }
    $modelFiles = new System_Model_Files();

    $tableFiles = $modelFiles->info(self::NAME);
    $tableObjects = $this->info(self::NAME);

    $folderSizes = $this->select(self::SELECT_WITHOUT_FROM_PART);
    $folderSizes->setIntegrityCheck(false);
    $folderSizes->from(array('o' => $tableObjects), array('id_parent', 'language'));
    $folderSizes->joinInner(array('f' => $tableFiles), 'o.id_file = f.id_file', 'SUM(file_size) AS size');
    $folderSizes->group(array('id_parent', 'language'));
    $folderSizes->where('o.language = ?', $language);
    $folderSizes = array('s' => new Zend_Db_Expr('('.$folderSizes.')'));

    $selectFolders = $this->select(self::SELECT_WITHOUT_FROM_PART);
    $selectFolders->setIntegrityCheck(false);
    $selectFolders->from(array('o' => $tableObjects), array(
      'id_object', 'id_parent', 'id_file', 'language', 'object_type AS type',
      'IF(size IS NULL, 0, size) AS size'
    ));
    $selectFolders->joinLeft($folderSizes, 'o.id_object = s.id_parent', '');
    $selectFolders->where('o.object_type = ?', System_Model_Object::SYSTEM_OBJECT_FOLDER);
    $selectFolders->where('o.language = ?', $language);

    $selectFiles = $this->select(self::SELECT_WITHOUT_FROM_PART);
    $selectFiles->setIntegrityCheck(false);
    $selectFiles->from(array('o' => $tableObjects), array(
      'id_object', 'id_parent', 'id_file', 'language'
    ));
    $selectFiles->joinInner(array('f' => $tableFiles), 'o.id_file = f.id_file', array(
      'file_type AS type', 'file_size AS size'
    ));
    $selectFiles->where('o.object_type = ?', System_Model_Object::SYSTEM_OBJECT_FILE);
    $selectFiles->where('o.language = ?', $language);

    $sizes = $this->select(self::SELECT_WITHOUT_FROM_PART);
    $sizes->setIntegrityCheck(false);
    $sizes->union(array("({$selectFolders})", "({$selectFiles})"));

    return array('s' => new Zend_Db_Expr('('.$sizes.')'));
  }

  /**
   * Vrátí hlavní select fitrovací funkce
   *
   * @param boolean $count  Vrátit pouze počet
   * @return Zend_Db_Table_Select
   */
  private function _select($count = false) {
    $flipped = array_flip($this->info(self::COLS));
    unset($flipped['id_object'], $flipped['object_type'], $flipped['object_order']);
    $columns = array_keys($flipped);

    $select = $this->select(self::SELECT_WITHOUT_FROM_PART);
    $select->setIntegrityCheck(false);
    $select->from($this->_mutations());
    $select->joinInner($this->_alternatives(), 'b.id_object = m.id_object', array(
      'alt_file', 'alt_name', 'alt_type', 'file_key'
    ));
    $select->joinLeft($this->_objects(), 'o.id_object = m.id_object', $columns);

    if ($count) {
      $select->reset(self::COLUMNS)->columns('COUNT(*) AS cnt');
    } else {
      $select->joinLeft($this->_sizes(), 'm.id_object = s.id_object', array('size', 'type'));
      $select->order('object_order ASC');
    }

    return $select;
  }

}
