<?php

/**
 * Třída pro práci s tabulkou webbia_rewrites
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Rewrites extends Zend_Db_Table_Abstract {

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_rewrites';

  /**
   * Sloupce primárního klíče
   *
   * @var array
   */
  protected $_primary = array('rewrite', 'language');

  /**
   * Třída řádku
   *
   * @var string
   */
  protected $_rowClass = 'System_Model_Rewrite';

  /**
   * Asociativní pole pravidel referenční integrity
   *
   * @var array
   */
  protected $_referenceMap = array(
    'Object' => array(
      'columns'       => array('rewrite', 'language'),
      'refTableClass' => 'System_Model_Objects',
      'refColumns'    => array('rewrite', 'language'),
      'onDelete'      => self::CASCADE
    )
  );

  /**
   * Vrátí objekt rewritu dle $rewrite
   *
   * @param string $rewrite
   * @param string $language
   * @return System_Model_Rewrite
   */
  public function getRewriteById($rewrite, $language = '') {
    $where = array('rewrite LIKE ?' => $rewrite);
    if ($language) {
      $where['language LIKE ?'] = $language;
    }
    return $this->fetchRow($where);
  }

  /**
   * Vrátí true pokud je $rewrite již uložený v DB, jinak false
   *
   * @param string $rewrite
   * @param string $language
   * @return boolean
   */
  public function hasRewrite($rewrite, $language = '') {
    $adapter = $this->getAdapter();
    $rewrite = $adapter->quote($rewrite);

    $sql = "SELECT COUNT(*) FROM {$this->_name} WHERE rewrite LIKE {$rewrite}";
    if ( $language) {
      $language = $adapter->quote($language);
      $sql .= " AND language LIKE {$language}";
    }

    return (bool) (int) $adapter->fetchOne($sql);
  }

  /**
   * Vrátí nový jedinečný rewrite na základě parametru $caption a databáze
   *
   * @param string $caption
   * @param string $language
   * @return string
   */
  public function getRewrite($caption, $language = '') {
    $counter = 0;
    $rewrite = $caption;

    $names = Custom_Tools::reservedNames();
    while ($this->hasRewrite($result = Custom_Tools::getRewrite($rewrite), $language) || in_array($result, $names)) {
      $rewrite = $caption.'-'.++$counter;
    }

    return $result;
  }

  /**
   * Vrátí objekt rewritu dle zadaného $rewrite
   *
   * @param string $rewrite Požadovaný rewrite
   * @return System_Model_Rewrite
   */
  public static function get($rewrite, $language = '') {
    $self = new self();
    return $self->getRewriteById($rewrite, $language);
  }

}
