<?php

/**
 * Třída objektu nastavení
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Setting extends Zend_Db_Table_Row_Abstract {

  /**
   * Název třídy modelu databáze
   *
   * @var string
   */
  protected $_tableClass = 'System_Model_Settings';

  /**
   * Vrátí hodnotu sloupce
   *
   * @param  string $columnName  Uživatelem zadaný název sloupce.
   * @return mixed               Odpovídající hodnota sloupce.
   * @throws Zend_Db_Table_Row_Exception v případě, že sloupec $columnName neexistuje
   */
  public function __get($columnName) {
    $result = parent::__get($columnName);

    if ($columnName == 'setting_value') {
      switch ($this->_data['setting_type']) {
        case 'object':
          $result = (object) unserialize($this->_data['setting_value']);
          break;
        case 'array':
          $result = (array) unserialize($this->_data['setting_value']);
          break;
        case 'real':
        case 'float':
        case 'double':
          $result = (float) $this->_data['setting_value'];
          break;
        case 'int':
        case 'integer':
          $result = (int) $this->_data['setting_value'];
          break;
        case 'bool':
        case 'boolean':
          $result = (bool) $this->_data['setting_value'];
          break;
        case 'string':
          $result = (string) $this->_data['setting_value'];
          break;
        default:
          $result = $this->_data['setting_value'];
      }
    }

    return $result;
  }

  /**
   * Nastaví hodnotu sloupce
   *
   * @param  string $columnName Uživatelem zadaný název sloupce.
   * @param  mixed  $value      Hodnota sloupce.
   * @return void
   * @throws Zend_Db_Table_Row_Exception
   */
  public function __set($columnName, $value) {
    if ($columnName == 'setting_value') {
      $locale = Zend_Registry::get('Zend_Locale');
      $options = array('locale' => $locale);

      if (is_object($value)) {
        $type = 'object';
        $value = serialize($value);
      } elseif (is_array($value)) {
        $type = 'array';
        $value = serialize($value);
      } elseif (Zend_Locale_Format::isNumber($value, $options)) {
       if (Zend_Locale_Format::isFloat($value, $options)) {
         $type = 'float';
       }
       if (Zend_Locale_Format::isInteger($value, $options)) {
         $type = 'int';
       }
      } elseif (is_bool($value)) {
         $type = 'bool';
      } elseif (is_string($value)) {
         $type = 'string';
      } else {
        $type = 'unknown';
      }

      parent::__set('setting_type', $type);
    }

    parent::__set($columnName, $value);
  }

}
