<?php

/**
 * Třída pro práci s nastavením systému
 *
 * @package    system
 * @author     Bc. Petr Jajtner <petr.jajtner@webbia.cz>
 * @copyright  Bc. Petr Jajtner, Webbia Standard © 2012
 */
class System_Model_Settings extends Zend_Db_Table_Abstract {

  /**
   * Název tabulky
   *
   * @var string
   */
  protected $_name = 'webbia_settings';

  /**
   * Sloupec primárního klíče
   *
   * @var string
   */
  protected $_primary = 'id_setting';

  /**
   * Třída řádku
   *
   * @var string
   */
  protected $_rowClass = 'System_Model_Setting';

  /**
   * Vrátí hodnotu klíče
   *
   * @param string $key      Klíč
   * $param mixed  $default  Výchozí hodnota
   * @return mixed
   */
  public static function getValue($key, $default = null) {
    $self = new self();

    if (is_array($values = $self->getValues()) &&
        is_string($key) && array_key_exists($key, $values)) {
      return $values[$key];
    }

    return $default;
  }

  /**
   * Načte konfigurační hodnoty z databáze
   *
   * @return array
   */
  public function getValues() {
    $cache = Zend_Registry::isRegistered('Custom_Cache') ? Zend_Registry::get('Custom_Cache') : null;
    $key = md5(__METHOD__);

    if (!is_object($cache) || (($rows = $cache->load($key)) === false)) {
      $rows = $this->fetchAll();
      if (is_object($cache)) {
        $cache->save($rows, $key);
      }
    }

    foreach ($rows as $row) {
      $data[$row->setting_name] = $row->setting_value;
    }

    return $data;
  }

}
