if ('undefined' === typeof toHHmmss) {

  /**
   * Převede číslo (udávající počet sekund) na čas
   *
   * @param {Number} number  Počet sekund
   * @returns {String}  Ve formátu času
   */
  var toHHmmss = function(number) {
    var hours   = Math.floor(number / 3600);
    var minutes = Math.floor((number - (hours * 3600)) / 60);
    var seconds = Math.round(number - (hours * 3600) - (minutes * 60));

    if (hours < 10) {
      hours = '0' + hours;
    }
    if (minutes < 10) {
      minutes = '0' + minutes;
    }
    if (seconds < 10) {
      seconds = '0' + seconds;
    }

    return hours + ':' + minutes + ':' + seconds;
  };
}

if ('undefined' === typeof String.prototype.toHHmmss) {

  /**
   * Prototyp stringu s převodem čísla (udávající počet sekund) na čas
   *
   * @returns {String}  Ve formátu času
   */
  String.prototype.toHHmmss = function() {
    var number = parseInt(this, 10);
    if (isNaN(number)) {
      return this;
    }
    return toHHmmss(number);
  };

}

if ('undefined' === typeof Number.prototype.toHHmmss) {

  /**
   * Prototyp čísla (udávající počet sekund) s převodem na čas
   *
   * @returns {String}  Ve formátu času
   */
  Number.prototype.toHHmmss = function() {
    return toHHmmss(this);
  };

}

if ('undefined' === typeof Webbia) {

  var INCOMING_LOGOUT = 60000; // Informace o času, za který dojde k odhlášení v ms (1 min.)

  /**
   * Objekt Webbia zastřešující různé metody systému
   *
   * @type Webbia
   */
  var Webbia = {

    /**
     * Čas, za který dojde k odhálšení uživatele
     *
     * @type Number
     */
    logoutTime: 0,

    /**
     * Text varování pro blížící se odhlášení
     *
     * @type String
     */
    incomingLogoutWarning: 'POZOR! Blíží se vaše odhlášení ze systému!',

    /**
     * Inline přihlašovací formulář
     *
     * @type Object
     */
    inlineLogin: {

      /**
       * Rám pro přihlašovací formulář
       *
       * @type jQuery
       */
      frame: null,

      /**
       * Zobrazí rám s přihlášením
       *
       * @returns {Void}
       */
      show: function() {
        Webbia.inlineLogin.frame = $('<iframe>').attr('src', '/login/inline');
        $('#overlay').removeClass('hidden').append(Webbia.inlineLogin.frame);
      },

      /**
       * Skryje rám s přihlášením
       *
       * @param {Boolean} logged
       * @returns {Void}
       */
      hide: function(logged) {
        logged = logged || false;

        Webbia.inlineLogin.frame && Webbia.inlineLogin.frame.remove();
        $('#overlay').addClass('hidden');

        if (logged && Webbia.logoutTime) {
          Webbia.countDown('#logout-counter', Webbia.logout);
        } else {
          location.href = '/admin';
        }
      }
    },

    /**
     * Objekt kontextového menu
     *
     * @type contextMenu
     */
    contextMenu: {

      /**
       * Výchozí nastavení položky
       * 
       * @type Object
       */
      _defaultItem: {
        label: '',
        classes: '',
        disabled: false,
        separator: false,
        action: function() {}
      },

      /**
       * Vytvoří kontextové menu
       *
       * @param {Object} items
       * @returns {jQuery}
       */
      _buildMenu: function(items) {
        items = items || {};

        $(document).off('click', '[data-contextmenu-action]');
        var cm = $('<ul>').addClass('context-menu');
        $.each(items, function(action, item) {
          var
            i = $.extend({}, Webbia.contextMenu._defaultItem, item),
            a = i.disabled
              ? $('<span>').addClass(i.classes).html(i.label)
              : $('<a>').attr('data-contextmenu-action', action).addClass(i.classes).html(i.label),
            li = $('<li>').addClass('context-item' + (i.separator ? ' separator' : '')).append(a)
          ;
          cm.append(li);
          $(document).on('click', '[data-contextmenu-action="' + action + '"]', i.action);
        });

        return cm;
      },

      /**
       * Zobrazí kontextové menu
       *
       * @param {Object} items
       * @param {Number} x
       * @param {Number} y
       * @return {Boolean}
       */
      show: function(items, x, y) {
        var cm = Webbia.contextMenu._buildMenu(items);
        $('body').append(cm.css({left: x + 'px', top: y + 'px'}));

        cm.show();
        $('html').one('click contextmenu', function() {
          if (cm.is(':visible')) {
            cm.hide().remove();
          }
        });

        return false;
      }
    },

    /**
     * Objekt galerie
     *
     * @type gallery
     */
    gallery: {

      /**
       * Text pro zobrazení galerie
       *
       * @type String
       */
      show: '',

      /**
       * Text pro zavření galerie
       *
       * @type String
       */
      hide: '',

      /**
       * Handle okna
       *
       * @type Number
       */
      win: null,

      /**
       * Akce pro vytvoření galerie v nové záložce / okně
       *
       * @param {Event} event
       * @returns {*}
       */
      open: function(event) {
        localStorage.setItem('refferer', location.href);
        Webbia.gallery.win = open(this.href);
        return false;
      },

      /**
       * Funkce pro zavření okna
       *
       * @return {*}
       */
      close: function() {
        if (opener) {
          return close();
        }
        if (Webbia.gallery.opener) {
          return opener.close();
        }

        var refferer = localStorage.getItem('refferer');
        if (refferer) {
          location.href = refferer;
        } else {
          history.back();
        }

        return false;
      },

      /**
       * Akce pro zachycování stisku kláves
       *
       * @param {Event} event
       * @returns {Void}
       */
      onKeyDown: function(event) {
        if (event.keyCode === 27) {
          Webbia.gallery.close();
        }
        if ((event.keyCode === 37) || (event.keyCode === 38)) {
          var href = $('#previous').attr('href');
          if (href) {
            location.href = href;
          }
        }
        if ((event.keyCode === 39) || (event.keyCode === 40)) {
          var href = $('#next').attr('href');
          if (href) {
            location.href = href;
          }
        }
      },

      /**
       * Akce po klik na lupu
       *
       * @returns {Void}
       */
      onSize: function() {
        var a = $('#toggle-size'), img = $('#image'), src = img.attr('src'), dataSrc = img.data('src');
        a.toggleClass('plus').toggleClass('minus');
        img.attr('src', dataSrc).data('src', src).toggleClass('draggable');
        img.is('.draggable') ? img.draggable('enable') : img.draggable('disable');
        img.css({top: 0, left: 0});
      },

      /**
       * Událost po tažení obrázku
       *
       * @param {Event} event
       * @param {Object} ui
       * @returns {Void}
       */
      onDrag: function(event, ui) {
        var
          img = $(this),
          container = $('#media-box'),
          xDiff = img.width() - container.innerWidth(),
          yDiff = img.height() - container.innerHeight()
        ;
        if (0 <= ui.position.top) {
          ui.position.top = 0;
        }
        if (0 <= ui.position.left) {
          ui.position.left = 0;
        }
        if (-ui.position.top >= yDiff) {
          ui.position.top = -yDiff;
        }
        if (-ui.position.left >= xDiff) {
          ui.position.left = -xDiff;
        }
        img.css(ui.position);
      },

      /**
       * Inicializace galerie
       *
       * @returns {Void}
       */
      init: function() {
        $(document).on('keydown', Webbia.gallery.onKeyDown);
        $(document).on('click', '#close-box a', Webbia.gallery.close);
        $(document).on('click', '#toggle-size', Webbia.gallery.onSize);
        $(document).on('click', '#hide', function() {
          $('#hide').attr({id: 'show', title: Webbia.gallery.show});
          $('#description').hide();
        });
        $(document).on('click', '#show', function() {
          $('#show').attr({id: 'hide', title: Webbia.gallery.hide});
          $('#description').show();
        });
        $('#image').draggable({
          drag: Webbia.gallery.onDrag,
          zIndex: 100
        }).draggable('disable');
      }
    },

    /**
     * Funkce pro přepínání záložek
     *
     * @returns {undefined}
     */
    switchTab: function() {
      var base = $(this).attr('id').split('_')[0];
      if ($('#' + base + '_header').length) {
        for (var i = 0, count = $('#' + base + '_header div').length; i < count; i++) {
          var caption = $('#' + base + '_caption' + i);
          var content = $('#' + base + '_content' + i);
          if ($(this).attr('id') === caption.attr('id')) {
            caption.addClass('active');
            content.addClass('visible');
            content.removeClass('invisible');
          } else {
            caption.removeClass('active');
            content.removeClass('visible');
            content.addClass('invisible');
          }
        }
      }
    },

    /**
     * Akce dialogu - aktuálně onClose
     *
     * @type Array
     */
    _closeHandlers: [],

    /**
     * Provolání metod uzavření dialogu
     *
     * @param {Boolean} isCancelled  Byl dialog stornován
     * @returns {Void}
     */
    _doCloseDialog: function(isCancelled) {
      var i = Webbia._closeHandlers.length;
      while (i--) {
        try {
          Webbia._closeHandlers[i](isCancelled);
        } catch (e) {
          console.log(e.message);
        }
      }
    },

    /**
     * Funkce pro otevření okna pro výběr souborů
     *
     * @param {String} field_name  Název formulářového pole pro soubor
     * @param {String} url         URL vloženého souboru
     * @param {String} type        Typ dialogového okna (soubory, obrázky, média)
     * @param {Object} win         Handle objekt okna
     * @returns {Boolean}
     */
    browse: function(field_name, url, type, win) {
      var cmsURL = '/dialog/browse?type=' + type + '&url=' + escape(url);
      tinyMCE.activeEditor.windowManager.open({
        file: cmsURL,
        width: 620,
        height: 425,
        resizable: 'no',
        inline: 'yes',
        close_previous: 'no'
      }, {
        window: win,
        input: field_name
      });
      return false;
    },

    /**
     * Kolektor obsluh událostí po uzavření dialogu
     *
     * @param {Function} callback
     * @returns {Void}
     */
    onBrowseClose: function(callback) {
      if (null === callback) {
        Webbia._closeHandlers = [];
        return;
      }
      Webbia._closeHandlers[Webbia._closeHandlers.length] = callback;
    },
    
    /**
     * „Očistí“ obsah od nežádoucích font-family
     * 
     * @param {String} content
     * @returns {String}
     */
    filterContent: function(content) {
      var
        wrapper = $('<div />').html(content),
        monospaced = 'Consolas,Monaco,Lucida Console,Liberation Mono,'
                   + 'DejaVu Sans Mono,Bitstream Vera Sans Mono,'
                   + 'Courier New,Courier,monospace',
        unwrapContent = function(element) {
          return element.contents().unwrap();
        }
      ;
      
      wrapper.find('[style*="font-family"]').each(function() {
        var
          self = $(this),
          family = /font-family\:(.*)?\;/gi.exec(self.attr('style')),
          list = family && family.length ? family[1].split(',') : [],
          match = false
        ;
        $.each(list, function(i, font) {
          match = !!monospaced.match(new RegExp($.trim(font), 'gi')) || match;
        });
        match ? unwrapContent(self).wrap('<pre />') : unwrapContent(self);
      });          
      
      wrapper.find('pre span, code span').each(function() {
        unwrapContent($(this));
      });          
      
      return wrapper.html();
    },

    /**
     * Funkce pro očištění vstupních dat TinyMCE
     *
     * @param {String} type     Typ
     * @param {String} content  Obsah
     * @returns {String}
     */
    wordCleanup: function(type, content) {
      switch (type) {
        case 'get_from_editor':
          content = content.replace(/<!(?:--[\s\S]*?--\s*)?>\s*/g, '');
          content = content.replace(/&lt;!(?:--[\s\S]*?--\s*)?&gt;\s*/g, '');
          break;
      }
      return Webbia.filterContent(content);
    },

    /**
     * Funkce pro odpočet času
     *
     * @param {String} element    Selektor elementu, do kterého bude vypsán čas
     * @param {Function} action   Akce k vykonání
     * @returns {undefined}
     */
    countDown: function(element, action) {
      var timer, end = Date.now() + (+Webbia.logoutTime * 1000), doCountDown = function() {
        var now = Date.now();
        if (now > (end - INCOMING_LOGOUT) && !$('#logout-warning').length) {
          var h6 = $('<h6>').attr({
            'class': 'warning', 
            'id': 'logout-warning', 
            'style': 'display: none;'
          });
          $('#breadcrumbs').after(h6.text(Webbia.incomingLogoutWarning));
          h6.show('slow');
        }
        if (now <= end) {
          element && $(element).html(((end - now) / 1000).toHHmmss());
        } else {
          clearInterval(timer);
          ('function' === typeof(action)) && action();
        }
      };
      $(element).html(Webbia.logoutTime.toHHmmss());
      timer = setInterval(doCountDown, 500);
    },

    /**
     * Funkce pro odhlášení uživatele
     *
     * @returns {undefined}
     */
    logout: function() {
      $.ajax('/logout/json').done(function(data) {
        if (!data) {
          return;
        }
        if (!!$('form[id*="form"]').length) {
          return Webbia.inlineLogin.show();
        }
        location.href = '/admin';
      });
    },

    /**
     * Metoda pro přepínání elementu pro překlady
     *
     * @param {String} langSelector  Selektor pro element jazyka
     * @param {String} inputName     Název elementu přepínaného vstupu
     * @returns {undefined}
     */
    switchTranslatableText: function(langSelector, inputName) {
      var doOnChange = function() {
        var value = $(langSelector).val();
        $('input[name^="' + inputName + '"]').prop('type', 'hidden');
        $('input#' + inputName + '-' + value).prop('type', 'text');
      };
      $(langSelector).bind('change', doOnChange);
      doOnChange(); // rovnou funkci zavoláme
    },

    /**
     * Nastavení obsluhy pro tlačítko Storno
     *
     * @param {Object} obj  Nastavení obsluhy zrušení; atributy: <br />
     *                      • form - selektor formuláře <br />
     *                      • exclude - pole názvů elementů k vyřazení <br />
     *                      • message - potvrzovací zpráva při změněných hodnotách <br />
     *                      • button - selektor tlačítka Storno <br />
     *                      • url - URL přesměrování stránky
     * @returns {cancellation}
     */
    cancellation: function cancellation(obj) {
      var form = obj.form || '<form />';
      var exclude = obj.exclude || [];
      var message = obj.message || 'Ukončit editaci?';
      var button = obj.button || '#cancel';
      var url = obj.url || '/';

      /**
       * Vrátí serializované hodnoty formuláře
       *
       * @returns {String}
       */
      var serialize = function() {
        var value = $(form).serializeArray();
        if (exclude.length) {
          $.each(exclude, function(key, element) {
            $.each(value, function(index, pairs) {
              if (element === pairs.name) {
                value.splice(index, 1);
              }
            });
          });
        }
        return $.param(value);
      };

      var initValue = serialize();

      /**
       * Otestuje změny a případně nezměněného formuláře nebo
       * potvrzení storna přesměruje
       *
       * @returns {undefined}
       */
      var cancel = function() {
        var result = (initValue === serialize());
        if (!result) {
          result = confirm(message);
        }
        if (result) {
          location.href = url;
        }
      };

      $(button).click(cancel); // nastavení obsluhy stisku tlačítka
    },

    /**
     * Zaktualizuje datalist pro vyhledávání
     *
     * @returns {Void}
     */
    setDLOptions: function() {
      var self = $(this), text = $.trim(self.val()), qdl = $('#qdl');
      if (1 === text.length) {
        $.getJSON('/search/terms', {term: text}, function(data) {
          qdl.empty();
          $.each(data, function(i, value) {
            $('<option>').html(value).appendTo(qdl);
          });
        });
      }
      if (0 === text.length) {
        qdl.empty();
      }
    },

    /**
     * Inicializace objektu Webbia
     *
     * @returns {undefined}
     */
    init: function() {
      $(document).on('click', '[data-bind="tabs"]', Webbia.switchTab);
      $(document).on('click', 'a.gallery', Webbia.gallery.open);
      $(document).on('keyup', '#q', Webbia.setDLOptions);
      if (Webbia.logoutTime) {
        Webbia.countDown('#logout-counter', Webbia.logout);
      }

      var complete = function() {
        var visible = $('nav ul').is(':visible'), menuBtn = $('.menu-btn');
        menuBtn[visible ? 'addClass' : 'removeClass']('icon-close');
        menuBtn[!visible ? 'addClass' : 'removeClass']('icon-menu');
      };
      $('html').click(function() {
        if ($('.menu-btn').is(':visible') && $('nav ul').is(':visible')) {
          $('nav ul').fadeToggle(400, 'swing', complete);
        }
      });
      $('.menu-btn').click(function() {
        $('nav ul').fadeToggle(400, 'swing', complete);
        return false;
      });
    }
  };

  /**
   * Akce po načtení dokumentu
   */
  $(Webbia.init);
}
