(function() {
  tinymce.PluginManager.requireLangPack('dynamic');

  tinymce.create('tinymce.plugins.Dynamic', {
    init: function(ed, url) {
      ed.addCommand('mceDynamic', function() {
        var vars = {plugin_url: url};
        if (ed.selection.getNode().className === 'dynamic') {
          var hash = ed.selection.getNode().textContent || ed.selection.getNode().innerText;
          var params = hash.replace(/\[\[\s*([^\s][^#]*[^\s])\s*\]\]/, '$1')
                           .replace(/^\s+|\s+$/g, '')
                           .replace(/( ){2,}/g, ' ')
                           .split(' ');

          vars.type = params[0].replace(/\s/g, '');
          for (var i = 1; i < params.length; i++) {
            params[i] = params[i].split('=');
          }
          for (var i = 1; i < params.length; i++) {
            vars[params[i][0]] = params[i][1];
          }
        }

        var language = window.top.document.getElementById('language').value;
        var popup_url = '/dialog/dynamic?language=' + language;

        ed.windowManager.open({
          file: popup_url,
          width: 600 + ed.getLang('dyncontent.delta_width', 0),
          height: 380 + ed.getLang('dyncontent.delta_height', 0),
          inline: 1
        }, vars);
      });

      ed.addButton('dynamic', {
        title: 'dynamic.desc',
        cmd:   'mceDynamic',
        image: url + '/img/dynamic.png'
      });

      ed.onNodeChange.add(function(ed, cm, n) {
        cm.setActive('dynamic', n.className && n.className === 'dynamic');
      });

      var _fixSelection = function(ed, e) {
        var node = ed.selection.getNode();
        if (node !== null && node.className && node.className === 'dynamic') {
          var rng = ed.selection.getRng();
          rng.selectNodeContents(node);
          ed.selection.setRng(rng);
        }
        return tinymce.dom.Event.cancel(e);
      };

      ed.onMouseUp.addToTop(function(ed, e) {
        if (e.button !== 2) {
          _fixSelection(ed, e);
        }
      });

      ed.onKeyDown.addToTop(function(ed, e) {
        var node = ed.selection.getNode(), before = e.shiftKey;
        if (e.keyCode === 13 && node !== null && node.className && node.className === 'dynamic') { // enter
          var dynamic = node;
          if (before) {
            node = node.previousSibling;
          }
          if (node) {
            ed.dom.insertAfter(ed.dom.create('p', {}, '&nbsp;'), node);
          } else {
            var parent = dynamic.parentNode;
            node = ed.dom.create('p', {}, '&nbsp;');
            parent.insertBefore(node, parent.childNodes[0]);
          }
          return tinymce.dom.Event.cancel(e);
        }
        if (e.keyCode === 46 && node !== null && node.className && node.className === 'dynamic') { // delete
          ed.dom.remove(node);
          return tinymce.dom.Event.cancel(e);
        }
      });

      ed.onInit.add(function(ed) {
        var body = ed.getBody(), walker = new tinymce.dom.TreeWalker(body, body), node;
        while (node = walker.next()) {
          if ((node.nodeName === 'DIV') && node.className && node.className === 'dynamic') {
            node.setAttribute('contenteditable', 'false');
          }
        }
      });
    },

    getInfo: function() {
      return {
        longname:  'Dynamický obsah',
        author:    'Bc. Petr Jajtner',
        authorurl: 'http://www.webbia.cz',
        infourl:   'http://www.webbia.cz',
        version:   '1.1'
      };
    }
  });

  tinymce.PluginManager.add('dynamic', tinymce.plugins.Dynamic);
})();
