tinyMCEPopup.requireLangPack();

/**
 * Dialog dynamického obsahu
 *
 * @type Object
 */
var DynamicDialog = {

  /**
   * Otestuje, zdali je n v daném rozsahu
   *
   * @param {*} n
   * @returns {Boolean}
   */
  isUint8: function(n) {
    return +n === n && !(n % 1) && n < 0x100 && n >= 0; // od 0 do 255
  },

  /**
   * Inicializace dialogu
   *
   * @returns {Void}
   */
  init: function() {

    /**
     * Vybere položku v seznamu
     *
     * @returns {Void}
     */
    var _select = function (options, value) {
      if (!options || !options.length) {
        return;
      }
      for (i = 0; i < options.length; i++) {
        options[i].selected = (options[i].value === value);
      }
    };

    var
      form = document.getElementById('insert_dynamic'),
      elementClass = tinyMCEPopup.editor.selection.getNode().className
    ;

    if (elementClass === 'dynamic') {
      var type = tinyMCEPopup.getWindowArg('type');
      if ('gallery' === type) {
        _select(form.language.options, tinyMCEPopup.getWindowArg('language'));

        var id = tinyMCEPopup.getWindowArg('id');
        document.getElementById('gallery-' + id).checked = true;

        var limit = tinyMCEPopup.getWindowArg('limit');
        form.limit.value = DynamicDialog.isUint8(+limit) ? limit : '0';

        var pager = tinyMCEPopup.getWindowArg('pager');
        form.pager.checked = ('1' === pager);

        var link = tinyMCEPopup.getWindowArg('link');
        form.link.checked = ('1' === link);

        var caption = tinyMCEPopup.getWindowArg('caption');
        form.caption.checked = ('1' === caption);
      }
      if ('browser' === type) {
        _select(form.language.options, tinyMCEPopup.getWindowArg('language'));

        var gallery = tinyMCEPopup.getWindowArg('gallery');
        if ('all' === gallery) {
          document.getElementById('gallery-all').checked = true;
        } else {
          document.getElementById('gallery-selected').checked = true;
          for (var i = 0, g = gallery.split(/,\s*/g), l = g.length; i < l; ++i) {
            document.getElementById('selection-' + g[i]).checked = true;
          }
        }

        var limit = tinyMCEPopup.getWindowArg('limit');
        form.count.value = DynamicDialog.isUint8(+limit) ? limit : '4';

        _select(form.size.options, tinyMCEPopup.getWindowArg('size') || 'thumb');

        var captions = tinyMCEPopup.getWindowArg('captions'), undef;
        form.captions.checked = ('1' === captions) || (undef === captions);

        var links = tinyMCEPopup.getWindowArg('links');
        form.links.checked = ('1' === links);

        var index = tinyMCEPopup.getWindowArg('index');
        form.index.checked = ('1' === index);
      }
      if ('contactus' === type) {
        form.recipients.value = tinyMCEPopup.getWindowArg('recipients');
      }
      if (dynamicHandler) {
        dynamicHandler.init(type);
      }
    }
  },

  /**
   * Vloží HTML kód dynamického obsahu do editoru
   *
   * @returns {Void}
   */
  insert: function() {
    var
      form = document.getElementById('insert_dynamic'),
      tabs = ['gallery_panel', 'browser_panel', 'contactus_panel'],
      type = 'gallery'
    ;

    for (var t = 0, l = tabs.length; t < l; t++) {
      var className = document.getElementById(tabs[t]).className;
      if (-1 < className.indexOf('current')) {
        type = tabs[t].replace('_panel', '');
        break;
      }
    }

    var code = '';
    if ('gallery' === type) {
      var id = '', language = getSelectValue(form, 'language');

      for (var i = 0, o = document.getElementsByName('gallery'), l = o.length; i < l; i++) {
        if (o[i].checked) {
          id = o[i].value;
          break;
        }
      }

      code = type + ' id=' + id;
      if ('' !== language) {
        code += ' language=' + language;
      }
      if (0 < +form.limit.value) {
        code += ' limit=' + form.limit.value;
      }
      if (form.pager.checked) {
        code += ' pager=1';
      }
      if (form.link.checked) {
        code += ' link=1';
      }
      if (form.caption.checked) {
        code += ' caption=1';
      }
      code = '[[' + code + ']]';
    }
    if ('browser' === type) {
      var language = getSelectValue(form, 'language'), gallery;

      if (document.getElementById('gallery-all').checked) {
        gallery = 'all';
      } else {
        gallery = [];
        for (var i = 0, s = document.getElementsByName('selection[]'), l = s.length; i < l; i++) {
          s[i].checked && gallery.push(s[i].value);
        }
        gallery = gallery.join(',');
      }

      code = type + ' gallery=' + gallery;
      if ('' !== language) {
        code += ' language=' + language;
      }
      code += ' size=' + getSelectValue(form, 'size');
      if (0 < +form.count.value) {
        code += ' limit=' + form.count.value;
      }
      code += ' captions=' + +form.captions.checked;
      if (form.captions.checked && form.links.checked) {
        code += ' links=1';
      }
      if (form.index.checked && form.index.checked) {
        code += ' index=1';
      }
      code = '[[' + code + ']]';
    }
    if ('contactus' === type) {
      code = type + ' recipients=' + form.recipients.value;
      code = '[[' + code + ']]';
    }

    var
      attributes = {
        class: 'dynamic',
        contenteditable: 'false'
      },
      element = tinyMCEPopup.editor.dom.create('div', attributes, ''),
      selection = tinyMCEPopup.editor.selection.getNode(),
      elementClass = selection.className,
      parent
    ;

    if (elementClass === 'dynamic') {
      tinyMCEPopup.editor.dom.setHTML(selection, code);
    } else {
      tinyMCEPopup.editor.dom.setHTML(element, code);

      parent = selection;
      if (!tinyMCEPopup.editor.dom.isBlock(selection)) {
        parent = tinyMCEPopup.editor.dom.getParent(selection, 'div,p,address,h3,h4,h5,h6,pre');
      }
      if (selection.tagName && selection.tagName === 'BODY') {
        tinyMCEPopup.editor.execCommand('mceInsertContent', false, tinyMCEPopup.editor.dom.getOuterHTML(element));
      } else {
        tinyMCEPopup.editor.dom.insertAfter(element, parent);
      }
    }

    tinyMCEPopup.close();
  }
};

// Inicializace tinyMCEPopupu - dialogu dynamického obsahu
tinyMCEPopup.onInit.add(DynamicDialog.init, DynamicDialog);