(function() {
  var Event = tinymce.dom.Event;

  // Load plugin specific language pack
  tinymce.PluginManager.requireLangPack('script');

  tinymce.create('tinymce.plugins.ScriptPlugin', {
    init: function(ed, url) {

      // Register commands
      ed.addCommand('mceScript', function() {
        ed.windowManager.open({
          file: url + '/dialog.htm',
          width: 600 + parseInt(ed.getLang('script.delta_width', 0)),
          height: 380 + parseInt(ed.getLang('script.delta_height', 0)),
          inline: 1
        }, {
          plugin_url: url // Plugin absolute URL
        });
      });

      // Remove script from editor
      ed.addCommand('mceScriptDelete', function() {
        var node = ed.selection.getNode();
        if (node.tagName && (node.tagName === 'SCRIPT')) {
          ed.dom.remove(node);
        }
      });

      // Register buttons
      ed.addButton('script', {
        title: 'script.desc',
        cmd: 'mceScript',
        image: url + '/img/script_add.png'
      });

      ed.addButton('delscript', {
        title: 'script.desc_del',
        cmd: 'mceScriptDelete',
        image: url + '/img/script_delete.png'
      });

      // Add a node change handler
      ed.onNodeChange.add(function(ed, cm, n) {
        cm.setDisabled('delscript', !n.tagName || (n.tagName !== 'SCRIPT'));
        cm.setDisabled('script', false);
        cm.setActive('script', n.tagName && (n.tagName === 'SCRIPT'));
        cm.setActive('delscript', n.tagName && (n.tagName === 'SCRIPT'));
      });

      var _fixSelection = function(ed, e) {
        var node = ed.selection.getNode();
        if (node !== null && node.tagName && (node.tagName === 'SCRIPT')) {
          var rng = ed.selection.getRng();
          rng.selectNodeContents(node);
          ed.selection.setRng(rng);
        }
        return Event.cancel(e);
      };

      ed.onMouseUp.addToTop(function(ed, e) {
        if (e.button !== 2) {
          _fixSelection(ed, e);
        }
      });

      ed.onKeyDown.addToTop(function(ed, e) {
        var node = ed.selection.getNode(), before = e.shiftKey;
        if (e.keyCode === 13 && node && node.tagName && (node.tagName === 'SCRIPT')) {
          if (before) {
            node = node.parentNode.previousSibling;
          }
          ed.selection.select(ed.dom.insertAfter(ed.dom.create('p', {}, '&nbsp;'), node));
          return Event.cancel(e);
        }
      });

      ed.onInit.add(function(ed) {
        var body = ed.getBody(), walker = new tinymce.dom.TreeWalker(body, body), node;
        while (node = walker.next()) {
          if (node && node.tagName && (node.tagName === 'SCRIPT')) {
            node.setAttribute('contenteditable', 'false');
          }
        }
      });
    },

    getInfo: function() {
      return {
        longname: 'Plugin pro javascript',
        author: 'Bc. Petr Jajtner',
        version: '1.0'
      };
    }
  });

  // Register plugin
  tinymce.PluginManager.add('script', tinymce.plugins.ScriptPlugin);
})();