// Požadovaný jazykový balíček
tinyMCEPopup.requireLangPack('script');

/**
 * Dialog pro javascript
 *
 * @type Object
 */
var ScriptDialog = {

  /**
   * Inicializace dialogu
   *
   * @returns {Void}
   */
  init: function() {

    /**
     * Otestuje element, zdali je tagem SCRIPT
     *
     * @param {Node} element
     * @returns {Boolean}
     */
    function _tagMatch(element) {
      while (null !== element) {
        if (element.tagName && (element.tagName === 'SCRIPT')) {
          return true;
        }
        element = element.parentNode;
      }
      return false;
    }

    var node = tinyMCEPopup.editor.selection.getNode();
    if (_tagMatch(node)) {
      document.getElementById('script_code').value = node.textContent;
    } else {
      document.getElementById('script_code').value = '// <![CDATA[\n\n// ]]>';
    }
  },

  /**
   * Vloží HTML kód javascriptu do editoru
   *
   * @returns {Void}
   */
  insert: function() {
    var code = document.getElementById('script_code').value;

    try {
      new Function(code);
    } catch (error) {
      return alert(error);
    }

    var
      node = tinyMCEPopup.editor.selection.getNode(),
      parent = tinyMCEPopup.editor.dom.getParent(node, 'script'),
      attributes = {
        type: 'text/javascript',
        contenteditable: 'false'
      },
      element
    ;
    if (parent && parent.tagName && (parent.tagName === 'SCRIPT')) {
      element = tinyMCEPopup.editor.dom.get(parent);
      tinyMCEPopup.editor.dom.setAttribs(element, attributes);
      tinyMCEPopup.editor.dom.setHTML(element, code);
    } else {
      element = tinyMCEPopup.editor.dom.create('script', attributes, code);

      parent = node;
      if (!tinyMCEPopup.editor.dom.isBlock(node)) {
        parent = tinyMCEPopup.editor.dom.getParent(node, 'div,p,address,h3,h4,h5,h6,pre');
      }
      if (node.tagName && node.tagName === 'BODY') {
        tinyMCEPopup.editor.execCommand('mceInsertContent', false, tinyMCEPopup.editor.dom.getOuterHTML(element));
      } else {
        tinyMCEPopup.editor.dom.insertAfter(element, parent);
      }
    }

    tinyMCEPopup.close();
  }
};

// Inicializace tinyMCEPopupu - dialogu pro javascript
tinyMCEPopup.onInit.add(ScriptDialog.init, ScriptDialog);
