(function() {
  tinymce.PluginManager.requireLangPack('sideblock');

  tinymce.create('tinymce.plugins.SideblockPlugin', {
    init: function(ed, url) {
      ed.addCommand('mceSideblock', function() {
        ed.windowManager.open({
          file: url + '/dialog.htm',
          width: 245 + parseInt(ed.getLang('sideblock.delta_width', 0)),
          height: 112 + parseInt(ed.getLang('sideblock.delta_height', 0)),
          inline: 1
        }, {
          plugin_url: url
        });
      });

      ed.addCommand('mceSideblockDelete', function() {
        var getElementByMatch = function (pattern, element) {
          while (null !== element) {
            if (element.id && element.id.match(pattern)) {
              return element;
            }
            element = element.parentNode;
          }
          return null;
        };

        var node = ed.selection.getNode();
        var element = null;
        var match = (element = getElementByMatch(/^sideblock(\d+)/, node))
                  ? element.id.match(/^sideblock(\d+)/)
                  : false;

        if (match) {
          ed.dom.remove(match[0]);
        }
      });

      ed.addButton('sideblock', {
        title: 'sideblock.desc',
        cmd: 'mceSideblock',
        image: url + '/img/sideblock_add.png'
      });

      ed.addButton('delsideblock', {
        title: 'sideblock.desc_del',
        cmd: 'mceSideblockDelete',
        image: url + '/img/sideblock_delete.png'
      });

      ed.onNodeChange.add(function(ed, cm, n) {
        var elementMatch = function (pattern, element, required) {
          if (!required && !element.id) {
            return true;
          }
          while (null !== element) {
            if (element.id && element.id.match(pattern)) {
              return true;
            }
            element = element.parentNode;
          }
          return false;
        };

        cm.setDisabled('delsideblock', !elementMatch(/^sideblock(\d+)/, n, true));
        cm.setDisabled('sideblock', !elementMatch(/^sideblock(\d+)/, n, false));
        cm.setActive('sideblock', elementMatch(/^sideblock(\d+)/, n, true));
        cm.setActive('delsideblock', elementMatch(/^sideblock(\d+)/, n, true));
      });

      ed.onInit.add(function(ed) {
        function fixCaretPos() {
          var last;
          for (last = ed.getBody().lastChild; last && last.nodeType === 3 && !last.nodeValue.length; last = last.previousSibling);

          if (last && last.nodeName === 'DIV' && last.id && last.id.match(/^sideblock/)) {
            if (ed.settings.forced_root_block)
              ed.dom.add(ed.getBody(), ed.settings.forced_root_block, null, tinymce.isIE && !tinymce.isIE11 ? '&nbsp;' : '<br data-mce-bogus="1" />');
            else
              ed.dom.add(ed.getBody(), 'br', {'data-mce-bogus': '1'});
          }
        };

        ed.onKeyUp.add(fixCaretPos);
        ed.onSetContent.add(fixCaretPos);
        ed.onVisualAid.add(fixCaretPos);

        (function(ed) {
          var body = ed.getBody(), walker = new tinymce.dom.TreeWalker(body, body), node;
          while (node = walker.next()) {
            if ((node.nodeName === 'DIV') && node.id && node.id.match(/^sideblock/)) {
              if (node.id.match(/^sideblock(\d+)$/) && !tinymce.isWebKit) {
                node.setAttribute('contenteditable', 'false');
              }
              if (node.id.match(/^sideblock(\d+)_inner/)) {
                node.setAttribute('contenteditable', 'true');
              }
            }
          }
        })(ed);
      });
    },

    getInfo: function() {
      return {
        longname: 'Plugin pro postranní blok',
        author: 'Bc. Petr Jajtner',
        version: '1.1'
      };
    }
  });

  tinymce.PluginManager.add('sideblock', tinymce.plugins.SideblockPlugin);
})();