// Požadovaný jazykový balíček
tinyMCEPopup.requireLangPack('sideblock');

/**
 * Dialog pro postranní blok
 *
 * @type Object
 */
var SideblockDialog = {

  /**
   * Limity šířky postranního bloku v pixelech
   *
   * @type Object
   */
  limits: {min: 85, max: 400},

  /**
   * Výchozí šířka postranního bloku v pixelech
   *
   * @type Number
   */
  width: 250,

  /**
   * Zarovnání bloku
   *
   * @type String
   */
  float: 'left',

  /**
   * Vnitřní obsah bloku
   *
   * @type String
   */
  inner: '',

  /**
   * Počítadlo instancí postranních bloků na stránce
   *
   * @type Number
   */
  counter: 0,

  /**
   * Vrátí šablonku postranního bloku
   *
   * @returns {String}
   */
  _template: function() {
    var
      idInner = 'sideblock' + SideblockDialog.counter + '_inner',
      inner = SideblockDialog.inner ? SideblockDialog.inner : '<p>Lorem ipsum</p>',
      column = '<div id="' + idInner + '" contenteditable="true">' + inner + '</div>\n'
    ;
    return column;
  },

  /**
   * Vrátí obalující prvek postranního bloku
   *
   * @param {Node} element
   * @returns {Node}
   */
  _getWrapper: function(element) {
    while (element) {
      if (element.id && element.id.match(/^sideblock(\d+)$/)) {
        return element;
      }
      element = element.parentNode;
    }
    return null;
  },

  /**
   * Inicializace dialogu
   *
   * @returns {Void}
   */
  init: function() {

    /**
     * Vrátí shodu vzoru pattern s ID elementu
     *
     * @param {Node} element
     * @param {Regexp} pattern
     * @returns {Array}
     */
    function _elementMatch(element, pattern) {
      var match = null;
      while (null !== element) {
        if (element.id && (match = element.id.match(pattern))) {
          return match;
        }
        element = element.parentNode;
      }
      return null;
    }

    /**
     * Vrátí vnitřní obsah elementu
     *
     * @param {Number} index
     * @returns {String}
     */
    function _getInner(index) {
      var node;
      if (node = tinyMCEPopup.editor.dom.get('sideblock' + index + '_inner')) {
        return node.innerHTML;
      }
      return '';
    }

    var
      node = tinyMCEPopup.editor.selection.getNode(),
      match = _elementMatch(node, /^sideblock(\d+)/),
      wrapper = match ? SideblockDialog._getWrapper(node) : null,
      settings = wrapper ? wrapper.getAttribute('data-settings') : null
    ;

    if (wrapper && match && settings) {
      var parts = settings.split('-');
      SideblockDialog.float = parts[0];
      SideblockDialog.width = +parts[1];
    }

    if (match) {
      SideblockDialog.counter = match[1];
      SideblockDialog.inner = _getInner(SideblockDialog.counter);
    } else {
      while (tinyMCEPopup.editor.dom.get('sideblock' + SideblockDialog.counter)) {
        SideblockDialog.counter++;
      }
    }

    var width = document.getElementById('width');
    width.setAttribute('min', SideblockDialog.limits.min);
    width.setAttribute('max', SideblockDialog.limits.max);
    width.value = SideblockDialog.width;

    var float = document.getElementById('float');
    float.value = SideblockDialog.float;
  },

  /**
   * Reakce na změnu šířky postranního bloku
   *
   * @returns {Void}
   */
  change: function() {
    var width = +(document.getElementById('width').value);
    if (isNaN(width)) {
      return alert(tinyMCEPopup.getLang('sideblock_dlg.notNumber'));
    }
    if ((SideblockDialog.limits.min > width) || (SideblockDialog.limits.max < width)) {
      var error = tinyMCEPopup.getLang('sideblock_dlg.notInRange');
      return alert(error.replace('%min%', SideblockDialog.limits.min).replace('%max%', SideblockDialog.limits.max));
    }

    SideblockDialog.width = width;

    var float = document.getElementById('float').value;
    if (('left' === float) || ('right' === float)) {
      SideblockDialog.float = float;
    }
  },

  /**
   * Vloží HTML kód postranního bloku do editoru
   *
   * @returns {Void}
   */
  insert: function() {
    SideblockDialog.change();

    var
      template = SideblockDialog._template(),
      node = tinyMCEPopup.editor.selection.getNode(),
      parent = SideblockDialog._getWrapper(node),
      settings = SideblockDialog.float + '-' + SideblockDialog.width,
      attributes = {
        class: 'sideblock-wrapper',
        contenteditable: 'false',
        'data-settings': settings,
        id: 'sideblock' + SideblockDialog.counter,
        style: 'float: ' + SideblockDialog.float + '; ' +
               'width: ' + SideblockDialog.width + 'px;'
      },
      parentId,
      element
    ;

    if (parent && (parentId = parent.id.match(/^sideblock(\d+)$/))) {
      element = tinyMCEPopup.editor.dom.get(parentId[0]);
      tinyMCEPopup.editor.dom.setAttribs(element, attributes);
      tinyMCEPopup.editor.dom.setHTML(element, template);
    } else {
      element = tinyMCEPopup.editor.dom.create('div', attributes, template);

      parent = node;
      if (!tinyMCEPopup.editor.dom.isBlock(node)) {
        parent = tinyMCEPopup.editor.dom.getParent(node, 'div,p,address,h3,h4,h5,h6,pre');
      }

      if (node.tagName && node.tagName === 'BODY') {
        tinyMCEPopup.editor.execCommand('mceInsertContent', false, tinyMCEPopup.editor.dom.getOuterHTML(element));
      } else {
        tinyMCEPopup.editor.dom.insertAfter(element, parent);
      }
    }

    tinyMCEPopup.close();
  }
};

// Inicializace tinyMCEPopupu - dialogu pro postranní blok
tinyMCEPopup.onInit.add(SideblockDialog.init, SideblockDialog);
