(function() {
  tinymce.PluginManager.requireLangPack('tabs');

  tinymce.create('tinymce.plugins.TabsPlugin', {
    init : function(ed, url) {
      ed.addCommand('mceTabs', function() {
        ed.windowManager.open({
          file: url + '/dialog.htm',
          width: 225 + parseInt(ed.getLang('tabs.delta_width', 0)),
          height: 113 + parseInt(ed.getLang('tabs.delta_height', 0)),
          inline: 1
        }, {
          plugin_url: url
        });
      });

      ed.addCommand('mceTabsDelete', function() {
        var match = ed.selection.getNode().id.match(/^tabs(\d*)/);
        if (match) {
          ed.dom.remove(match[0]);
        }
      });

      ed.addButton('tabs', {
        title: 'tabs.desc',
        cmd: 'mceTabs',
        image: url + '/img/tab_add.png'
      });

      ed.addButton('deltabs', {
        title: 'tabs.desc_del',
        cmd: 'mceTabsDelete',
        image: url + '/img/tab_delete.png'
      });

      ed.onNodeChange.add(function(ed, cm, n) {
        var elementMatch = function (pattern, element, required) {
          if (!required && !element.id) {
            return true;
          }
          while (null !== element) {
            if (element.id && element.id.match(pattern)) {
              return true;
            }
            element = element.parentNode;
          }
          return false;
        };

        cm.setDisabled('deltabs', !elementMatch(/^tabs(\d+)/, n, true));
        cm.setDisabled('tabs', !elementMatch(/^tabs(\d+)/, n, false));
        cm.setActive('tabs', elementMatch(/^tabs(\d+)/, n, true));
        cm.setActive('deltabs', elementMatch(/^tabs(\d+)/, n, true));
      });

      ed.onInit.add(function(ed) {
        function fixCaretPos() {
          var last;
          for (last = ed.getBody().lastChild; last && last.nodeType === 3 && !last.nodeValue.length; last = last.previousSibling);

          if (last && last.nodeName === 'DIV' && last.id && last.id.match(/^tabs/)) {
            if (ed.settings.forced_root_block)
              ed.dom.add(ed.getBody(), ed.settings.forced_root_block, null, tinymce.isIE && !tinymce.isIE11 ? '&nbsp;' : '<br data-mce-bogus="1" />');
            else
              ed.dom.add(ed.getBody(), 'br', {'data-mce-bogus': '1'});
          }
        };

        ed.onKeyUp.add(fixCaretPos);
        ed.onSetContent.add(fixCaretPos);
        ed.onVisualAid.add(fixCaretPos);

        (function(ed) {
          var body = ed.getBody(), walker = new tinymce.dom.TreeWalker(body, body), node;
          while (node = walker.next()) {
            if ((node.nodeName === 'DIV') && node.id && node.id.match(/^tabs/)) {
              if (node.id.match(/^tabs(\d+)_(header|body)|tabs$/)) {
                node.setAttribute('contenteditable', 'false');
              }
              if (node.id.match(/^tabs(\d+)_(caption|content)/)) {
                node.setAttribute('contenteditable', 'true');
              }
            }
          }
        })(ed);
      });
    },

    getInfo: function() {
      return {
        longname: 'Plugin pro záložky',
        author: 'Bc. Petr Jajtner',
        version: '1.1'
      };
    }
  });

  tinymce.PluginManager.add('tabs', tinymce.plugins.TabsPlugin);
})();