// Požadovaný jazykový balíček
tinyMCEPopup.requireLangPack('tabs');

/**
 * Dialog pro záložky
 *
 * @type Object
 */
var TabsDialog = {

  /**
   * Minimum a maximum záložek
   *
   * @type Object
   */
  limits: {min: 1, max: 20},

  /**
   * Vnitřní obsah oušek
   *
   * @type Array
   */
  captions: [],

  /**
   * Vnitřní obsah bloků
   *
   * @type Array
   */
  contents: [],

  /**
   * Počítadlo instancí bloků se záložkami na stránce
   *
   * @type Number
   */
  counter: 0,

  /**
   * Vrátí shodu vzoru pattern s ID elementu
   *
   * @param {Node} element
   * @param {Regexp} pattern
   * @param {Boolean} asElement
   * @returns {Node|Array}
   */
  _elementMatch: function(element, pattern, asElement) {
    var match = null, asElement = asElement || false;
    while (null !== element) {
      if (element.id && (match = element.id.match(pattern))) {
        return asElement ? element : match;
      }
      element = element.parentNode;
    }
    return null;
  },

  /**
   * Vrátí šablonku záložek
   *
   * @param {Number} count
   * @param {String} _class
   * @returns {String}
   */
  _template: function(count, _class) {
    var
      captions = '',
      contents = '',
      header = '<div id="tabs' + TabsDialog.counter +  '_header" '
             + 'contenteditable="false" class="' + _class + '-header">\n',
      body = '<div id="tabs' + TabsDialog.counter + '_body" '
           + 'contenteditable="false" class="' + _class + '-body">\n';
    ;
    for (var i = 0; i < count; i++) {
      var
        caption = TabsDialog.captions[i] ? TabsDialog.captions[i] : '--- ' + (i + 1) + ' ---',
        content = TabsDialog.contents[i] ? TabsDialog.contents[i] : 'Lorem ipsum ' + (i + 1),
        captionId = 'tabs' + TabsDialog.counter + '_caption' + i,
        contentId = 'tabs' + TabsDialog.counter + '_content' + i,
        captionClass = _class + '-caption' + ((i === 0) ? ' active' : ''),
        contentClass = _class + '-content' + ((i === 0) ? ' visible' : ' invisible')
      ;
      captions += '  <div id="' + captionId + '" contenteditable="true" class="' + captionClass + '" data-bind="tabs">' + caption + '</div>\n';
      contents += '  <div id="' + contentId + '" contenteditable="true" class="' + contentClass + '">' + content + '</div>\n';
    }
    header += captions + '</div>\n';
    body += contents + '</div>\n';
    return header + body;
  },

  /**
   * Inicializace dialogu
   *
   * @returns {Void}
   */
  init: function() {

    /**
     * Vrátí vnitřní obsahy oušek a bloků
     *
     * @param {Number} count
     * @param {Number} index
     * @returns {Object}
     */
    function _getTexts(count, index) {
      var
        captions = [],
        contents = [],
        captionId = 'tabs' + index + '_caption',
        contentId = 'tabs' + index + '_content'
      ;
      for (var i = 0, caption = null, content = null; i < count; i++) {
        if (caption = tinyMCEPopup.editor.dom.get(captionId + i)) {
          captions[i] = caption.innerHTML;
        }
        if (content = tinyMCEPopup.editor.dom.get(contentId + i)) {
          contents[i] = content.innerHTML;
        }
      }
      return {
        captions: captions,
        contents: contents
      };
    }

    /**
     * Připraví selectbox pro výběr CSS třídy
     *
     * @param {Node} container
     * @returns {Void}
     */
    function _prepareSelectBox(container) {
      var
        classes = tinyMCEPopup.getParam('tabs_css_classes', 'tabs').split(','),
        selected = 'tabs',
        selectBox = '<select id="class" name="class" size="1">\n'
      ;
      for (var i = 0, l = classes.length, option; i < l; i++) {
        option = '  <option value="' + classes[i] + '"';
        if (classes[i] === selected) {
          option += ' selected="selected"';
        }
        option += '>' + classes[i] + '</option>\n';
        selectBox += option;
      }
      selectBox += '</select>\n';
      container.innerHTML = selectBox;
    }

    var
      node = tinyMCEPopup.editor.selection.getNode(),
      match = TabsDialog._elementMatch(node, /^tabs(\d+)/),
      wrapper = match ? TabsDialog._elementMatch(node, /^tabs(\d+)/, true) : null,
      count = match ? tinyMCEPopup.editor.dom.get('tabs' + match[1] + '_header').getElementsByTagName('div').length : null
    ;

    if (wrapper && match && count) {
      TabsDialog.counter = +match[1];
      TabsDialog = tinymce.extend(TabsDialog, _getTexts(count, TabsDialog.counter));
    } else {
      while (tinyMCEPopup.editor.dom.get('tabs' + TabsDialog.counter)) {
        TabsDialog.counter++;
      }
    }

    var element = document.getElementById('count');
    element.setAttribute('min', TabsDialog.limits.min);
    element.setAttribute('max', TabsDialog.limits.max);
    element.value = count ? count : 2;

    _prepareSelectBox(document.getElementById('tabs_css_classes'));
  },

  /**
   * Zkontroluje data formuláře
   *
   * @returns {Boolean}
   */
  check: function() {
    var count = +(document.getElementById('count').value);
    if (isNaN(count)) {
      alert(tinyMCEPopup.getLang('tabs_dlg.notNumber'));
      return false;
    }
    if ((TabsDialog.limits.min > count) || (TabsDialog.limits.max < count)) {
      var error = tinyMCEPopup.getLang('tabs_dlg.notInRange');
      return alert(error.replace('%min%', TabsDialog.limits.min).replace('%max%', TabsDialog.limits.max));
      return false;
    }
    return true;
  },

  /**
   * Vloží HTML kód záložek do editoru
   *
   * @returns {Void}
   */
  insert: function() {
    if (!TabsDialog.check()) {
      return;
    }

    var
      count = +document.getElementById('count').value,
      _class = document.getElementById('class').value,
      template = TabsDialog._template(count, _class),
      attributes = {
        id: 'tabs' + TabsDialog.counter,
        class: _class,
        contenteditable: 'false'
      },
      node = tinyMCEPopup.editor.selection.getNode(),
      match = TabsDialog._elementMatch(node, /^tabs(\d+)$/),
      element
    ;

    if (match) {
      element = tinyMCEPopup.editor.dom.get(match[0]);
      tinyMCEPopup.editor.dom.setAttribs(element, attributes);
      tinyMCEPopup.editor.dom.setHTML(element, template);
    } else {
      element = tinyMCEPopup.editor.dom.create('div', attributes, template);

      var parent = node;
      if (!tinyMCEPopup.editor.dom.isBlock(node)) {
        parent = tinyMCEPopup.editor.dom.getParent(node, 'div,p,address,h3,h4,h5,h6,pre');
      }

      if (node.tagName && node.tagName === 'BODY') {
        tinyMCEPopup.editor.execCommand('mceInsertContent', false, tinyMCEPopup.editor.dom.getOuterHTML(element));
      } else {
        tinyMCEPopup.editor.dom.insertAfter(element, parent);
      }
    }

    tinyMCEPopup.close();
  }
};

// Inicializace tinyMCEPopupu - dialogu pro záložky
tinyMCEPopup.onInit.add(TabsDialog.init, TabsDialog);
