/**
 * Anonymní funkce pro Uploader
 *
 * @returns {Void}
 */
(function() {
  var
    Uploader = window.Uploader = window.Uploader ? (function() {
      throw 'Uploader již ve scope window existuje!';
    })() : {},
    filesUpload = document.getElementById('files-upload'),
    dropArea = document.getElementById('drop-area') || document.getElementsByTagName('body')[0],
    fileList = document.getElementById('file-list'),
    fileCounts = {
      total:    0,
      uploaded: 0
    },
    defaultTexts = {
      MsgFileName:    'Název:',
      MsgFileSize:    'Velikost:',
      MsgFileType:    'Typ:',
      MsgAbortUpload: 'Storno',
      MsgNotAllowed:  'Soubor(y):\n\n%files%\n\nnelze nahrávat',
      MsgNoSupport:   'Tento webový prohlížeč nepodporuje File API'
    },
    defaultLocale = 'cs-CZ',
    defaultOptions = {
      minimumFractionDigits: 2,
      maximumFractionDigits: 2
    },
    action = 'upload.php',
    redirect = 'reload',
    extensions = [],
    notAllowed = [],
    texts = defaultTexts
  ;

  if (!fileList) {
    fileList = document.createElement('div');
    fileList.setAttribute('id', 'file-list');
    document.getElementsByTagName('body')[0].appendChild(fileList);
  }

  /**
   * Rutiny po úspěšném nahrání všech souborů
   *
   * @returns {Void}
   */
  function afterUploading() {
    fileCounts = {uploaded: 0, total: 0};
    if (redirect === Uploader.redirect) {
      location.reload();
    } else if ('' !== Uploader.redirect) {
      location.href = Uploader.redirect;
    }
  }

  /**
   * Otestuje, zdali soubor má příponu dle seznamu.
   * Pokud je seznam prázdný, přípona se netestuje.
   *
   * @param {File} file
   * @returns {Boolean}
   */
  function hasExtension(file) {
    if (!Uploader.extensions.length) {
      return true;
    }

    var parts = file.name.split('.');
    if (1 === parts.length) {
      return false;
    }

    var
      extensions = Uploader.extensions.join(',').toLowerCase().split(','),
      extension = parts[parts.length - 1].toLowerCase()
    ;

    return (-1 < extensions.indexOf(extension));
  }

  /**
   * Pomocí XHR nahraje soubor na server
   *
   * @param {File} file
   * @returns {Void}
   */
  function uploadFile(file) {
    fileCounts.total++;

    if (!hasExtension(file)) {
      notAllowed.push('• ' + file.name);
      return;
    }

    var
      fileItem = document.createElement('div'),
      fileInfo = document.createElement('div'),
      progressBarContainer = document.createElement('div'),
      progressBar = document.createElement('div'),
      progressValue = document.createElement('div'),
      anchor = document.createElement('a'),
      sizes = [' B', ' KB', ' MB', ' GB', ' TB'],
      percentLocale =  Uploader.locale,
      percentOptions = extend({style: 'percent'}, Uploader.options),
      abortEvent,
      sizes,
      bytes,
      spans,
      xhr
    ;

    // Nahrávání - pro Firefox, Google Chrome a Safari
    xhr = new XMLHttpRequest();

    progressBar.className = 'progress-bar';
    progressValue.className = 'progress-value';
    progressValue.innerHTML = '?';
    progressBarContainer.className = 'progress-bar-container';
    fileInfo.className = 'file-info';
    fileItem.className = 'file-item';

    fileItem.appendChild(fileInfo);
    fileItem.appendChild(progressBarContainer);
    fileItem.appendChild(anchor);
    progressBarContainer.appendChild(progressBar);
    progressBarContainer.appendChild(progressValue);

    anchor.setAttribute('href', 'javascript:void(0);');
    anchor.innerHTML = Uploader.texts.MsgAbortUpload;
    anchor.addEventListener('click', abortEvent = function() {
      xhr.abort();
      this.removeEventListener('click', abortEvent, false);
      this.parentNode.removeChild(this);
    }, false);

    // Aktualizace ukazatele průběhu nahrávání
    xhr.upload.addEventListener('progress', function(e) {
      if (e.lengthComputable) {
        var percent = e.loaded / e.total;
        progressBar.style.width = (percent * 100).toFixed(2) + '%';
        progressValue.innerHTML = percent.toLocaleString(percentLocale, percentOptions);
      }
    }, false);

    // Soubor nahrán
    xhr.addEventListener('load', function() {
      progressBar.style.width = '100%';
      anchor.parentNode.removeChild(anchor);
      fileList.scrollTop += fileItem.clientHeight;
      var done = (4 === xhr.readyState && 200 === xhr.status);
      if (done) {
        progressBarContainer.className += ' uploaded';
        progressValue.innerHTML = (1.0).toLocaleString(percentLocale, percentOptions);
        if (++fileCounts.uploaded === fileCounts.total) {
          Uploader.afterUploading();
        }
      } else {
        var result = JSON.parse(xhr.response);
        progressBarContainer.className += ' failed';
        progressValue.innerHTML = result.status + ': ' + result.data;
      }
    }, false);

    // Zrušení nahrávání
    xhr.addEventListener('abort', function() {
      progressBarContainer.className += ' aborted';
    }, false);

    xhr.open('post', Uploader.action, true);

    // Nastavení příslušných hlaviček
    xhr.setRequestHeader('Content-Type', 'multipart/form-data');
    xhr.setRequestHeader('X-File-Name', encodeURI(file.name));
    xhr.setRequestHeader('X-File-Size', file.size);
    xhr.setRequestHeader('X-File-Type', encodeURI(file.type));

    // Odeslání souboru
    xhr.send(file);

    bytes = +file.size;
    var i = 0;
    while (1023 < bytes){
      bytes /= 1024;
      ++i;
    };
    bytes = bytes.toLocaleString(Uploader.locale, Uploader.options) + sizes[i];

    // Vypíše informace o souboru a přidá jej do seznamu souborů
    spans  = '<p><strong>' + Uploader.texts.MsgFileName + '</strong> ' + file.name + '</p>\n';
    spans += '<p><strong>' + Uploader.texts.MsgFileSize + '</strong> ' + bytes + ' </p>\n';
    spans += '<p><strong>' + Uploader.texts.MsgFileType + '</strong> ' + file.type + '</p>\n';
    fileInfo.innerHTML = spans;

    fileList.appendChild(fileItem);
  }

  /**
   * Zavolá nad souborem funkci pro upload
   *
   * @param {DataTransferItem} fileEntry
   * @param {Function} callback
   * @returns {Void}
   */
  function readFile(fileEntry, callback) {
    fileEntry.file(function(callback, file) {
      if (callback) {
        callback(file);
      }
    }.bind(this, callback));
  }

  /**
   * Projde rekurzivně strom a nastaví souborům callback
   *
   * @param {DataTransferItem} itemEntry
   * @param {Function} callback
   * @returns {Void}
   */
  function readTree(itemEntry, callback) {
    if (itemEntry.isFile) {
      readFile(itemEntry, callback);
    }
    if (itemEntry.isDirectory) {
      var dirReader = itemEntry.createReader();
      dirReader.readEntries(function(entries) {
        var count = entries.length;
        while (count--) {
          readTree(entries[count], callback);
        }
      });
    }
  }

  /**
   * Informuje uživatele o souborech, které nejsou povoleny nahrávat
   *
   * @returns {Void}
   */
  function alertNotAllowed() {
    if (notAllowed.length) {
      alert(texts.MsgNotAllowed.replace('%files%', notAllowed.join(',\n')));
    }
    notAllowed = [];
  }

  /**
   * Projde předanou strukturu a nastaví callback pro nahrávání. Pokud není
   * struktura typu DataTransferItemList, pak neprochází adresářovou strukturu,
   * ale nahraje rovnou zkusí nahrát soubory na server.
   *
   * @param {DataTransferItemList | FileList} items
   * @returns {Void}
   */
  function traverseItems(items) {
    if ('undefined' !== typeof items) {
      var count = items.length, entry;
      for (var i = 0; i < count; i++) {
        entry = items[i];
        if (entry.getAsEntry) {  // Standard HTML5 API
          entry = entry.getAsEntry();
        } else if (entry.webkitGetAsEntry) {  // WebKit HTML5 API.
          entry = entry.webkitGetAsEntry();
        }
        if (entry.isFile) {
          readFile(entry, uploadFile);
        } else if (entry.isDirectory) {
          readTree(entry, uploadFile);
        } else if (entry instanceof File) {
          uploadFile(entry);
        }
      }
      alertNotAllowed();
    } else {
      fileList.innerHTML = Uploader.texts.MsgNoSupport;
    }
  }

  /**
   * Pokud existuje tlačítko (prvek formuláře) pro násobné nahrávání souborů,
   * přiřadíme událost na change
   */
  if (filesUpload) {
    // Událost pro formulářový prvek
    filesUpload.addEventListener('change', function() {
      fileCounts = {uploaded: 0, total: 0};
      traverseItems(this.files);
      fileList.scrollTop = fileList.height;
    }, false);
  }

  var
    dropHelper = document.getElementById('drop-helper')

    // Callback pro dragenter
    dE = function(e) {
      if (dropHelper) {
        dropHelper.className = 'over';
      } else {
        this.className = 'over';
      }
      e.preventDefault();
      e.stopPropagation();
    },

    // Callback pro dragover
    dO = function(e) {
      this.className = 'over';
      e.preventDefault();
      e.stopPropagation();
    },

    // Callback pro dragleave
    dL = function(e) {
      this.className = '';
      e.preventDefault();
      e.stopPropagation();
    },

    // Callback pro drop
    D = function(e) {
      this.className = '';

      if (e.dataTransfer) {
        if (e.dataTransfer.items) {
          traverseItems(e.dataTransfer.items);
        } else if (e.dataTransfer.files) {
          traverseItems(e.dataTransfer.files);
        }
      }

      fileList.scrollTop = fileList.height;
      e.preventDefault();
      e.stopPropagation();
    }
  ;

  dropArea.addEventListener('dragenter', dE, false);
  if (dropHelper) {
    dropHelper.addEventListener('dragover', dO, false);
    dropHelper.addEventListener('dragleave', dL, false);
    dropHelper.addEventListener('drop', D, false);
  } else {
    dropArea.addEventListener('dragover', dO, false);
    dropArea.addEventListener('dragleave', dL, false);
    dropArea.addEventListener('drop', D, false);
  }

  /**
   * Rozšíří objekt A o vlastnosti objektu B
   *
   * @param {Object} a
   * @param {Object} b
   * @returns {Object}
   */
  function extend(a, b) {
    var n;
    if (!a) {
      a = {};
    }
    for (n in b) {
      a[n] = b[n];
    }
    return a;
  }

  /**
   * Inicializace objektu Uploader
   *
   * @param {Object} obj
   * @returns {Void}
   */
  function init(obj) {
    Uploader = extend(Uploader, obj);
  }

  // Nastaví výchotí atributy Uploaderu
  extend(Uploader, {
    action: action,
    afterUploading: afterUploading,
    init: init,
    extensions: extensions,
    redirect: redirect,
    texts: texts,
    locale: defaultLocale,
    options: defaultOptions
  });

})();
